//*****************************************************************************
// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the user's sole risk and will be unsupported.
//
// Copyright (c) 2006-2007 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//*****************************************************************************
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor: Xilinx
// \   \   \/     Version: 1.1
//  \   \         Filename: bcd6.v
//  /   /         Timestamp: 24 August 2007
// /___/   /\     
// \   \  /  \
//  \___\/\___\
//
//
//	Device:  Virtex-4 and Virtex-5
//	Purpose: Fast 6 decade bcd up counter with synchronous enable and clear
//	
// Revision History:
//    Rev 1.0 - initial release
//    Rev 1.1 - updated the header comment information to be consistent with other files	
//*****************************************************************************
// 
`timescale 1 ps / 1ps

module  bcd6  (
   clk,       // clock
   rst,       // reset
   clr,       // clear to 0000000
   enablein,  // enable in
   bcdvec     // 6 decade output
);


input         clk, rst, clr, enablein;
output [23:0] bcdvec;

wire 		low;
wire 		high;
reg 	[23:0]	bcd;
reg	[5:0]	pd;
reg 		pipe;
reg 	[3:0]	partiala;
reg 	[3:0]	partialb;
reg 	[3:0]	partialc;
reg 	[3:0]	partiald;
reg 	[3:0]	partiale;
reg 	[3:0]	partialf;

assign low = 1'b0 ;
assign high = 1'b1 ;


always @ (posedge clk or posedge rst)
begin
if (rst) begin
	bcd <= 24'h0000 ;
	pipe <= 1'b0 ;
	partiala <= 4'h0 ;
	partialb <= 4'h0 ;
	partialc <= 4'h0 ;
	partiald <= 4'h0 ;
	partiale <= 4'h0 ;
	partialf <= 4'h0 ;
	pd <= 6'b00000 ;
end
else begin
	pipe <= enablein ;
	if (clr == 1'b1) begin
		bcd <= 24'h0000;
	end
	else if (pipe == 1'b1) begin	
		if (pd == 6'b111111) begin 							// 999999 ?
			bcd <= 24'h0000;
		end
		else if (pd[4:0] == 5'b11111) begin 						// x99999 ?
			bcd <= {partialf, 20'b00000000000000000000};
		end
		else if (pd[3:0] == 4'b1111) begin 						// xx9999 ?
			bcd <= {bcd[23:20], partiale, 16'b0000000000000000};
		end
		else if (pd[2:0] == 3'b111) begin 						// xxx999 ?
			bcd <= {bcd[23:16], partiald, 12'b000000000000};
		end
		else if (pd[1:0] == 2'b11) begin 						// xxxx99 ?
			bcd <= {bcd[23:12], partialc, 8'b00000000};
		end
		else if (pd[0] == 1'b1) begin 							// xxxxx9 ?
			bcd <= {bcd[23:8], partialb, 4'b0000};
		end
		else  begin
			bcd <= {bcd[23:4], partiala};
		end
	end
	if (enablein == 1'b1) begin
		if (bcd[23:20] == 4'b1001) begin pd[5] <= 1'b1; end else begin pd[5] <= 1'b0; end
		if (bcd[19:16] == 4'b1001) begin pd[4] <= 1'b1; end else begin partialf <= bcd[23:20] + 4'h1; pd[4] <= 1'b0; end
		if (bcd[15:12] == 4'b1001) begin pd[3] <= 1'b1; end else begin partiale <= bcd[19:16] + 4'h1; pd[3] <= 1'b0; end
		if (bcd[11: 8] == 4'b1001) begin pd[2] <= 1'b1; end else begin partiald <= bcd[15:12] + 4'h1; pd[2] <= 1'b0; end
		if (bcd[ 7: 4] == 4'b1001) begin pd[1] <= 1'b1; end else begin partialc <= bcd[11: 8] + 4'h1; pd[1] <= 1'b0; end
		if (bcd[ 3: 0] == 4'b1001) begin pd[0] <= 1'b1; end else begin partialb <= bcd[ 7: 4] + 4'h1; pd[0] <= 1'b0; end
		partiala <= bcd[3:0] + 4'h1 ;
	end
end
end

assign bcdvec = bcd;



endmodule