//*****************************************************************************
// DISCLAIMER OF LIABILITY
// 
// This text/file contains proprietary, confidential
// information of Xilinx, Inc., is distributed under license
// from Xilinx, Inc., and may be used, copied and/or
// disclosed only pursuant to the terms of a valid license
// agreement with Xilinx, Inc. Xilinx hereby grants you a 
// license to use this text/file solely for design, simulation, 
// implementation and creation of design files limited 
// to Xilinx devices or technologies. Use with non-Xilinx 
// devices or technologies is expressly prohibited and 
// immediately terminates your license unless covered by
// a separate agreement.
//
// Xilinx is providing this design, code, or information 
// "as-is" solely for use in developing programs and 
// solutions for Xilinx devices, with no obligation on the 
// part of Xilinx to provide support. By providing this design, 
// code, or information as one possible implementation of 
// this feature, application or standard, Xilinx is making no 
// representation that this implementation is free from any 
// claims of infringement. You are responsible for 
// obtaining any rights you may require for your implementation. 
// Xilinx expressly disclaims any warranty whatsoever with 
// respect to the adequacy of the implementation, including 
// but not limited to any warranties or representations that this
// implementation is free from claims of infringement, implied 
// warranties of merchantability or fitness for a particular 
// purpose.
//
// Xilinx products are not intended for use in life support
// appliances, devices, or systems. Use in such applications is
// expressly prohibited.
//
// Any modifications that are made to the Source Code are 
// done at the user's sole risk and will be unsupported.
//
// Copyright (c) 2006-2007 Xilinx, Inc. All rights reserved.
//
// This copyright and support notice must be retained as part 
// of this text at all times. 
//*****************************************************************************
//   ____  ____
//  /   /\/   /
// /___/  \  /    Vendor: Xilinx
// \   \   \/     Version: 1.1
//  \   \         Filename: test_bench.v
//  /   /         Timestamp: 24 August 2007
// /___/   /\     
// \   \  /  \
//  \___\/\___\
//
//
//	Device:  Virtex-4 and Virtex-5
//	Purpose: Synthesizable Testbench for Hardware Verification
// Revision History:
//   Rev 1.0 - initial release
//   Rev 1.1 - Remove some use of RAF_ALMOST_FULL/WDF_ALMOST_FULL to improve timing
//*****************************************************************************

`timescale 1ns/100ps


module test_bench (
   CLK,
   RESET,
   RESET_90,
   WDF_ALMOST_FULL,
   WDF_WR_COUNT,
   RAF_ALMOST_FULL,
   RAF_WR_COUNT,
   rlafEmpty,
   BURST_LENGTH,
   READ_DATA_FIFO_OUT,
   READ_DATA_VALID,
   Init_Done,
   //Init_Done_270,
   clk90,
   rlRdfEmpty,
   rlWdfEmpty,

   APADDR,
   APVALID,
   APP_WDF_DATA,
   APP_WDF_WREN,

   DISPLAY,
   error_count_BCD,
  
   APCONFA,
   APCONFWRD,
   APCONFRD,
   APCONFRDD,
   APCONFWR,
   APWRITEDM,
   apRdfRdEn,
   issueMRS,
   issueCalibration,
   
   PASS_FAIL,
   
   // chipscope debug ports
   cs_io
);

// parameter definitions
parameter RL_DQ_WIDTH     = 36;
parameter RL_MRS_BURST_LENGTH    = 2'b00;  // 2'b00: BL2;  2'b01: BL4;  2'b10: BL8
// end of parameter definitions

// private parameters -- do not change
parameter idle          = 3'b000;
parameter start         = 3'b001;
parameter write         = 3'b010;
parameter read          = 3'b011;
parameter mrs           = 3'b100;
parameter restart_state = 3'b101;

   input         CLK;
   input         RESET;
   input         RESET_90;
   input         WDF_ALMOST_FULL;
   //input [4:0]   WDF_WR_COUNT;
   input [12:0]   WDF_WR_COUNT;
   input         RAF_ALMOST_FULL;
   //input [3:0]   RAF_WR_COUNT;
   input [12:0]   RAF_WR_COUNT;
   input         rlafEmpty;
   //input [3:0]   BURST_LENGTH;
   input [1:0]   BURST_LENGTH;
   input [2*RL_DQ_WIDTH-1:0]  READ_DATA_FIFO_OUT;
   input                      READ_DATA_VALID;
   input 	 Init_Done;
   //input 	 Init_Done_270;
   input 	 clk90; 
   input         rlRdfEmpty;
   input 	 rlWdfEmpty;
   
   
   output [25:0] APADDR;
   output        APVALID;
   output [2*RL_DQ_WIDTH-1:0] APP_WDF_DATA;
   output        APP_WDF_WREN;

   output [6:0]  DISPLAY;
   output [23:0] error_count_BCD;

   output [3:0]  APCONFA;
   output [7:0]  APCONFWRD;
   output 	 APCONFRD;
   input  [7:0]  APCONFRDD;
   output        APCONFWR;
   output [3:0]  APWRITEDM;
   output        apRdfRdEn;
   output        issueMRS;
   output        issueCalibration;
   
   output [2:0] PASS_FAIL;
   
   inout [1023:0] cs_io;

   reg [2:0]     state;
   reg [2:0]     burst_count;
   reg [2:0]     temp_count;
   reg           write_data_en;
   reg           write_addr_en;
   reg           read_addr_en;
   reg [3:0] 	 wr_count;
   reg [3:0] 	 rd_count;
   reg 		 end_of_write;
   reg 		 ready_for_rd;
   reg           restart_state_en;
   reg           issueMRS;
   reg [5:0]     Trc_cnt;
   
   reg [7:0]     APCONFWRD;
   reg [3:0]     APCONFA;
   reg           APCONFWR;
   reg           rlWdfEmpty_clk0;
   reg           rlWdfEmpty_r;
   
   reg [1:0]   burst_length_r;

   wire [2*RL_DQ_WIDTH-1:0]   app_cmp_data;
   wire [2*RL_DQ_WIDTH-1:0]   app_cmp_data_tmp;
   wire [25:0]   APP_WAF_ADDR;
   wire [25:0]   APP_RAF_ADDR;
   wire 	 APP_WAF_WREN;
   wire 	 APP_RAF_WREN;
   wire [2*RL_DQ_WIDTH-1:0]   APP_WDF_DATA_tmp;
   
   wire enable_error_counter;

   reg apRdfRdEn_r0;
   reg apRdfRdEn;
   
   reg APP_WAF_WREN_r;
   reg APP_RAF_WREN_r;

   assign APADDR          = APP_WAF_ADDR | APP_RAF_ADDR;
   assign APVALID         = APP_WAF_WREN_r | ((burst_length_r == 2'b01) ? APP_RAF_WREN_r : APP_RAF_WREN);
   assign APCONFRD        = 1'b0;
   assign APWRITEDM[3:0]  = 4'd0;
   assign issueCalibration= 1'b0;
   
   always @ (posedge CLK)
   begin
    if (RESET) begin
      APP_WAF_WREN_r <= 1'b0;
      APP_RAF_WREN_r <= 1'b0;
    end else begin
      if ( BURST_LENGTH == 2'b01 ) begin
        APP_WAF_WREN_r <= APP_WAF_WREN;
      end else begin
        APP_WAF_WREN_r <= write_addr_en;
      end

      APP_RAF_WREN_r <= APP_RAF_WREN;
    end
   end
   
always @ (posedge CLK) burst_length_r <= BURST_LENGTH;   

// Txfer rlWdfEmpty to clk0 (clkGlob) domain
always @ (posedge CLK) rlWdfEmpty_r    <= rlWdfEmpty;
always @ (posedge CLK) rlWdfEmpty_clk0 <= rlWdfEmpty_r;
  
  // Read from read data fifo if it is not empty.
  // register apRdfRdEn for better timing
  always @ (posedge CLK) apRdfRdEn <= ~rlRdfEmpty;
                
/* State Machine               
   -------------
   1- After power-up, wait until Initialization of the RLDRAM is complete
   --> (Burst Length has been set at compile time to either 2 or 4 in rld_conf.v)
   2- Make 8 writes followed by 8 reads
   3- Make another 8 writes followed by 8 reads
   4- Drive DISPLAY: Pass (P), Error(E)
   5- Go to 2nd step after Trc_cnt elapses
*/

// State Machine for generating write address and data enables and read address enable
  always @ (posedge CLK)
  begin
     if (RESET == 1'b1)   // State Machine in IDLE state
     begin
        write_data_en   <= 1'b0;
        write_addr_en   <= 1'b0;
        read_addr_en    <= 1'b0;
        wr_count        <= 4'b0000;
        rd_count        <= 4'b0000;
        end_of_write    <= 1'b0;
        ready_for_rd    <= 1'b0;
        restart_state_en<= 1'b0;
        APCONFWR        <= 1'b0;
        issueMRS        <= 1'b0;
        Trc_cnt         <= 6'b000000;
        state[2:0]      <= idle;
     end else begin
     
     case (state[2:0])
	
        3'b000:   // idle 
        begin 
          write_data_en <= 1'b0;
          write_addr_en <= 1'b0;
          read_addr_en  <= 1'b0;
          restart_state_en<= 1'b0;
          APCONFWR        <= 1'b0;
          issueMRS        <= 1'b0;
          Trc_cnt         <= 6'b000000;
          
          // power-up - wait until init_done
          if (Init_Done == 1'b1) begin
            state[2:0] <= start;
          end
        end
	 
         
        3'b001:   // start 
        begin 
           write_data_en   <= 1'b0;
           write_addr_en   <= 1'b0;
           read_addr_en    <= 1'b0;
	   
        // do 8 writes
        if (  wr_count < 4'b1000 )  begin
          state[2:0]       <= write;
          if (burst_length_r == 2'b01) begin
            burst_count <= 3'b010;
            temp_count  <= 3'b001;
          end else begin
            burst_count <= 3'b001;
            temp_count  <= 3'b000;
          end
        end
	   
	   // do 8 reads when 8 writes are complete
        else if (  (ready_for_rd == 1'b1) && rd_count < 4'b1000) begin
              state[2:0]   <= read;
        end
	   
	   // wait after 8 writes for writes to complete (rlWdfEmpty flag) before reads can be started
        else if (wr_count == 4'b1000 && rlWdfEmpty_clk0 == 1'b1 && rd_count == 4'b0000) begin
          ready_for_rd <= 1'b1;
          wr_count     <= wr_count + 1'b1;	         
          state[2:0]   <= start;
        end
	   
        //after 8 reads are done, do another 8 writes followed by 8 reads, then go to mrs state
        else if (rd_count == 4'b1000) begin
          if (restart_state_en == 1'b1) begin
                 state[2:0]   <= mrs;
          end else begin
            if (rlafEmpty == 1'b1) begin
              wr_count        <= 4'b0000;
              ready_for_rd    <= 1'b0;
              rd_count        <= 4'b0000;
              restart_state_en<= 1'b1;
              state[2:0]      <= start;
            end	 
          end
        end	      
      end 

	// write  
        3'b010: 
        begin  
           //if ( WDF_ALMOST_FULL == 1'b0 ) begin
              if (burst_count != 3'b000) begin
                 read_addr_en  <= 1'b0;
                 write_data_en <= 1'b1;
                 burst_count   <= burst_count - 1'b1;
                 wr_count      <= wr_count;
                 state[2:0]    <= write;              
              end else if (wr_count < 4'b0111) begin
                 read_addr_en  <= 1'b0;
                 write_data_en <= 1'b1;
                 write_addr_en <= 1'b0;
                 burst_count   <= temp_count;
                 wr_count      <= wr_count + 1'b1;
                 state[2:0]    <= write;
              end else begin
                 read_addr_en  <= 1'b0;
                 write_data_en <= 1'b0;
                 write_addr_en <= 1'b0;
                 wr_count      <= wr_count + 1'b1;
                 state[2:0]    <= start;  
              end 
              
              if ( (burst_length_r == 2'b00 && burst_count == 3'b000) || 
                   (burst_length_r == 2'b01 && burst_count == 3'b001) ) begin
                 write_addr_en <= 1'b1;
              end else begin
                 write_addr_en <= 1'b0;
              end         
           //end else begin
           //   read_addr_en  <= 1'b0;
           //   write_data_en <= 1'b0;
           //   write_addr_en <= 1'b0;
           //   burst_count   <= burst_count;
           //   wr_count      <= wr_count;
           //   state[2:0]    <= write;  // Control will stay in the same state until wdfifo is not full
           //end
        end

	
        // read   
        3'b011:
        begin
          write_data_en <= 1'b0;
          write_addr_en <= 1'b0;
          if ( RAF_ALMOST_FULL == 1'b0) begin
            read_addr_en  <= 1'b1;
            rd_count      <= rd_count + 1'b1;
          
            if (rd_count < 4'b0111)
              state[2:0]    <= read;
            else
              state[2:0]    <= start;
          end else begin
            read_addr_en  <= 1'b0;
            rd_count      <= rd_count;
            state[2:0]    <= start;
          end
        end

	// mrs  
	3'b100:
        begin
	   /* This functionality is taken out, but Mode Register bits can be programmed if desired as shown below
           if (restart_edge == 1'b1) // wait for restart, then program burst length
           begin
              APCONFA     <= 4'b0000;
	      APCONFWR    <= 1'b1;
   	      APCONFWRD[7:0] <= {4'b0000, BURST_LENGTH, 3'b010}; // except BURST LENGTH make the other bits equal to their default values
	      issueMRS    <= 1'b1;
	      Trc_cnt     <= 6'b000000;
              state[2:0]  <= restart_state;
           end   
	   */ 
	   Trc_cnt     <= 6'b000000;
	   if (rlafEmpty == 1'b1) begin
	      state[2:0]  <= restart_state;
	   end
        end

	// restart_state
	3'b101:
     begin // go to start after Trc_cnt elapses
      wr_count        <= 4'b0000;
      ready_for_rd    <= 1'b0;
   	  rd_count        <= 4'b0000;
      restart_state_en<= 1'b0;
      APCONFWR        <= 1'b0;
      issueMRS        <= 1'b0;
      if (Trc_cnt == 6'b111111) begin
        state[2:0]    <= start;
      end else begin
        Trc_cnt       <= Trc_cnt + 1'b1;
        state[2:0]    <= restart_state;
      end  
     end
		  
   default:
    begin
      write_data_en   <= 1'b0;          
      write_addr_en   <= 1'b0;          
      read_addr_en    <= 1'b0;
      wr_count        <= 4'b0000;
     	rd_count        <= 4'b0000;
   	  end_of_write    <= 1'b0;
      ready_for_rd    <= 1'b0;
      restart_state_en<= 1'b0;
      APCONFWR        <= 1'b0;
      issueMRS        <= 1'b0;
      Trc_cnt         <= 6'b000000;
      state[2:0]      <= idle;   
    end
   endcase
  end  
end 



   cmp_rd_data #(
      .RL_DQ_WIDTH (RL_DQ_WIDTH)
   ) 
   cmp_rd_data0  (
      .CLK                ( CLK ),                      // data read from memory is in clkGlob domain
      .RESET              ( RESET ),
      .READ_DATA_VALID    ( READ_DATA_VALID & Init_Done ),
      .READ_DATA_FIFO_OUT ( READ_DATA_FIFO_OUT[2*RL_DQ_WIDTH-1:0] ),
      .APP_COMPARE_DATA   ( app_cmp_data[2*RL_DQ_WIDTH-1:0] ), 
      .DISPLAY            ( DISPLAY ),
      .enable_error_counter  ( enable_error_counter ),
      
      .PASS_FAIL          ( PASS_FAIL ),
      
      .cs_io              ( cs_io )
   );


   backend_rom #(
      .RL_DQ_WIDTH (RL_DQ_WIDTH)
   ) backend_rom0  (
      .clk              ( CLK ),
      .rst              ( RESET ),
      .rst90            ( RESET_90 ),
      .clk90            ( clk90 ),
      .bkend_data_en    ( write_data_en ),
      .bkend_wraddr_en  ( write_addr_en ),
      .bkend_rdaddr_en  ( read_addr_en ),
      .apRdfRdEn        ( READ_DATA_VALID ),
      .burst_length     ( burst_length_r ),

      .app_Waf_addr     ( APP_WAF_ADDR ), 
      .app_Waf_WrEn     ( APP_WAF_WREN ),
      .app_Wdf_data     ( APP_WDF_DATA_tmp ),
      .app_compare_data ( app_cmp_data_tmp ),
      .app_Wdf_WrEn     ( APP_WDF_WREN ),
      .app_Raf_addr     ( APP_RAF_ADDR ),
      .app_Raf_WrEn     ( APP_RAF_WREN ),
      
      .cs_io              ( cs_io )
   );

assign APP_WDF_DATA = APP_WDF_DATA_tmp;
assign app_cmp_data = app_cmp_data_tmp;


// BCD error counter
bcd6  RLD2_bcd_error_counter  (
   .clk      ( CLK                  ),   // clock
   .rst      ( RESET                ),   // reset
   .clr      ( 1'b0 ),                   // clear to 0000000
   .enablein ( enable_error_counter ),   // enable in
   .bcdvec   ( error_count_BCD      )    // 6 decade output
);


endmodule