--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: bcd6.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: Fast 6 decade bcd up counter with synchronous enable and clear
--		
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY bcd6 IS
   PORT (
      clk        : IN  std_logic;                        -- clock
      rst        : IN  std_logic;                        -- reset
      clr        : IN  std_logic;                        -- clear to 0000000
      enablein   : IN  std_logic;                        -- enable in
      bcdvec     : OUT std_logic_vector(23 DOWNTO 0)     -- 6 decade output
   );
END ENTITY bcd6;



ARCHITECTURE rtl OF bcd6 IS

   SIGNAL low           :  std_logic;
   SIGNAL high          :  std_logic;
   SIGNAL bcd           :  std_logic_vector(23 DOWNTO 0);
   SIGNAL pd            :  std_logic_vector(5 DOWNTO 0);
   SIGNAL pipe          :  std_logic;
   SIGNAL partiala      :  std_logic_vector(3 DOWNTO 0);
   SIGNAL partialb      :  std_logic_vector(3 DOWNTO 0);
   SIGNAL partialc      :  std_logic_vector(3 DOWNTO 0);
   SIGNAL partiald      :  std_logic_vector(3 DOWNTO 0);
   SIGNAL partiale      :  std_logic_vector(3 DOWNTO 0);
   SIGNAL partialf      :  std_logic_vector(3 DOWNTO 0);


BEGIN
   low  <= '0';
   high <= '1';

   PROCESS ( clk, rst )
   BEGIN
      IF ( rst = '1' ) THEN
         bcd      <= ( others => '0');
         pipe     <= '0';
         partiala <= "0000";
         partialb <= "0000";
         partialc <= "0000";
         partiald <= "0000";
         partiale <= "0000";
         partialf <= "0000";
         pd       <= "000000";

      ELSIF RISING_EDGE( CLK ) THEN
         pipe <= enablein;

         IF ( clr = '1' ) THEN
            bcd <= "000000000000000000000000";
         ELSE
            IF ( pipe = '1' ) THEN
               IF ( pd = "111111" ) THEN                                           -- 999999 ?
                  bcd <= "000000000000000000000000";
               ELSE
                  IF ( pd(4 DOWNTO 0) = "11111" ) THEN                             -- x99999 ?
                     bcd <= partialf & "00000000000000000000";
                  ELSE
                     IF ( pd(3 DOWNTO 0) = "1111" ) THEN                           -- xx9999 ?
                        bcd <= bcd(23 DOWNTO 20) & partiale & "0000000000000000";
                     ELSE
                        IF ( pd(2 DOWNTO 0) = "111" ) THEN                         -- xxx999 ?
                           bcd <= bcd(23 DOWNTO 16) & partiald & "000000000000";
                        ELSE
                           IF ( pd(1 DOWNTO 0) = "11" ) THEN                       -- xxxx99 ?
                              bcd <= bcd(23 DOWNTO 12) & partialc & "00000000";
                           ELSE
                              IF ( pd(0) = '1' ) THEN                              -- xxxxx9 ?
                                 bcd <= bcd(23 DOWNTO 8) & partialb & "0000";
                              ELSE
                                 bcd <= bcd(23 DOWNTO 4) & partiala;
                              END IF;
                           END IF;
                        END IF;
                     END IF;
                  END IF;
               END IF;
            END IF;
         END IF;

         IF ( enablein = '1' ) THEN
            IF ( bcd(23 DOWNTO 20) = "1001" ) THEN
               pd(5) <= '1';
            ELSE
               pd(5) <= '0';
            END IF;

            IF ( bcd(19 DOWNTO 16) = "1001" ) THEN
               pd(4) <= '1';
            ELSE
               partialf <= std_logic_vector( unsigned( bcd(23 DOWNTO 20) ) + "0001" );
               pd(4) <= '0';
            END IF;

            IF ( bcd(15 DOWNTO 12) = "1001" ) THEN
               pd(3) <= '1';
            ELSE
               partiale <= std_logic_vector( unsigned( bcd(19 DOWNTO 16) ) + "0001" );
               pd(3) <= '0';
            END IF;

            IF ( bcd(11 DOWNTO 8) = "1001" ) THEN
               pd(2) <= '1';
            ELSE
               partiald <= std_logic_vector( unsigned( bcd(15 DOWNTO 12) ) + "0001" );
               pd(2) <= '0';
            END IF;

            IF ( bcd(7 DOWNTO 4) = "1001" ) THEN
               pd(1) <= '1';
            ELSE
               partialc <= std_logic_vector( unsigned( bcd(11 DOWNTO 8) ) + "0001" );
               pd(1) <= '0';
            END IF;

            IF ( bcd(3 DOWNTO 0) = "1001" ) THEN
               pd(0) <= '1';
            ELSE
               partialb <= std_logic_vector( unsigned( bcd(7 DOWNTO 4) ) + "0001" );
               pd(0) <= '0';
            END IF;

            partiala <= std_logic_vector( unsigned( bcd(3 DOWNTO 0) ) + "0001" );
         END IF;

      END IF;
   END PROCESS;

   bcdvec <= bcd;

END ARCHITECTURE rtl;
