--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: cmp_rd_data.vhd
--  /   /         Timestamp: 14 March 2008
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This module compares the read data with expected data value.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Added fix to catch error at the end of a burst
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY cmp_rd_data IS
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_BYTE_WIDTH                 : integer := 9     -- the width of a "byte" on the memory device (x8 or x9)
   );
   PORT (
      CLK                     : IN  std_logic;
      RESET                   : IN  std_logic;
      READ_DATA_VALID         : IN  std_logic;
      READ_DATA_FIFO_OUT      : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);

      APP_COMPARE_DATA        : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
      DISPLAY                 : OUT std_logic_vector(6 DOWNTO 0);
      enable_error_counter    : OUT std_logic;

      PASS_FAIL               : OUT std_logic_vector(2 DOWNTO 0);

      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY cmp_rd_data;



ARCHITECTURE rtl OF cmp_rd_data IS


   function AND_REDUCE(ARG: in STD_LOGIC_VECTOR) return std_logic is
      variable result: STD_LOGIC := '1';
   begin
      for i in ARG'range loop
         result := result AND ARG(i);
      end loop;
      return result;
   end function AND_REDUCE;


   CONSTANT NUM_OF_BYTES            : integer := RL_DQ_WIDTH/DEV_BYTE_WIDTH;

   -- meaningful codes for partial and full comparisons
   CONSTANT RESET_CODE    : std_logic_vector(2 DOWNTO 0) := "001";
   CONSTANT PASS_CODE     : std_logic_vector(2 DOWNTO 0) := "010";
   CONSTANT ERROR_CODE    : std_logic_vector(2 DOWNTO 0) := "100";
   CONSTANT INVALID_CODE  : std_logic_vector(2 DOWNTO 0) := "111";


   SIGNAL PASS_FAIL_msb1           : std_logic_vector(2 DOWNTO 0);
   SIGNAL PASS_FAIL_lsb1           : std_logic_vector(2 DOWNTO 0);

   SIGNAL valid_d2                 : std_logic;
   SIGNAL valid_d1                 : std_logic;
   SIGNAL valid_cmp                : std_logic;
   SIGNAL valid_cmp_r              : std_logic;
   SIGNAL read_data                : std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);

   SIGNAL display_buf              : std_logic_vector(6 DOWNTO 0);

   SIGNAL msb_cmp                  : std_logic_vector(NUM_OF_BYTES-1 DOWNTO 0);
   SIGNAL lsb_cmp                  : std_logic_vector(NUM_OF_BYTES-1 DOWNTO 0);
   SIGNAL msb_compare              : std_logic;
   SIGNAL lsb_compare              : std_logic;

   SIGNAL msb_error                : std_logic;
   SIGNAL lsb_error                : std_logic;

   SIGNAL read_data_MSB            : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL APP_COMPARE_DATA_MSB     : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

   --for chipscope
   --chipscope read data from output of user interface FIFOs
   SIGNAL fifo_data_rise           : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL fifo_data_fall           : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   --end of whats for chipscope

   SIGNAL enable_error_counter_sig : std_logic;
   SIGNAL PASS_FAIL_sig            : std_logic_vector(2 DOWNTO 0);


BEGIN
   cs_io <= (OTHERS => 'Z');
   enable_error_counter <= enable_error_counter_sig;
   PASS_FAIL            <= PASS_FAIL_sig;

--   fifo_data_rise(RL_DQ_WIDTH-1 DOWNTO 0) <= READ_DATA_FIFO_OUT(RL_DQ_WIDTH-1 DOWNTO 0);
--   fifo_data_fall(RL_DQ_WIDTH-1 DOWNTO 0) <= READ_DATA_FIFO_OUT((2*RL_DQ_WIDTH)-1 DOWNTO RL_DQ_WIDTH);
   fifo_data_rise(RL_DQ_WIDTH-1 DOWNTO 0) <= read_data((2*RL_DQ_WIDTH)-1 DOWNTO RL_DQ_WIDTH);
   fifo_data_fall(RL_DQ_WIDTH-1 DOWNTO 0) <= read_data(RL_DQ_WIDTH-1 DOWNTO 0);

   --for chipscope
   cs_io(92 DOWNTO 91)  <= msb_error & lsb_error;
   cs_io(90)            <= valid_d1;
   cs_io(80 DOWNTO 72)  <= APP_COMPARE_DATA(44 DOWNTO 36);
   cs_io(89 DOWNTO 81)  <= APP_COMPARE_DATA(8 DOWNTO 0);

--   cs_io(8 DOWNTO 0)    <= fifo_data_rise(8 DOWNTO 0);     -- 1st byte
--   cs_io(17 DOWNTO 9)   <= fifo_data_fall(8 DOWNTO 0);

--   cs_io(26 DOWNTO 18)  <= fifo_data_rise(17 DOWNTO 9);    -- 2nd byte
--   cs_io(35 DOWNTO 27)  <= fifo_data_fall(17 DOWNTO 9);

--   cs_io(44 DOWNTO 36)  <= fifo_data_rise(26 DOWNTO 18);   -- 3rd byte
--   cs_io(53 DOWNTO 45)  <= fifo_data_fall(26 DOWNTO 18);

--   cs_io(62 DOWNTO 54)  <= fifo_data_rise(35 DOWNTO 27);   -- 4th byte
--   cs_io(71 DOWNTO 63)  <= fifo_data_fall(35 DOWNTO 27);
   --end of whats for chipscope


   -- -----------------------------------------------------------------------------
   -- create flag for when data should be compared   
   -- -----------------------------------------------------------------------------
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
--         IF ( RESET = '1' ) THEN
--            valid_d1  <= '0';
--            valid_d2  <= '0';
--            valid_cmp <= '0';
--         ELSE
            valid_d1  <= READ_DATA_VALID;
            valid_d2  <= valid_d1;
            valid_cmp <= valid_d2;
            valid_cmp_r <= valid_cmp;
--         END IF;
      END IF;
   END PROCESS;


   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         read_data <= READ_DATA_FIFO_OUT;
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- Compare read data with data generated for comparison.
   -- Set PASS_FAIL flag accordingly.
   -- Reset PASS_FAIL flag when restart occurs    
   -- -----------------------------------------------------------------------------

   -- -------------------
   -- MSB comparison
   -- -------------------
 
   read_data_MSB        <= read_data(2*RL_DQ_WIDTH-1 DOWNTO RL_DQ_WIDTH);
   APP_COMPARE_DATA_MSB <= APP_COMPARE_DATA(2*RL_DQ_WIDTH-1 DOWNTO RL_DQ_WIDTH);

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         FOR K IN 0 TO (NUM_OF_BYTES-1) LOOP
            msb_cmp(K) <= AND_REDUCE( NOT( read_data_MSB((K+1)*DEV_BYTE_WIDTH-1 DOWNTO K*DEV_BYTE_WIDTH) XOR APP_COMPARE_DATA_MSB((K+1)*DEV_BYTE_WIDTH-1 DOWNTO K*DEV_BYTE_WIDTH) ) );
         END LOOP;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         msb_compare <= AND_REDUCE(msb_cmp);
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF ( RESET = '1' ) THEN
            msb_error <= '0';
         ELSE
            IF ( valid_cmp = '1' ) THEN
               msb_error <= NOT (msb_compare);
            ELSE
               msb_error <= '0';
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF ( RESET = '1' ) THEN
            PASS_FAIL_msb1 <= RESET_CODE;   -- set "reset" flag
         ELSE
            --IF ( valid_cmp = '1' ) THEN
            IF ( valid_cmp_r = '1' ) THEN -- changed to catch last burst error
               IF ( (msb_error = '0') AND (PASS_FAIL_msb1 /= ERROR_CODE) ) THEN
                  PASS_FAIL_msb1 <= PASS_CODE;   -- set "pass" flag
               ELSE
                  PASS_FAIL_msb1 <= ERROR_CODE;   -- flag an error
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;


   -- -------------------
   -- LSB comparison
   -- -------------------
 
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         FOR K IN 0 TO (NUM_OF_BYTES-1) LOOP
            lsb_cmp(K) <= AND_REDUCE( NOT( read_data((K+1)*DEV_BYTE_WIDTH-1 DOWNTO K*DEV_BYTE_WIDTH) XOR APP_COMPARE_DATA((K+1)*DEV_BYTE_WIDTH-1 DOWNTO K*DEV_BYTE_WIDTH) ) );
         END LOOP;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         lsb_compare <= AND_REDUCE(lsb_cmp);
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF ( RESET = '1' ) THEN
            lsb_error <= '0';
         ELSE
            IF ( valid_cmp = '1' ) THEN
               lsb_error <= NOT (lsb_compare);
            ELSE
               lsb_error <= '0';
            END IF;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF ( RESET = '1' ) THEN
            PASS_FAIL_lsb1 <= RESET_CODE;   -- set "reset" flag
         ELSE
            --IF (valid_cmp = '1') THEN
            IF (valid_cmp_r = '1') THEN   -- changed to catch last burst error
               IF ( (lsb_error = '0') AND (PASS_FAIL_lsb1 /= ERROR_CODE) ) THEN
                  PASS_FAIL_lsb1 <= PASS_CODE;   -- set "pass" flag
               ELSE
                  PASS_FAIL_lsb1 <= ERROR_CODE;   -- flag an error
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;


   -- --------------------------------------
   -- enable signal for error counter
   -- --------------------------------------
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         enable_error_counter_sig <= msb_error OR lsb_error;
      END IF;
   END PROCESS;


   -- --------------------------------------
   -- Join MSB and LSB comparisons
   -- --------------------------------------
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF ( RESET = '1' ) THEN
            PASS_FAIL_sig <= RESET_CODE;
         ELSE
            IF ( PASS_FAIL_msb1 = ERROR_CODE OR PASS_FAIL_lsb1 = ERROR_CODE ) THEN          -- check if one flag shows "error"
               PASS_FAIL_sig <= ERROR_CODE;
            ELSE
               IF ( PASS_FAIL_msb1 = PASS_CODE AND PASS_FAIL_lsb1 = PASS_CODE ) THEN        -- make sure all flags "pass"
                  PASS_FAIL_sig <= PASS_CODE;
               ELSE
                  IF ( PASS_FAIL_msb1 = RESET_CODE AND PASS_FAIL_lsb1 = RESET_CODE ) THEN   -- make sure all flags "reset"
                     PASS_FAIL_sig <= RESET_CODE;
                  END IF;
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- generate DISPLAY signals based on comparison results
   -- -----------------------------------------------------------------------------
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF ( PASS_FAIL_sig = ERROR_CODE ) THEN
            display_buf <= "1111001";         -- E  0x79 or 0x06(inverted)
         ELSE
            IF ( PASS_FAIL_sig = PASS_CODE ) THEN
               display_buf <= "1110011";      -- P  0x73 or 0x0C(inverted)
            ELSE
               IF ( PASS_FAIL_sig = RESET_CODE ) THEN
                  display_buf <= "0111111";   -- 0  0x3F or 0x40(inverted)	 
               END IF;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   DISPLAY <= NOT display_buf;


   -- -----------------------------------------------------------------------------
   -- Verification stage (simulation only)
   --    synthesis tool ignores assertions but simulator check them and report messages
   -- -----------------------------------------------------------------------------
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF (RESET = '0') THEN
            assert enable_error_counter_sig = '0' report ": ************* Data Comparison Error *************" severity error;
         END IF;
      END IF;
   END PROCESS;


END ARCHITECTURE rtl;
