--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_clk_module.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  virtex-4 and Virtex-5
--	Purpose: This module instantiates DCM_BASE primitive for the memory
--		interface and the 200 MHz clock for the IDELAYCTRL module.
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;


ENTITY rld_clk_module IS
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;    --  number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_AD_WIDTH            : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;    -- bank address width of the memory device
      DUPLICATE_CONTROLS      : boolean := FALSE;   -- Duplicate the ports for A, BA, WE# and REF#
      --
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      sysClk_p   : IN  std_logic;
      sysClk_n   : IN  std_logic;
      refClk_p   : IN  std_logic;
      refClk_n   : IN  std_logic;
      clk33      : IN  std_logic;
      sysReset   : IN  std_logic;
      clk        : OUT std_logic;
      clk90      : OUT std_logic;
      sysClk     : OUT std_logic;
      refClk     : OUT std_logic;
      locked     : OUT std_logic
   );
END ENTITY rld_clk_module;



ARCHITECTURE rtl OF rld_clk_module IS


   SIGNAL clk0_bufg_in    : std_logic;
   SIGNAL clk90_bufg_in   : std_logic;
   SIGNAL clk0_bufg_out   : std_logic;
   SIGNAL clk90_bufg_out  : std_logic;
   SIGNAL clkfx_bufg_in   : std_logic;
   SIGNAL clk200          : std_logic;
   --added for the Demo
   SIGNAL clkin_ibufg     : std_logic;
   SIGNAL clk0_pll        : std_logic;
   SIGNAL clk0_pll_bufg   : std_logic;
   SIGNAL locked_pll_i    : std_logic;

   -- internal signals
   SIGNAL sysClk_ibufgds  : std_logic;


BEGIN
   sysClk <= sysClk_ibufgds;

   -- -----------------------------------------------------------------------------
   -- sysClk differential input buffer
   -- -----------------------------------------------------------------------------
   sysClk0 : IBUFGDS
   PORT MAP (
      I  => sysClk_p,        -- differential input pins
      IB => sysClk_n,
      O  => sysClk_ibufgds   -- system clock, differential clock output
   );

   refClk0 : IBUFGDS
   PORT MAP (
      I  => refClk_p,   -- differential input pins
      IB => refClk_n,
      O  => clk200      -- reference clock, differential clock output
   );


   -- -----------------------------------------------------------------------------
   -- slow clock for Demo
   -- -----------------------------------------------------------------------------  
   --   IBUFG U_IBUFG_CLKIN
   --    IBUFG U_IBUFG_CLKIN
   --     (
   --      .O(clkin_ibufg),
   --      .I(clk33)
   --      );   
   -- PLL_BASE #
   --     (
   --      .BANDWIDTH("LOW"),                    // don't care about in-out skew,
   --      .CLKFBOUT_MULT(20),
   --      .CLKFBOUT_PHASE(0.0),
   --      .CLKIN_PERIOD(30.30),
   --      .CLKOUT0_DIVIDE(2),  //3
   --      .CLKOUT0_DUTY_CYCLE(0.5),
   --      .CLKOUT0_PHASE(0.0),
   --      .COMPENSATION("SYSTEM_SYNCHRONOUS"),  // should really set to INTERNAL,
   --                                            // but it's not an option - tool
   --                                            // s/w should infer internal fdbk
   --      .DIVCLK_DIVIDE(1)
   --      )
   --     U_PLL_BASE 
   --       (
   --        .CLKFBOUT(clkfb_pll),
   --        .CLKOUT0(clk0_pll),
   --        .CLKOUT1(),
   --        .CLKOUT2(),
   --        .CLKOUT3(),
   --        .CLKOUT4(),
   --        .CLKOUT5(),
   --        .LOCKED(locked_pll_i),
   --        .CLKFBIN(clkfb_pll),
   --        .CLKIN(clkin_ibufg),
   --        .RST(sysReset) //rst
   --        );
   -- BUFG U_BUFG_CLK0_PLL 
   --     (
   --      .I(clk0_pll),
   --      .O(clk0_pll_bufg)
   --      );
   --   //assign locked_pll = locked_pll_i;
   --   //***************************************************************************
   --   // DCM (fed from PLL CLKOUT0 output)
   --   //***************************************************************************
   --   // take DCM out of reset once PLL has locked (assume PLL doesn't assert
   --   // locked for at least 3 CLKOUT0 clock cycles)
   --   assign rst_dcm = ~locked_pll_i; 
   -- ----------------------------------------------------------------------------- 


   -- -----------------------------------------------------------------------------
   -- instantiate one DCM
   -- -----------------------------------------------------------------------------
   DCM_BASE0 : DCM_BASE
   GENERIC MAP (
      DLL_FREQUENCY_MODE    => "HIGH",
      DUTY_CYCLE_CORRECTION => TRUE,
      CLKDV_DIVIDE          => 16.0,
      FACTORY_JF            => X"F0F0",
      DFS_FREQUENCY_MODE    => "LOW",
      CLKFX_MULTIPLY        => 2,
      CLKFX_DIVIDE          => 8
   )
   PORT MAP (
      CLK0      => clk0_bufg_in,
      CLK180    => open,
      CLK270    => open,
      CLK2X     => open,
      CLK2X180  => open,
      CLK90     => clk90_bufg_in,
      CLKDV     => open,
      CLKFX     => open,
      CLKFX180  => open,
      LOCKED    => locked,
      CLKFB     => clk0_bufg_out,
      CLKIN     => sysClk_ibufgds,   -- sysClk   clk0_pll_bufg
      RST       => sysReset          -- sysReset      rst_dcm
   );

   dcm_clk0 : BUFG
   PORT MAP (
      O => clk0_bufg_out,
      I => clk0_bufg_in
   );

   dcm_clk90 : BUFG
   PORT MAP (
      O => clk90_bufg_out,
      I => clk90_bufg_in
   );

   clk   <= clk0_bufg_out;   -- CLK0 DCM output, with BUFG
   clk90 <= clk90_bufg_out;  -- CLK90 DCM output, with BUFG


   -- -----------------------------------------------------------------------------
   -- One BUFG is absolutely required to drive the IDELAYCTRL blocks 
   -- -----------------------------------------------------------------------------
   refClk_buf : BUFG
   PORT MAP (
      O => refClk,
      I => clk200
   );


END ARCHITECTURE rtl;
