--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_conf.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This module contains the user interface to the internal 
--		configuration registers.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - changed the clock frequency parameter to a clock period parameter
--
--  ** Features **
--  - Changed "`define" to "parameter", module can be instantiated
--    multiple times with different configurations
--  - Automatically calculate TRC, RD_LATENCY, and WR_LATENCY
--    based on the configuration setting
--  - Support of configuration 3
--
--  Register Map:
--  Addr    Mode   Description
--   0x0     RW     apConfWrD[7:0] = Mode[7:0]  -- LSB
--   0x1     RW     apConfWrD[3:0] = Read latency
--   0x2     RW     apConfWrD[3:0] = Write latency
--   0x3     RW     apConfWrD[7:0] = Refresh interval counter[7:0]
--   0x4     RW     apConfWrD[3:0] = Refresh interval counter[11:8]
--   0x5     RW     apConfWrD[7:0] = Refresh interval counter margin
--   0x6     RW     apConfWrD[3:0] = tRC
--   0x7     --     unused
--   0x8     --     unused
--   0x9     RW     apCOnfWrD[3:0] = burst length (valid values are 2, 4, 8)
--   0xA     RW     apConfWrD[1:0] = Mode[9:8]  -- MSB
--   0xB     RW     apConfWrD[0]   = Auto Refresh enable
--   0xC     --     unused
--   0xD     --     unused
--   0xE     --     unused
--   0xF     --     unused
--
--
--  Definition of "MODE_DEF[17:0]", i.e. MRS, Mode Register Set
--   [17:10]     [9]               [8]                  [7]         [6]         [5]               [4:3]           [2:0]
--  RESERVED, RL_MRS_ODT, RL_MRS_IMPEDANCE_MATCH, RL_MRS_DLL_RESET, '0',  RL_MRS_ADDR_MUX, RL_MRS_BURST_LENGTH, RL_MRS_CONF
--
--  bit[17:10] RESERVED               ("0000_0000")
--  bit[9]     RL_MRS_ODT             ('0': disabled termination;  '1': enabled termination)
--  bit[8]     RL_MRS_IMPEDANCE_MATCH ('0': internal 50ohms output buffer impedance;  '1':use external resistor)
--  bit[7]     RL_MRS_DLL_RESET       ('0':DLL reset (default); '1':DLL enabled)
--  bit[6]     ..static zero..        ('0')
--  bit[5]     RL_MRS_ADDR_MUX        ('0': non-muxed addr;  '1': muxed addr)
--  bit[4:3]   RL_MRS_BURST_LENGTH    ("00": BL2;  "01": BL4;  "10": BL8)
--  bit[2:0]   RL_MRS_CONF            ("001": mode1;  "010": mode2;  "011": mode3)
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


-- Configuration registers
ENTITY rld_conf IS
   GENERIC (
      -- specific rld_conf parameters
      RL_CK_PERIOD            : integer := 3003;    -- CK clock frequency of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "010";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '0';    -- 
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "00";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "00"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk            : IN  std_logic;
      rstHard        : IN  std_logic;
      rstConfig      : IN  std_logic;                      -- unused...

      apConfA        : IN  std_logic_vector(3 DOWNTO 0);   -- Address to access configuration registers
      apConfWr       : IN  std_logic;                      -- Write enable to configuration registers
      apConfWrD      : IN  std_logic_vector(7 DOWNTO 0);   -- Configuration write data
      apConfRd       : IN  std_logic;                      -- Read enable to configuration registers
      apConfRdD      : OUT std_logic_vector(7 DOWNTO 0);   -- Configuration read data

      confMReg       : OUT std_logic_vector(17 DOWNTO 0);  -- Mode Register Set (MRS)
      confRL         : OUT std_logic_vector(3 DOWNTO 0);   -- Read latency
      confWL         : OUT std_logic_vector(3 DOWNTO 0);   -- Write latency
      confRcCnt0     : OUT std_logic_vector(7 DOWNTO 0);   -- Refresh interval counter [7:0]
      confRcCnt1     : OUT std_logic_vector(7 DOWNTO 0);   -- Refresh interval counter [11:8]
      confRcMargin   : OUT std_logic_vector(7 DOWNTO 0);   -- Refresh interval counter margin
      confTrc        : OUT std_logic_vector(3 DOWNTO 0);   -- tRC value
      confTrdWr      : OUT std_logic_vector(2 DOWNTO 0);   -- TrdWr value, unused...
      confTwrRd      : OUT std_logic_vector(2 DOWNTO 0);   -- TwrRd value, unused...
      confBL         : OUT std_logic_vector(3 DOWNTO 0);   -- burst length
      confCycRef     : OUT std_logic                       -- control cyclic refresh (auto refresh enable)
   );
END ENTITY rld_conf;



ARCHITECTURE rtl OF rld_conf IS


   -- Conditional Constant for unconstrained std_logic_vector, based on one "equal to" condition
   function CCUSLV (COND_EXPR:std_logic_vector; COND_VAL:std_logic_vector; THEN_VAL:std_logic_vector; ELSE_VAL:std_logic_vector) return std_logic_vector is 
   begin 
      if COND_EXPR = COND_VAL then
         return THEN_VAL; 
      else
         return ELSE_VAL; 
      end if; 
   end;


   CONSTANT RESERVED      : std_logic_vector(7 DOWNTO 0) := "00000000";
   CONSTANT MODE_DEF      : std_logic_vector(17 DOWNTO 0) := RESERVED & RL_MRS_ODT & RL_MRS_IMPEDANCE_MATCH & RL_MRS_DLL_RESET & '0' & RL_MRS_ADDR_MUX & RL_MRS_BURST_LENGTH & RL_MRS_CONF;
   CONSTANT CYC_REF       : std_logic := '1';

   CONSTANT TRC              : std_logic_vector(3 DOWNTO 0) := CCUSLV( RL_MRS_CONF, "001", "0100", CCUSLV( RL_MRS_CONF, "010", "0110", "1000" ) );
   CONSTANT RD_LATENCY       : std_logic_vector(3 DOWNTO 0) := CCUSLV( RL_MRS_CONF, "001", "0100", CCUSLV( RL_MRS_CONF, "010", "0110", "1000" ) );
   CONSTANT WR_LATENCY       : std_logic_vector(3 DOWNTO 0) := CCUSLV( RL_MRS_CONF, "001", "0101", CCUSLV( RL_MRS_CONF, "010", "0111", "1001" ) );
   CONSTANT BLEN             : std_logic_vector(3 DOWNTO 0) := CCUSLV( RL_MRS_BURST_LENGTH, "00", "0010", CCUSLV( RL_MRS_BURST_LENGTH, "01", "0100", "1000" ) );
   CONSTANT AUTO_REF_CNT     : integer := 3880*1000/RL_CK_PERIOD;  -- determine count for 3.880us minimum Refresh time, based on CK period in ps
   CONSTANT AUTO_REF_CNT_SLV : std_logic_vector(15 DOWNTO 0) := STD_LOGIC_VECTOR( TO_UNSIGNED( AUTO_REF_CNT, 16 ) );  -- 16-bit value
   CONSTANT RCCNT0           : std_logic_vector(7 DOWNTO 0) := AUTO_REF_CNT_SLV( 7 DOWNTO 0);
   CONSTANT RCCNT1           : std_logic_vector(7 DOWNTO 0) := AUTO_REF_CNT_SLV(15 DOWNTO 8);
   CONSTANT RCMGN            : std_logic_vector(7 DOWNTO 0) := "00100000";
   CONSTANT TRW              : std_logic_vector(2 DOWNTO 0) := "101";
   CONSTANT TWR              : std_logic_vector(2 DOWNTO 0) := "101";


   -- for reading the configuration registers   
   SIGNAL confMRegI      : std_logic_vector(9 DOWNTO 0);
   SIGNAL confRLI        : std_logic_vector(3 DOWNTO 0);
   SIGNAL confWLI        : std_logic_vector(3 DOWNTO 0);
   SIGNAL confRcCnt0I    : std_logic_vector(7 DOWNTO 0);
   SIGNAL confRcCnt1I    : std_logic_vector(7 DOWNTO 0);
   SIGNAL confRcMarginI  : std_logic_vector(7 DOWNTO 0);
   SIGNAL confTrcI       : std_logic_vector(3 DOWNTO 0);
   SIGNAL confTrdWrI     : std_logic_vector(2 DOWNTO 0);
   SIGNAL confTwrRdI     : std_logic_vector(2 DOWNTO 0);

   SIGNAL cDInt          : std_logic_vector(7 DOWNTO 0);

   -- internal signals
   -- for writing the configuration registers
   SIGNAL confMReg_int      : std_logic_vector(17 DOWNTO 0);
   SIGNAL confRL_int        : std_logic_vector(3 DOWNTO 0);
   SIGNAL confWL_int        : std_logic_vector(3 DOWNTO 0);
   SIGNAL confRcCnt0_int    : std_logic_vector(7 DOWNTO 0);
   SIGNAL confRcCnt1_int    : std_logic_vector(7 DOWNTO 0);
   SIGNAL confRcMargin_int  : std_logic_vector(7 DOWNTO 0);
   SIGNAL confTrc_int       : std_logic_vector(3 DOWNTO 0);
   SIGNAL confTrdWr_int     : std_logic_vector(2 DOWNTO 0);
   SIGNAL confTwrRd_int     : std_logic_vector(2 DOWNTO 0);
   SIGNAL confBL_int        : std_logic_vector(3 DOWNTO 0);
   SIGNAL confCycRef_int    : std_logic;


BEGIN
   confMReg      <= confMReg_int;
   confRL        <= confRL_int;
   confWL        <= confWL_int;
   confRcCnt0    <= confRcCnt0_int;
   confRcCnt1    <= confRcCnt1_int;
   confRcMargin  <= confRcMargin_int;
   confTrc       <= confTrc_int;
   confTrdWr     <= confTrdWr_int;
   confTwrRd     <= confTwrRd_int;
   confBL        <= confBL_int;
   confCycRef    <= confCycRef_int;


   -- -----------------------------------------------------------------------------
   -- Buffered version of each registers, for reading 
   -- -----------------------------------------------------------------------------
   confMRegI      <= confMReg_int(9 DOWNTO 0);
   confRLI        <= confRL_int;
   confWLI        <= confWL_int;
   confRcCnt0I    <= confRcCnt0_int;
   confRcCnt1I    <= confRcCnt1_int;
   confRcMarginI  <= confRcMargin_int;
   confTrcI       <= confTrc_int;
   confTrdWrI     <= confTrdWr_int;
   confTwrRdI     <= confTwrRd_int;

   -- 8-bit read data, transfered to output read port
   apConfRdD <= cDInt;


   -- -----------------------------------------------------------------------------
   -- Read and Write to configuration registers
   -- -----------------------------------------------------------------------------
   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         IF (rstHard = '1') THEN
            confMReg_int      <= MODE_DEF;
            confRL_int        <= RD_LATENCY;
            confWL_int        <= WR_LATENCY;
            confRcCnt0_int    <= RCCNT0;
            confRcCnt1_int    <= RCCNT1;
            confRcMargin_int  <= RCMGN;
            confTrc_int       <= TRC;
            confTrdWr_int     <= TRW;
            confTwrRd_int     <= TWR;
            confBL_int        <= BLEN;
            confCycRef_int    <= CYC_REF;
         ELSE
            IF ( apConfWr = '1' ) THEN
               -- Write to configuration registers
               CASE apConfA IS
                  WHEN "0000" =>  confMReg_int(7)          <= apConfWrD(7);
                                  -- bit 6 is unused
                                  confMReg_int(5 DOWNTO 0) <= apConfWrD(5 DOWNTO 0);
                  WHEN "0001" =>  confRL_int               <= apConfWrD(3 DOWNTO 0);   -- read latency 
                  WHEN "0010" =>  confWL_int               <= apConfWrD(3 DOWNTO 0);   -- write latency
                  WHEN "0011" =>  confRcCnt0_int           <= apConfWrD(7 DOWNTO 0);
                  WHEN "0100" =>  confRcCnt1_int           <= apConfWrD(7 DOWNTO 0);
                  WHEN "0101" =>  confRcMargin_int         <= apConfWrD(7 DOWNTO 0);
                  WHEN "0110" =>  confTrc_int              <= apConfWrD(3 DOWNTO 0);
                  WHEN "0111" =>  confTrdWr_int            <= apConfWrD(2 DOWNTO 0);
                  WHEN "1000" =>  confTwrRd_int            <= apConfWrD(2 DOWNTO 0);
                  WHEN "1001" =>  confBL_int               <= apConfWrD(3 DOWNTO 0);
                  WHEN "1010" =>  confMReg_int(9 DOWNTO 8) <= apConfWrD(1 DOWNTO 0);
                  WHEN "1011" =>  confCycRef_int           <= apConfWrD(0);
                  -- "1100" to "1111" are covered in "default"
                  WHEN OTHERS =>  NULL;
               END CASE;

            ELSE
               IF ( apConfRd = '1' ) THEN
                  -- Read from configuration registers
                  CASE apConfA IS
                     WHEN "0000" =>  cDInt <= confMRegI(7 DOWNTO 0);
                     WHEN "0001" =>  cDInt <= "0000" & confRLI;
                     WHEN "0010" =>  cDInt <= "0000" & confWLI;
                     WHEN "0011" =>  cDInt <= confRcCnt0I;
                     WHEN "0100" =>  cDInt <= confRcCnt1I;
                     WHEN "0101" =>  cDInt <= confRcMarginI;
                     WHEN "0110" =>  cDInt <= "0000" & confTrcI(3 DOWNTO 0);
                     WHEN "0111" =>  cDInt <= "00000" & confTrdWrI(2 DOWNTO 0);
                     WHEN "1000" =>  cDInt <= "00000" & confTwrRdI(2 DOWNTO 0);
                     WHEN "1001" =>  cDInt <= "0000" & confBL_int;
                     WHEN "1010" =>  cDInt <= "000000" & confMRegI(9 DOWNTO 8);
                     WHEN "1011" =>  cDInt <= "0000000" & confCycRef_int;
                     WHEN OTHERS =>  cDInt <= confMRegI(7 DOWNTO 0);  -- default
                  END CASE;
               END IF;

            END IF;
         END IF;
      END IF;
   END PROCESS;


END ARCHITECTURE rtl;
