--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_controller.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	 Virtex-5
--	Purpose: This module instantiates the control and configuration modules
--		* Control module has the main control logic
--		* Includes refresh logic,  write logic and read logic
--		* config module has the parameters for config register
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - changed the clock frequency parameter to a clock period parameter
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_controller IS
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4; 
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps   
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "010";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '0';    -- 
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      -- Global signals
      clk                     : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstConfig               : IN  std_logic;
      afEmpty                 : IN  std_logic;
      afA                     : IN  std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      -- Reset configuration signals from reset module
      rstCmd                  : IN  std_logic_vector(3 DOWNTO 0);
      rstAd                   : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      ctlafRdEn               : OUT std_logic;
      -- write data enable for pads
      ctlIoWdEn               : OUT std_logic;
      -- commands and address, bank outputs
      ctlCmd                  : OUT std_logic_vector(3 DOWNTO 0);
      ctlAd                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      -- write data fifo read
      ctlWdfRdEn              : OUT std_logic;
      ctlBa                   : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      -- indicates okay to change data valid edge for per-bit deskew
      ctlOkEdge               : OUT std_logic;
      rldWriteDone            : OUT std_logic;
      apWdfWrEn               : IN  std_logic;
      wdfAlmostFull           : OUT std_logic;
      apConfRd                : IN  std_logic;
      apConfWrD               : IN  std_logic_vector(7 DOWNTO 0);
      apConfRdD               : OUT std_logic_vector(7 DOWNTO 0);
      apConfWr                : IN  std_logic;
      apConfA                 : IN  std_logic_vector(3 DOWNTO 0);
      confMReg                : OUT std_logic_vector(17 DOWNTO 0);
      confBL                  : OUT std_logic_vector(3 DOWNTO 0);
      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY rld_controller;



ARCHITECTURE rtl OF rld_controller IS


   COMPONENT rld_conf
   GENERIC (
      -- specific rld_conf parameters
      RL_CK_PERIOD            : integer := 3003;    -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "010";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '0';    -- 
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk            : IN  std_logic;
      rstHard        : IN  std_logic;
      rstConfig      : IN  std_logic;
      apConfA        : IN  std_logic_vector(3 DOWNTO 0);
      apConfWr       : IN  std_logic;
      apConfWrD      : IN  std_logic_vector(7 DOWNTO 0);
      apConfRd       : IN  std_logic;
      apConfRdD      : OUT std_logic_vector(7 DOWNTO 0);
      confMReg       : OUT std_logic_vector(17 DOWNTO 0);
      confRL         : OUT std_logic_vector(3 DOWNTO 0);
      confWL         : OUT std_logic_vector(3 DOWNTO 0);
      confRcCnt0     : OUT std_logic_vector(7 DOWNTO 0);
      confRcCnt1     : OUT std_logic_vector(7 DOWNTO 0);
      confRcMargin   : OUT std_logic_vector(7 DOWNTO 0);
      confTrc        : OUT std_logic_vector(3 DOWNTO 0);
      confTrdWr      : OUT std_logic_vector(2 DOWNTO 0);
      confTwrRd      : OUT std_logic_vector(2 DOWNTO 0);
      confBL         : OUT std_logic_vector(3 DOWNTO 0);
      confCycRef     : OUT std_logic
   );
   END COMPONENT;


   COMPONENT rld_ctl
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4; 
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      -- specific rld_conf parameters
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "010";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '0';    -- 
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN std_logic;
      rstHard                 : IN std_logic;
      rstConfig               : IN std_logic;
      cmdFifoEmpty            : IN std_logic;
      cmdFifoOut              : IN std_logic_vector(25 DOWNTO 0);
      -- Reset configuration signals
      rstCmd                  : IN std_logic_vector(3 DOWNTO 0);
      rstAd                   : IN std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : IN std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      -- Configuration Register values
      confRL                  : IN std_logic_vector(3 DOWNTO 0);
      confWL                  : IN std_logic_vector(3 DOWNTO 0);
      confRcCnt0              : IN std_logic_vector(7 DOWNTO 0);
      confRcCnt1              : IN std_logic_vector(7 DOWNTO 0);
      confRcMargin            : IN std_logic_vector(7 DOWNTO 0);
      confTrc                 : IN std_logic_vector(3 DOWNTO 0);
      confTrdWr               : IN std_logic_vector(2 DOWNTO 0);
      confTwrRd               : IN std_logic_vector(2 DOWNTO 0);
      confBL                  : IN std_logic_vector(3 DOWNTO 0);
      confAutoRefrEn          : IN std_logic;
      cmdFifoRdEn             : OUT std_logic;
      -- write data enable for pads
      ctlIoWdEn               : OUT std_logic;
      -- commands and address, bank outputs
      ctlCmdOut               : OUT std_logic_vector(3 DOWNTO 0);
      ctlAddrOut              : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      -- write data fifo read
      ctlWdfRdEn              : OUT std_logic;
      ctlBankOut              : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      ctlOkEdge               : OUT std_logic;
      rldWriteDone            : OUT std_logic;
      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   -- configuration register's content
   SIGNAL confRL           : std_logic_vector(3 DOWNTO 0);
   SIGNAL confWL           : std_logic_vector(3 DOWNTO 0);
   SIGNAL confRcCnt0       : std_logic_vector(7 DOWNTO 0);
   SIGNAL confRcCnt1       : std_logic_vector(7 DOWNTO 0);
   SIGNAL confRcMargin     : std_logic_vector(7 DOWNTO 0);
   SIGNAL confTrc          : std_logic_vector(3 DOWNTO 0);
   SIGNAL confTwrRd        : std_logic_vector(2 DOWNTO 0);
   SIGNAL confTrdWr        : std_logic_vector(2 DOWNTO 0);
   SIGNAL confCycRef       : std_logic;   -- also named "confAutoRefrEn"

   -- internal signals
   SIGNAL confBL_sig  : std_logic_vector(3 DOWNTO 0);


BEGIN
   cs_io <= (OTHERS => 'Z');
   wdfAlmostFull  <= '0';  -- not assigned...

   confTwrRd  <= (others => '0');  -- not assigned...
   confTrdWr  <= (others => '0');  -- not assigned...

   confBL  <= confBL_sig;


   -- -----------------------------------------------------------------------------
   -- rld_conf
   --   Contains the user interface to the internal configuration registers
   -- -----------------------------------------------------------------------------
   rld_conf0 : rld_conf 
   GENERIC MAP (
      -- specific rld_conf parameters
      RL_CK_PERIOD            => RL_CK_PERIOD,
      RL_MRS_CONF             => RL_MRS_CONF,
      RL_MRS_BURST_LENGTH     => RL_MRS_BURST_LENGTH,
      RL_MRS_ADDR_MUX         => RL_MRS_ADDR_MUX,
      RL_MRS_DLL_RESET        => RL_MRS_DLL_RESET,
      RL_MRS_IMPEDANCE_MATCH  => RL_MRS_IMPEDANCE_MATCH,
      RL_MRS_ODT              => RL_MRS_ODT,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              => RL_IO_TYPE,
      DEVICE_ARCH             => DEVICE_ARCH,
      CAPTURE_METHOD          => CAPTURE_METHOD
   )
   PORT MAP (
      clk           => clk,
      rstHard       => rstHard,
      rstConfig     => rstConfig,         -- unused...

      apConfA       => apConfA,
      apConfWr      => apConfWr,
      apConfWrD     => apConfWrD,
      apConfRd      => apConfRd,
      apConfRdD     => apConfRdD,

      confMReg      => confMReg,          -- Mode Register Set (MRS)
      confRL        => confRL,            -- Read latency  
      confWL        => confWL,            -- Write latency 
      confRcCnt0    => confRcCnt0,        -- Refresh interval counter [7:0]
      confRcCnt1    => confRcCnt1,        -- Refresh interval counter [11:8]
      confRcMargin  => confRcMargin,      -- Refresh interval counter margin
      confTrc       => confTrc,           -- tRC value
      confTrdWr     => open,
      confTwrRd     => open,
      confBL        => confBL_sig,        -- burst length
      confCycRef    => confCycRef         -- control cyclic refresh (auto refresh enable)
   );


   -- -----------------------------------------------------------------------------
   -- rld_ctl
   --   low-level memory controller :
   --     * Main control is implemented in this module
   --     * Initiates read, write and refresh commands to rld memory
   -- -----------------------------------------------------------------------------
   rld_ctl0 : rld_ctl 
   GENERIC MAP (
      RL_DQ_WIDTH             => RL_DQ_WIDTH,
      DEV_DQ_WIDTH            => DEV_DQ_WIDTH,
      NUM_OF_DEVS             => NUM_OF_DEVS,
      NUM_OF_DKS              => NUM_OF_DKS,
      DEV_AD_WIDTH            => DEV_AD_WIDTH,
      DEV_BA_WIDTH            => DEV_BA_WIDTH,
      APP_AD_WIDTH            => APP_AD_WIDTH,
      DUPLICATE_CONTROLS      => DUPLICATE_CONTROLS,
      -- specific to rld_conf
      RL_CK_PERIOD            => RL_CK_PERIOD,    
      RL_MRS_CONF             => RL_MRS_CONF,
      RL_MRS_BURST_LENGTH     => RL_MRS_BURST_LENGTH,
      RL_MRS_ADDR_MUX         => RL_MRS_ADDR_MUX,
      RL_MRS_DLL_RESET        => RL_MRS_DLL_RESET,
      RL_MRS_IMPEDANCE_MATCH  => RL_MRS_IMPEDANCE_MATCH,
      RL_MRS_ODT              => RL_MRS_ODT,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              => RL_IO_TYPE,
      DEVICE_ARCH             => DEVICE_ARCH,
      CAPTURE_METHOD          => CAPTURE_METHOD
   )
   PORT MAP (
      clk             => clk,
      rstHard         => rstHard,
      rstConfig       => rstConfig,

      cmdFifoEmpty    => afEmpty,   -- merged empty signal
      cmdFifoOut      => afA,       -- merged address bus
      rstCmd          => rstCmd,
      rstAd           => rstAd,
      rstBa           => rstBa,

      confRL          => confRL,
      confWL          => confWL,
      confRcCnt0      => confRcCnt0,
      confRcCnt1      => confRcCnt1,
      confRcMargin    => confRcMargin,
      confTrc         => confTrc,
      confTrdWr       => confTwrRd,  -- unused...
      confTwrRd       => confTrdWr,  -- unused...
      confBL          => confBL_sig,
      confAutoRefrEn  => confCycRef,

      cmdFifoRdEn     => ctlafRdEn,
      ctlIoWdEn       => ctlIoWdEn,
      ctlCmdOut       => ctlCmd,
      ctlAddrOut      => ctlAd,
      ctlWdfRdEn      => ctlWdfRdEn,
      ctlBankOut      => ctlBa,
      ctlOkEdge       => ctlOkEdge,
      rldWriteDone    => rldWriteDone,

      cs_io           => cs_io
-- -- OLD ---------------------------------------------
--      apWdfWrEn       => apWdfWrEn,
--      wdfAlmostFull   => wdfAlmostFull,
--      confMReg        => confMReg
-- ----------------------------------------------------
   );


END ARCHITECTURE rtl;
