--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007, 2008 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_controller_iobs.vhd
--  /   /         Timestamp: 14 March 2008
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	 Virtex-5
--	Purpose: This module holds the control physical layer that interfaces 
--		      with the memory pins: A, BA, WE#, REF# and CS#.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Added IOB=TRUE to more of the control signals
--*****************************************************************************

library	IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library	UNISIM;
use UNISIM.VCOMPONENTS.ALL;

ENTITY rld_controller_iobs IS
   GENERIC (
      RL_DQ_WIDTH                    :  integer := 36;    
      DEV_DQ_WIDTH                   :  integer := 18;    --  data width of the memory device
      NUM_OF_DEVS                    :  integer := 2;    --  number of memory devices
      NUM_OF_DKS                     :  integer := 4;    
      DEV_AD_WIDTH                   :  integer := 20;    --  address width of the memory device
      DEV_BA_WIDTH                   :  integer := 3;    --  bank address width of the memory device
      DUPLICATE_CONTROLS             :  boolean := FALSE;    --  Duplicate the ports for A, BA, WE# and REF#
      --
      RL_IO_TYPE                     :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH                    :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD                 :  std_logic_vector(1 DOWNTO 0) := "00"    --  Direct Clocking=2'b00  SerDes=2'b01
   );   
   PORT (
      clk                     : IN std_logic;   
      clk90                   : IN std_logic;   
      sync_sysReset           : IN std_logic;   
      rstHard                 : IN std_logic;   
      rstHard_90              : IN std_logic;   
      rstHard_180             : IN std_logic;   
      rstHard_270             : IN std_logic;   

      phy_a                   : OUT std_logic_vector(DEV_AD_WIDTH - 1 DOWNTO 0);   
      phy_ba                  : OUT std_logic_vector(DEV_BA_WIDTH - 1 DOWNTO 0);   
      phy_cs_n                : OUT std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0); 
      phy_we_n                : OUT std_logic;   
      phy_ref_n               : OUT std_logic; 
        
      ctl_Cmd                 : IN std_logic_vector(3 DOWNTO 0);   
      ctl_Ad                  : IN std_logic_vector(DEV_AD_WIDTH - 1 DOWNTO 0);   
      ctl_Ba                  : IN std_logic_vector(DEV_BA_WIDTH - 1 DOWNTO 0);   
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );   
END ENTITY rld_controller_iobs;

ARCHITECTURE rtl OF rld_controller_iobs IS

--   ATTRIBUTE iostandard: STRING;
--   -- using component name, might need to details each bit of the bus independently
--   ATTRIBUTE iostandard OF phy_a     : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF phy_ba    : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF phy_we_n  : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF phy_ref_n : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF phy_cs_n  : LABEL IS "HSTL_II_18";
--   ATTRIBUTE IOB OF phy_a      : LABEL IS "TRUE";
--   ATTRIBUTE IOB OF phy_ba     : LABEL IS "TRUE";
--   ATTRIBUTE IOB OF phy_we_n   : LABEL IS "TRUE";
--   ATTRIBUTE IOB OF phy_ref_n  : LABEL IS "TRUE";
--   ATTRIBUTE IOB OF phy_cs_n   : LABEL IS "TRUE";


   SIGNAL ctlCmd                   :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL ctlAd                    :  std_logic_vector(DEV_AD_WIDTH - 1 DOWNTO 0);   
   SIGNAL ctlBa                    :  std_logic_vector(DEV_BA_WIDTH - 1 DOWNTO 0);
      
   -- specific to Virtex-5
   SIGNAL ctlBa_d1                 :  std_logic_vector(DEV_BA_WIDTH - 1 DOWNTO 0);   
   SIGNAL ctlAd_d1                 :  std_logic_vector(DEV_AD_WIDTH - 1 DOWNTO 0);   
   SIGNAL ctlCmd_d1                :  std_logic_vector(3 DOWNTO 0);   
   
   -- specific to Virtex-4
   SIGNAL ctlBa_d7                 :  std_logic_vector(DEV_BA_WIDTH - 1 DOWNTO 0);   
   SIGNAL ctlBa_d8                 :  std_logic_vector(DEV_BA_WIDTH - 1 DOWNTO 0);   
   SIGNAL ctlAd_d7                 :  std_logic_vector(DEV_AD_WIDTH - 1 DOWNTO 0);   
   SIGNAL ctlAd_d8                 :  std_logic_vector(DEV_AD_WIDTH - 1 DOWNTO 0);   
   SIGNAL ctlCmd_d7                :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL ctlCmd_d8                :  std_logic_vector(3 DOWNTO 0);   
   SIGNAL phy_cs_dup               :  std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0); 
     
   -- test points
   SIGNAL TEST_CS_N                :  std_logic;   
   SIGNAL TEST_REF_N               :  std_logic;   
   SIGNAL TEST_WE_N                :  std_logic;   
   SIGNAL TEST_BA                  :  std_logic_vector(DEV_BA_WIDTH - 1 DOWNTO 0);
      
   SIGNAL clk180                   :  std_logic;   
   SIGNAL clk270                   :  std_logic;   
   
   SIGNAL bit_0                    :  std_logic;   --  constants
   SIGNAL bit_1                    :  std_logic;      


BEGIN
   cs_io <= (OTHERS => 'Z');
   bit_0      <= '0' ;
   bit_1      <= '1' ;
   clk180     <= NOT clk ;
   clk270     <= NOT clk90 ;

   -- -----------------------------------------------------------------------------
   -- Bank Address (BA) bits
   -- -----------------------------------------------------------------------------
   ba_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      ctlBa <= ctl_Ba;
   
      ba_srl_gen_inst : FOR i_ba IN  DEV_BA_WIDTH-1 DOWNTO 0  GENERATE 
      BEGIN
         Ba_srl_delay : SRL16 
         PORT MAP (
            Q     => ctlBa_d7(i_ba),
            A0    => bit_0,
            A1    => bit_1,
            A2    => bit_1,
            A3    => bit_0,
            CLK   => clk,
            D     => ctlBa(i_ba)
         );   
      END GENERATE ba_srl_gen_inst;
   
      Ba_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlBa_d8 <= ctlBa_d7;    
         END IF;
      END PROCESS Ba_fd;

      Ba_phy_fd : PROCESS (clk180, rstHard_180)-- D-flop, async clear
      BEGIN
         IF (rstHard_180 = '1') THEN
            phy_ba <= (OTHERS => '0');     
         ELSIF RISING_EDGE ( clk180 ) THEN 
            phy_ba <= ctlBa_d8;
         END IF;
      END PROCESS Ba_phy_fd;

      -- Duplicate the ports for BA
      ba_dup_virtex4_insts : IF ( DUPLICATE_CONTROLS = TRUE) GENERATE
      BEGIN
         Ba_dup_phy_fd : PROCESS (clk180, rstHard_180)-- D-flop, async clear
         BEGIN
            IF (rstHard_180 = '1') THEN
               phy_ba(2*DEV_BA_WIDTH-1 DOWNTO DEV_BA_WIDTH) <= (OTHERS => '0');
            ELSIF RISING_EDGE ( clk180 ) THEN
               phy_ba(2*DEV_BA_WIDTH-1 DOWNTO DEV_BA_WIDTH) <= ctlBa_d8;
            END IF;
         END PROCESS Ba_dup_phy_fd;
      END GENERATE ba_dup_virtex4_insts;
      
   END GENERATE ba_virtex4_insts;


   ba_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE   
      Ba_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlBa    <= ctl_Ba;    
            ctlBa_d1 <= ctlBa;    
            TEST_BA  <= ctlBa_d1;    --  TEST probe for RC_BA[2:0] output signals
         END IF;
      END PROCESS Ba_fd;

      Ba_phy_fd : PROCESS (clk180)
      BEGIN
         IF RISING_EDGE ( clk180 ) THEN
            IF (rstHard_180 = '1') THEN 
               phy_ba <= (OTHERS => '0');    
            ELSE
               phy_ba <= ctlBa_d1;     
            END IF;
         END IF;
      END PROCESS Ba_phy_fd;

   END GENERATE ba_virtex5_insts;

   
   -- -----------------------------------------------------------------------------
   -- Address (A) bits
   -- -----------------------------------------------------------------------------
   ad_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      ctlAd <= ctl_Ad;
   
      ad_bit_insts : FOR i_ad IN  DEV_AD_WIDTH-1 DOWNTO 0  GENERATE 
      Ad_srl_delay : SRL16 
         PORT MAP (
            Q     => ctlAd_d7(i_ad),
            A0    => bit_0,
            A1    => bit_1,
            A2    => bit_1,
            A3    => bit_0,
            CLK   => clk,
            D     => ctlAd(i_ad)
         );   
      END GENERATE ad_bit_insts;

      Ad_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlAd_d8 <= ctlAd_d7;    
           END IF;
      END PROCESS Ad_fd;

      Ad_phy_fd : PROCESS (clk180, rstHard_180)-- D-flop, async clear
      BEGIN
         IF (rstHard_180 = '1') THEN
            phy_a <= (OTHERS => '0');     
         ELSIF RISING_EDGE ( clk180 ) THEN
            phy_a <= ctlAd_d8;     
         END IF;
      END PROCESS Ad_phy_fd;

      -- Duplicate the ports for A
      ad_dup_virtex4_insts : IF ( DUPLICATE_CONTROLS = TRUE) GENERATE
      BEGIN
         Ad_dup_phy_fd : PROCESS (clk180, rstHard_180)-- D-flop, async clear
         BEGIN
            IF (rstHard_180 = '1') THEN
               phy_a(2*DEV_AD_WIDTH-1 DOWNTO DEV_AD_WIDTH) <= (OTHERS => '0');
            ELSIF RISING_EDGE ( clk180 ) THEN
               phy_a(2*DEV_AD_WIDTH-1 DOWNTO DEV_AD_WIDTH) <= ctlAd_d8;
            END IF;
         END PROCESS Ad_dup_phy_fd;
      END GENERATE ad_dup_virtex4_insts;
      
   END GENERATE ad_virtex4_insts;

   ad_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      
      Ad_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlAd    <= ctl_Ad;    
            ctlAd_d1 <= ctlAd;    
         END IF;
      END PROCESS Ad_fd;

      Ad_phy_fd : PROCESS (clk180)
      BEGIN
         IF RISING_EDGE ( clk180 ) THEN
            IF (rstHard_180 = '1') THEN
               phy_a <= (OTHERS => '0');    
            ELSE
               phy_a <= ctlAd_d1;    
            END IF;
         END IF;
      END PROCESS Ad_phy_fd;

   END GENERATE ad_virtex5_insts;
   
   -- -----------------------------------------------------------------------------
   -- Signal "ctl_Cmd" holds control bits for CS#, WE# and REF#
   --    ctl_Cmd[3] : CS#
   --    ctl_Cmd[2] : <unused>  (was AS#)
   --    ctl_Cmd[1] : WE#
   --    ctl_Cmd[0] : REF#
   -- -----------------------------------------------------------------------------
   
   
   -- -----------------------------------------------------------------------------
   -- CS chip select (control)
   -- -----------------------------------------------------------------------------
   cs_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      ctlCmd(3) <= ctl_Cmd(3);

      Cs_srl_delay : SRL16 
         PORT MAP (
            Q     => ctlCmd_d7(3),
            A0    => bit_0,
            A1    => bit_1,
            A2    => bit_1,
            A3    => bit_0,
            CLK   => clk,
            D     => ctlCmd(3)
         );   
      
--      Cs_fd : PROCESS (clk)
--      BEGIN
--       IF RISING_EDGE ( clk ) THEN
--          phy_cs_dup <= (OTHERS => ctlCmd_d7(3));  -- duplicate ctl_Cmd[3]
--       END IF;
--      END PROCESS Cs_fd;
      
      cs_bit_insts : FOR i_cs IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE 
      Cs_fd : FD 
         PORT MAP (
            C => clk,
            D => ctlCmd_d7(3),
            Q => phy_cs_dup(i_cs)-- duplicate ctl_Cmd[3]
         );
      END GENERATE cs_bit_insts;      
   
      Cs_phy_fd : PROCESS (clk180, sync_sysReset)-- D-flop, async preset
      BEGIN
         IF (sync_sysReset = '1') THEN 
            phy_cs_n <= (NUM_OF_DEVS-1 DOWNTO 0  => '1');
         ELSIF RISING_EDGE ( clk180 ) THEN
            phy_cs_n <= phy_cs_dup;
         END IF;
      END PROCESS Cs_phy_fd;
   END GENERATE cs_virtex4_insts;
   
   cs_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      Cmd3_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlCmd(3)      <= ctl_Cmd(3);    
            ctlCmd_d1(3)   <= ctlCmd(3);    
            TEST_CS_N      <= ctlCmd_d1(3);    --  TEST probe for RC_CS_N output signal
         END IF;
      END PROCESS Cmd3_fd;

      Cs_phy_fd : PROCESS (clk180, sync_sysReset)-- D-flop, async preset
      BEGIN
         IF (sync_sysReset = '1') THEN 
            phy_cs_n <= (NUM_OF_DEVS-1 DOWNTO 0 => '1');
         ELSIF RISING_EDGE ( clk180 ) THEN
            phy_cs_n <= (NUM_OF_DEVS-1 DOWNTO 0 => ctlCmd_d1(3));
         END IF;
      END PROCESS Cs_phy_fd;
   
--      cs_bit_insts : FOR i_cs IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
--      BEGIN
--         Cs_phy_fd : FDPE
--         GENERIC MAP (
--         IOB => "TRUE"
--         )
--         PORT MAP (
--            C     => clk180,
--            PRE   => sync_sysReset,
--            CE    => bit_1,
--            D     => ctlCmd_d1[3],
--            Q     => phy_cs_n[i_cs]
--         );
--      END GENERATE cs_bit_insts; 
   END GENERATE cs_virtex5_insts;
   
   
   -- -----------------------------------------------------------------------------
   -- ctl_Cmd[2] is unused
   -- -----------------------------------------------------------------------------
   PROCESS (clk)
   BEGIN
      IF RISING_EDGE ( clk ) THEN
         ctlCmd_d1(2) <= '0';    
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- WE read/write command
   -- -----------------------------------------------------------------------------
   we_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      ctlCmd(1) <= ctl_Cmd(1);
   
      We_srl_delay : SRL16 
         PORT MAP (
            Q     => ctlCmd_d7(1),
            A0    => bit_0,
            A1    => bit_1,
            A2    => bit_1,
            A3    => bit_0,
            CLK   => clk,
            D     => ctlCmd(1)
         );   
   
      We_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlCmd_d8(1) <= ctlCmd_d7(1);    
         END IF;
      END PROCESS We_fd;

      We_phy_fd : PROCESS (clk180, sync_sysReset)-- D-flop, async preset
      BEGIN
         IF (sync_sysReset = '1') THEN
            phy_we_n <= '1';    
         ELSIF RISING_EDGE ( clk180 ) THEN
            phy_we_n <= ctlCmd_d8(1);    
         END IF;
      END PROCESS We_phy_fd;
   END GENERATE we_virtex4_insts;
   
   we_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      We_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlCmd(1)      <= ctl_Cmd(1);    
            ctlCmd_d1(1)   <= ctlCmd(1);    
            TEST_WE_N      <= ctlCmd_d1(1);    --  TEST probe for RC_WE_N output signal
         END IF;
      END PROCESS We_fd;

      We_phy_fd : PROCESS (clk180, sync_sysReset)-- D-flop, async preset
      BEGIN
         IF (sync_sysReset = '1') THEN
            phy_we_n <= '1';    
         ELSIF RISING_EDGE ( clk180 ) THEN
            phy_we_n <= ctlCmd_d1(1);    
         END IF;
      END PROCESS We_phy_fd;
   END GENERATE we_virtex5_insts;
   
   
   -- -----------------------------------------------------------------------------
   -- REF refresh command
   -- -----------------------------------------------------------------------------
   ref_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      ctlCmd(0) <= ctl_Cmd(0);

      Ref_srl_delay : SRL16 
         PORT MAP (
            Q     => ctlCmd_d7(0),
            A0    => bit_0,
            A1    => bit_1,
            A2    => bit_1,
            A3    => bit_0,
            CLK   => clk,
            D     => ctlCmd(0));   

      Ref_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlCmd_d8(0) <= ctlCmd_d7(0);    
         END IF;
      END PROCESS Ref_fd;

      We_phy_fd : PROCESS (clk180, sync_sysReset)-- D-flop, async preset
      BEGIN
         IF (sync_sysReset = '1') THEN
            phy_ref_n <= '1';    
         ELSIF RISING_EDGE ( clk180 ) THEN
            phy_ref_n <= ctlCmd_d8(0);    
         END IF;
      END PROCESS We_phy_fd;
   END GENERATE ref_virtex4_insts;

   ref_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      Ref_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            ctlCmd(0)      <= ctl_Cmd(0);    
            ctlCmd_d1(0)   <= ctlCmd(0);    
            TEST_REF_N     <= ctlCmd_d1(0);    --  TEST probe for RC_REF_N output signal
         END IF;
      END PROCESS Ref_fd;

      Ref_phy_fd : PROCESS (clk180, sync_sysReset)-- D-flop, async preset
      BEGIN
         IF (sync_sysReset = '1') THEN
            phy_ref_n <= '1';    
         ELSIF RISING_EDGE ( clk180 ) THEN
            phy_ref_n <= ctlCmd_d1(0);    
         END IF;
      END PROCESS Ref_phy_fd;
   END GENERATE ref_virtex5_insts;
   
   
   -- -----------------------------------------------------------------------------
   -- Assign chipscope probes
   -- -----------------------------------------------------------------------------
   cs_CTL_virtex4_probes : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      -- allocate 256-511 to RLDRAM2 PHY related signals
      cs_io(0)             <= ctlCmd_d8(3) ;
      cs_io(1)             <= ctlCmd_d8(1) ;
      cs_io(2)             <= ctlCmd_d8(0) ;
      cs_io(22 DOWNTO 3)   <= ctlAd_d8(19 DOWNTO 0) ;
   END GENERATE cs_CTL_virtex4_probes;

   cs_CTL_virtex5_probes : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      -- chipscope PHY related signals
      cs_io(93)            <= TEST_CS_N ;
      cs_io(94)            <= TEST_WE_N ;
      cs_io(95)            <= TEST_REF_N ;
      cs_io(98 DOWNTO 96)  <= TEST_BA ;
      cs_io(99)            <= '0' ;
   END GENERATE cs_CTL_virtex5_probes;
   

END ARCHITECTURE rtl;
