--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_data_path.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This module instantiates data_write and tap_logic modules.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Pass RL_CK_PERIOD parameter to sub-modules
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_data_path IS
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      QK_DATA_WIDTH                  : integer := 9;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE; -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD                   : integer := 3003;  -- CK clock period of the RLDRAM in ps
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      rstHard_refClk          : IN  std_logic;

      qk_idelay_inc           : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);  -- IDELAY controls for QK (read clock)
      qk_idelay_ce            : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_rst           : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_inc         : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);  -- IDELAY controls for DQ (read data)
      data_idelay_ce          : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_rst         : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_inc         : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);    -- IDELAY controls for QVLD (read enable)
      qvld_idelay_ce          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_rst         : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

      rd_data_rise            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rd_data_fall            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

      okToSelTap              : IN  std_logic;

      ctlIoWdEn               : IN  std_logic;
      wdfDM                   : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      wdfD                    : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_rise            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_fall            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_en_clk90             : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_rise          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_fall          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

      CTRL_DUMMYREAD_START    : IN  std_logic;
      refresh_cmd             : IN  std_logic;
      CTRL_DQS_RST            : IN  std_logic;
      CTRL_DQS_EN             : IN  std_logic;
      read_enable_rise        : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      read_enable_fall        : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      first_stage_done        : OUT std_logic;
      second_stage_begin      : IN  std_logic;
      qvld_cal                : IN  std_logic;
      SEL_DONE                : OUT std_logic;
      issueCalibration        : IN  std_logic;

      read_en_delayed         : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      read_data_rise          : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      read_data_fall          : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

      idelay_ctrl_rdy         : OUT std_logic;

      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY rld_data_path;



ARCHITECTURE rtl OF rld_data_path IS


   function AND_REDUCE(ARG: in STD_LOGIC_VECTOR) return std_logic is
      variable result: STD_LOGIC := '1';
   begin
      for i in ARG'range loop
         result := result AND ARG(i);
      end loop;
      return result;
   end function AND_REDUCE;


   -- Delay select conditions
   CONSTANT ALIGNED  : std_logic_vector(1 DOWNTO 0) := "00";
   CONSTANT EARLY    : std_logic_vector(1 DOWNTO 0) := "01";
   CONSTANT LATE     : std_logic_vector(1 DOWNTO 0) := "10";


   COMPONENT rld_tap_logic
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      QK_DATA_WIDTH           : integer := 9;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      rstHard_refClk          : IN  std_logic;

      read_enable_rise        : IN  std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      read_enable_fall        : IN  std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      read_enable_rise_r      : IN  std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      CTRL_DUMMYREAD_START    : IN  std_logic;
      refresh_cmd             : IN  std_logic;
      issueCalibration        : IN  std_logic;

      qk_idelay_inc           : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qk_idelay_ce            : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qk_idelay_rst           : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      data_idelay_inc         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      data_idelay_ce          : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      data_idelay_rst         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      qvld_idelay_inc         : OUT std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qvld_idelay_ce          : OUT std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qvld_idelay_rst         : OUT std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      read_enable_out         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      delay_select            : OUT std_logic_vector(4*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      DQ_tap_count            : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)*6-1 DOWNTO 0);  -- 6-bit DQ counts, expandable per device, 2 counts per device

      first_stage_done        : OUT std_logic;   -- QK centered in DQ window 
      second_stage_begin      : IN  std_logic;   -- ready to start next phase
      qvld_cal                : IN  std_logic;   -- align qvld
      data_tap_select         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      idelay_ctrl_rdy_r1      : OUT std_logic;

      rd_data_rise            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rd_data_fall            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;

   COMPONENT rld_data_write
      GENERIC (
         RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
         DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
         NUM_OF_DEVS                    : integer := 2;    -- number of memory devices
         NUM_OF_DKS                     : integer := 4;
         QK_DATA_WIDTH                  : integer := 9;
         DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
         DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
         APP_AD_WIDTH                   : integer := 26;
         DUPLICATE_CONTROLS             : boolean := FALSE;   -- Duplicate the ports for A, BA, WE# and REF#
         -- specific to FPGA/memory devices and capture method
         RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
         DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
         CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
      );
      PORT (
         clk                     : IN  std_logic;
         clk90                   : IN  std_logic;
         rstHard                 : IN  std_logic;
         rstHard_90              : IN  std_logic;
         rstHard_180             : IN  std_logic;
         rstHard_270             : IN  std_logic;
         ctlIoWdEn               : IN  std_logic;
         wdfDM                   : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
         wdfD                    : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
         wr_data_rise            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
         wr_data_fall            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
         wr_en_clk90             : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
         data_mask_rise          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
         data_mask_fall          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0)
      );
   END COMPONENT;


   SIGNAL read_enable_rise_r       : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL SEL_DONE_a1              : std_logic;
   SIGNAL SEL_DONE_a2              : std_logic;
   SIGNAL rd_data_rise_r1          : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL rd_data_fall_r1          : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

   SIGNAL data_tap_select          : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL DQ_tap_count             : std_logic_vector(2*NUM_OF_DEVS*6-1 DOWNTO 0);

   SIGNAL delay_select             : std_logic_vector(4*NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL read_enable_out          : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL late_valid               : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL delay                    : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL clk180                   : std_logic;
   SIGNAL clk270                   : std_logic;

   SIGNAL delay_r                  : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL late_valid_r             : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL read_en_delayed_sig      : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL read_data_rise_sig       : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL read_data_fall_sig       : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);


BEGIN
   cs_io <= (OTHERS => 'Z');
   read_en_delayed  <= read_en_delayed_sig;
   read_data_rise   <= read_data_rise_sig;
   read_data_fall   <= read_data_fall_sig;

   clk180 <= NOT clk;
   clk270 <= NOT clk90;


   -- -----------------------------------------------------------------------------
   -- Assign data outputs per byte
   -- -----------------------------------------------------------------------------
   read_byte_insts : FOR K IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
   BEGIN
      -- ------------------------------
      -- even bytes (0, 2, 4, ...)
      -- ------------------------------
      PROCESS (late_valid_r, read_enable_rise_r, read_enable_rise, delay_r, rd_data_rise_r1, rd_data_rise, rd_data_fall_r1, rd_data_fall)
      BEGIN
         IF ( late_valid_r(K) = '1' ) THEN
            read_en_delayed_sig(2*K) <= read_enable_rise_r(K);
         ELSE
            read_en_delayed_sig(2*K) <= read_enable_rise(K);
         END IF;
         --
         IF ( delay_r(2*K) = '1' ) THEN
            read_data_rise_sig((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH) <= rd_data_rise_r1((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH);
            read_data_fall_sig((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH) <= rd_data_fall_r1((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH);
         ELSE
            read_data_rise_sig((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH) <= rd_data_rise((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH);
            read_data_fall_sig((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH) <= rd_data_fall((2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH);
         END IF;
      END PROCESS;

      -- ------------------------------
      -- odd bytes  (1, 3, 5, ...)
      -- ------------------------------
      PROCESS (read_en_delayed_sig, delay_r, rd_data_rise_r1, rd_data_rise, rd_data_fall_r1, rd_data_fall)
      BEGIN
         read_en_delayed_sig(2*K+1) <= read_en_delayed_sig(2*K);
         --
         IF ( delay_r(2*K+1) = '1' ) THEN
            read_data_rise_sig((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH) <= rd_data_rise_r1((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH);
            read_data_fall_sig((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH) <= rd_data_fall_r1((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH);
         ELSE
            read_data_rise_sig((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH) <= rd_data_rise((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH);
            read_data_fall_sig((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH) <= rd_data_fall((2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH);
         END IF;
      END PROCESS;
   END GENERATE;


   -- to be used with Chipscope for checking output of ISERDES (DEBUG)   
   cs_io(8 DOWNTO 0)   <= rd_data_rise(8 DOWNTO 0);     -- 1st byte
   cs_io(17 DOWNTO 9)  <= rd_data_fall(8 DOWNTO 0);
   --
   cs_io(26 DOWNTO 18) <= rd_data_rise(17 DOWNTO 9);    -- 2nd byte
   cs_io(35 DOWNTO 27) <= rd_data_fall(17 DOWNTO 9);
   --
   cs_io(44 DOWNTO 36) <= rd_data_rise(26 DOWNTO 18);   -- 3rd byte
   cs_io(53 DOWNTO 45) <= rd_data_fall(26 DOWNTO 18);
   --
   cs_io(62 DOWNTO 54) <= rd_data_rise(35 DOWNTO 27);   -- 4th byte
   cs_io(71 DOWNTO 63) <= rd_data_fall(35 DOWNTO 27);


   -- -----------------------------------------------------------------------------
   -- For controller to stop (dummy reads) waiting for calibration to finish...
   --   transfer clock domains from clk270 to clk0
   -- -----------------------------------------------------------------------------
   PROCESS (clk180)
   BEGIN
      IF RISING_EDGE( clk180 ) THEN
         -- from CLK270 to CLK180 before transfer to CLK0
         IF (rstHard_180 = '1') THEN
            SEL_DONE_a1 <= '0';
         ELSE
            SEL_DONE_a1 <= AND_REDUCE(data_tap_select(2*NUM_OF_DEVS-1 DOWNTO 0));
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         -- from CLK180 to CLK0
         IF (rstHard = '1') THEN
            SEL_DONE_a2 <= '0';
         ELSE
            SEL_DONE_a2 <= SEL_DONE_a1;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         IF (rstHard = '1') THEN
            SEL_DONE <= '0';
         ELSE
            SEL_DONE <= SEL_DONE_a2;
         END IF;
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- May need to delay data by one clock to line up both bytes with the enable
   --   Mux used to select between either normal or registered output
   -- -----------------------------------------------------------------------------	
   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         rd_data_rise_r1 <= rd_data_rise;
         rd_data_fall_r1 <= rd_data_fall;
      END IF;
   END PROCESS;

   qvld_r_insts : FOR K IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
   BEGIN
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               read_enable_rise_r(K) <= '0';
            ELSE
               read_enable_rise_r(K) <= read_enable_rise(K);
            END IF;
         END IF;
      END PROCESS;
   END GENERATE;

   -- Debug the QVLD alignment logic
   --cs_io(79 DOWNTO 72) <= delay_select;
   --cs_io(86 DOWNTO 81) <= '0' & late_valid_r & delay_r;


   -- want to register the result of camparisons for better timing
   delay_insts : FOR K IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
   BEGIN

      -- Two bytes share the same QVLD, and both bytes can be misaligned by 1 clock cycle
      -- If byte early or the byte sharing valid with is aligned and I am late, delay me one clock

      -- Even Bytes
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               delay_r(2*K) <= '0';
            ELSE
               IF ( delay_select(4*K+1 DOWNTO 4*K) = EARLY OR 
                   (delay_select(4*K+1 DOWNTO 4*K) = ALIGNED AND
                    delay_select(4*K+3 DOWNTO 4*K+2) = LATE) ) THEN
                  delay_r(2*K) <= '1';
               ELSE
                  delay_r(2*K) <= '0';
               END IF;
            END IF;
         END IF;
      END PROCESS;

      -- Odd Bytes
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               delay_r(2*K+1) <= '0';
            ELSE
               IF ( delay_select(4*K+3 DOWNTO 4*K+2) = EARLY OR 
                   (delay_select(4*K+3 DOWNTO 4*K+2) = ALIGNED AND
                    delay_select(4*K+1 DOWNTO 4*K) = LATE) ) THEN
                  delay_r(2*K+1) <= '1';
               ELSE
                  delay_r(2*K+1) <= '0';
               END IF;
            END IF;
         END IF;
      END PROCESS;

      -- Determine which QVLD to use: Output of ISERDES, or 1 clk registered version
      -- If either byte is late then use the registered version, else use ISERDES output
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               late_valid_r(K) <= '0';
            ELSE
               IF ( delay_select(4*K+1 DOWNTO 4*K) = LATE OR 
                    delay_select(4*K+3 DOWNTO 4*K+2) = LATE ) THEN
                  late_valid_r(K) <= '1';
               ELSE
                  late_valid_r(K) <= '0';
               END IF;
            END IF;
         END IF;
      END PROCESS;

   END GENERATE;


   ---------------------------------------------------------------------
   -- data_write instantiation
   ---------------------------------------------------------------------
   data_write0 : rld_data_write
   GENERIC MAP (
      RL_DQ_WIDTH     => RL_DQ_WIDTH,
      DEV_DQ_WIDTH    => DEV_DQ_WIDTH,
      NUM_OF_DEVS     => NUM_OF_DEVS,
      NUM_OF_DKS      => NUM_OF_DKS,
      QK_DATA_WIDTH   => QK_DATA_WIDTH,
      DEV_AD_WIDTH    => DEV_AD_WIDTH,
      DEV_BA_WIDTH    => DEV_BA_WIDTH,
      APP_AD_WIDTH    => APP_AD_WIDTH,
      RL_IO_TYPE      => RL_IO_TYPE,
      DEVICE_ARCH     => DEVICE_ARCH,
      CAPTURE_METHOD  => CAPTURE_METHOD
   )
   PORT MAP (
      clk             => clk,
      clk90           => clk90,
      rstHard         => rstHard,
      rstHard_90      => rstHard_90,
      rstHard_180     => rstHard_180,
      rstHard_270     => rstHard_270,

      ctlIoWdEn       => ctlIoWdEn,
      wdfDM           => wdfDM,
      wdfD            => wdfD,
      wr_data_rise    => wr_data_rise,
      wr_data_fall    => wr_data_fall,
      wr_en_clk90     => wr_en_clk90,
      data_mask_rise  => data_mask_rise,
      data_mask_fall  => data_mask_fall
   );


   ---------------------------------------------------------------------
   -- tap_logic instantiation
   ---------------------------------------------------------------------
   taplogic0 : rld_tap_logic 
   GENERIC MAP (
      RL_DQ_WIDTH     => RL_DQ_WIDTH,
      DEV_DQ_WIDTH    => DEV_DQ_WIDTH,
      NUM_OF_DEVS     => NUM_OF_DEVS,
      NUM_OF_DKS      => NUM_OF_DKS,
      QK_DATA_WIDTH   => QK_DATA_WIDTH,
      DEV_AD_WIDTH    => DEV_AD_WIDTH,
      DEV_BA_WIDTH    => DEV_BA_WIDTH,
      APP_AD_WIDTH    => APP_AD_WIDTH,
      RL_CK_PERIOD    => RL_CK_PERIOD,
      RL_IO_TYPE      => RL_IO_TYPE,
      DEVICE_ARCH     => DEVICE_ARCH,
      CAPTURE_METHOD  => CAPTURE_METHOD
   )
   PORT MAP (
      clk                   => clk,
      clk90                 => clk90,
      refClk                => refClk,
      rstHard               => rstHard,
      rstHard_90            => rstHard_90,
      rstHard_180           => rstHard_180,
      rstHard_270           => rstHard_270,
      rstHard_refClk        => rstHard_refClk,

      read_enable_rise      => read_enable_rise,
      read_enable_fall      => read_enable_fall,
      read_enable_rise_r    => read_enable_rise_r,
      CTRL_DUMMYREAD_START  => CTRL_DUMMYREAD_START,
      refresh_cmd           => refresh_cmd,
      issueCalibration      => issueCalibration,

      qk_idelay_inc         => qk_idelay_inc,
      qk_idelay_ce          => qk_idelay_ce,
      qk_idelay_rst         => qk_idelay_rst,

      data_idelay_inc       => data_idelay_inc,
      data_idelay_ce        => data_idelay_ce,
      data_idelay_rst       => data_idelay_rst,

      qvld_idelay_inc       => qvld_idelay_inc,
      qvld_idelay_ce        => qvld_idelay_ce,
      qvld_idelay_rst       => qvld_idelay_rst,

      read_enable_out       => read_enable_out,
      delay_select          => delay_select,

      DQ_tap_count          => DQ_tap_count,

      first_stage_done      => first_stage_done,
      second_stage_begin    => second_stage_begin,
      qvld_cal              => qvld_cal,
      data_tap_select       => data_tap_select,
      idelay_ctrl_rdy_r1    => idelay_ctrl_rdy,

      rd_data_rise          => rd_data_rise,
      rd_data_fall          => rd_data_fall,

      cs_io                 => cs_io
   );


END ARCHITECTURE rtl;
