--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_data_path_iobs.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	 Virtex-5
--	Purpose: This module instantiates the iob modules for QK, DQ, QVLD and DM
--
--*****************************************************************************

library	IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library	UNISIM;
use UNISIM.VCOMPONENTS.ALL;

ENTITY rld_data_path_iobs IS
   GENERIC (
      RL_DQ_WIDTH                    :  integer := 36;    
      DEV_DQ_WIDTH                   :  integer := 18;      --  data width of the memory device
      NUM_OF_DEVS                    :  integer := 2;       --  number of memory devices
      NUM_OF_DKS                     :  integer := 4;     
      DEV_BYTE_WIDTH                 :  integer := 9;       --  the width of a "byte" on the memory device (x8 or x9)   
      DEV_AD_WIDTH                   :  integer := 20;      --  address width of the memory device
      DEV_BA_WIDTH                   :  integer := 3;       --  bank address width of the memory device
      --
      RL_IO_TYPE                     :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH                    :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD                 :  std_logic_vector(1 DOWNTO 0) := "00"    --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk                     : IN std_logic;   
      clk90                   : IN std_logic;   
      refClk                  : IN std_logic;   
      rstHard                 : IN std_logic;   
      rstHard_90              : IN std_logic;   
      rstHard_180             : IN std_logic;   
      rstHard_270             : IN std_logic;   
      phy_qk_p                : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);  -- QK, read data clock
      phy_qk_n                : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);  -- QK#  
      phy_dm                  : OUT std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);     -- DM 
      phy_dq                  : INOUT std_logic_vector(RL_DQ_WIDTH - 1 DOWNTO 0);   -- DQ, read and write data bus 
      phy_qvld                : IN std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);      -- QVLD  
      qk_idelay_inc           : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);  -- IDELAY controls for QK (read clock)
      qk_idelay_ce            : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);   
      qk_idelay_rst           : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);  
      data_idelay_inc         : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);  -- IDELAY controls for DQ (read data)
      data_idelay_ce          : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);   
      data_idelay_rst         : IN std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0); 
      qvld_idelay_inc         : IN std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);      -- IDELAY controls for QVLD (read enable)   
      qvld_idelay_ce          : IN std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);   
      qvld_idelay_rst         : IN std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);  
      wr_data_rise            : IN std_logic_vector(RL_DQ_WIDTH - 1 DOWNTO 0);   
      wr_data_fall            : IN std_logic_vector(RL_DQ_WIDTH - 1 DOWNTO 0);   
      wr_en_clk90             : IN std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0); 
      data_mask_rise          : IN std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);   
      data_mask_fall          : IN std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);   
      rd_data_rise            : OUT std_logic_vector(RL_DQ_WIDTH - 1 DOWNTO 0);   
      rd_data_fall            : OUT std_logic_vector(RL_DQ_WIDTH - 1 DOWNTO 0);   
      read_enable_rise        : OUT std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0);   
      read_enable_fall        : OUT std_logic_vector(NUM_OF_DEVS - 1 DOWNTO 0)
   );   
END ENTITY rld_data_path_iobs;

ARCHITECTURE rtl OF rld_data_path_iobs IS

-- Conditional Constant, based on one "equal to" condition
   function CC (COND_EXPR:INTEGER; COND_VAL:INTEGER; THEN_VAL:INTEGER; ELSE_VAL:INTEGER) return INTEGER is 
   begin 
      if COND_EXPR = COND_VAL then
         return THEN_VAL; 
      else
         return ELSE_VAL; 
      end if; 
   end;

   -- Conditional Constant, based on two "less than" conditions
   function CC_LT2 (COND_EXPR1:INTEGER; COND_VAL1:INTEGER; THEN_VAL1:INTEGER; 
                    COND_EXPR2:INTEGER; COND_VAL2:INTEGER; THEN_VAL2:INTEGER; 
                    ELSE_VAL:INTEGER) return INTEGER is 
   begin
      if COND_EXPR1 < COND_VAL1 then
         return THEN_VAL1;
      else
         if COND_EXPR2 < COND_VAL2 then
           return THEN_VAL2;
         else
            return ELSE_VAL;
         end if;
      end if;
   end;
   
   CONSTANT QK_DATA_WIDTH           : integer := CC(DEV_DQ_WIDTH, 9, DEV_DQ_WIDTH, DEV_DQ_WIDTH/2);

   -- -----------------------------------------------------------------------------
   -- Component Declarations
   -- -----------------------------------------------------------------------------
   COMPONENT rld_v4_qk_iob
   GENERIC (
      RL_IO_TYPE              :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH             :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD          :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk                     : IN std_logic;   
      refClk                  : IN std_logic;
      dlyinc                  : IN std_logic;   
      dlyce                   : IN std_logic;   
      dlyrst                  : IN std_logic;   
      phy_qk_p                : IN std_logic;
      phy_qk_n                : IN std_logic;
      rise_capture_qk_level   : OUT std_logic;   
      fall_capture_qk_level   : OUT std_logic
   );   
   END COMPONENT;

   COMPONENT rld_v5_qk_iob
   GENERIC (
      RL_IO_TYPE              :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH             :  std_logic_vector(1 DOWNTO 0) := "01";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD          :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk270                  : IN std_logic;   
      dlyinc                  : IN std_logic;   
      dlyrst                  : IN std_logic;   
      dlyce                   : IN std_logic;   
      phy_qk_p                : IN std_logic;   
      phy_qk_n                : IN std_logic;   
      phy_qk_bufio            : OUT std_logic
   );   
   END COMPONENT;

   COMPONENT rld_v4_dq_iob
   GENERIC (
      RL_IO_TYPE              :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH             :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD          :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk                     : IN std_logic;   
      clk90                   : IN std_logic;   
      refClk                  : IN std_logic;
      rstHard                 : IN std_logic;   
      rstHard_90              : IN std_logic;   
      rstHard_180             : IN std_logic;   
      rstHard_270             : IN std_logic;   
      dlyinc                  : IN std_logic;   
      dlyce                   : IN std_logic;   
      dlyrst                  : IN std_logic;   
      write_data_rise         : IN std_logic;   
      write_data_fall         : IN std_logic;   
      ctrl_wren               : IN std_logic;   
      phy_dq                  : INOUT std_logic;
      read_data_rise          : OUT std_logic;   
      read_data_fall          : OUT std_logic
   );   
   END COMPONENT;
   
   COMPONENT rld_v5_dq_iob
   GENERIC (
      RL_IO_TYPE              :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH             :  std_logic_vector(1 DOWNTO 0) := "01";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD          :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk270                  : IN std_logic;   
      bufio_clk               : IN std_logic;   
      reset                   : IN std_logic;   
      dlyinc                  : IN std_logic;
      dlyce                   : IN std_logic;   
      dlyrst                  : IN std_logic;   
      write_data_rise         : IN std_logic;   
      write_data_fall         : IN std_logic;   
      ctrl_wren               : IN std_logic;   
      phy_dq                  : INOUT std_logic;
      read_data_rise          : OUT std_logic;   
      read_data_fall          : OUT std_logic
   );   
   END COMPONENT;
   
   COMPONENT rld_qvld_iob
   GENERIC (
      RL_IO_TYPE              :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH             :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD          :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk                     : IN std_logic;   
      refClk                  : IN std_logic;   
      clk270                  : IN std_logic;   
      bufio_clk               : IN std_logic;   
      reset                   : IN std_logic;   
      dlyinc                  : IN std_logic;   
      dlyce                   : IN std_logic;   
      dlyrst                  : IN std_logic;   
      phy_qvld                : IN std_logic;
      -- Virtex-4 interface

      first_rising_byte0      : IN std_logic;   
      qvld_delayed_rise_byte0 : OUT std_logic;   
      qvld_delayed_fall_byte0 : OUT std_logic;   
      first_rising_byte1      : IN std_logic;   
      qvld_delayed_rise_byte1 : OUT std_logic;   
      qvld_delayed_fall_byte1 : OUT std_logic;   
      -- Virtex-5 interface

--      qvld_out                : OUT std_logic;

      read_enable_rise        : OUT std_logic;   
      read_enable_fall        : OUT std_logic
   );   
   END COMPONENT;

   COMPONENT rld_v4_dm_iob
   GENERIC (
      RL_IO_TYPE              :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH             :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD          :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk                     : IN std_logic;   
      clk90                   : IN std_logic;   
      rstHard                 : IN std_logic;   
      rstHard_90              : IN std_logic;   
      rstHard_180             : IN std_logic;   
      rstHard_270             : IN std_logic;   
      mask_data_rise          : IN std_logic;   
      mask_data_fall          : IN std_logic;   
      phy_dm                  : OUT std_logic
   );
   END COMPONENT;
   
   COMPONENT rld_v5_dm_iob
   GENERIC (
      RL_IO_TYPE              :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH             :  std_logic_vector(1 DOWNTO 0) := "01";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD          :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk270                  : IN std_logic;   
      reset                   : IN std_logic;   
      mask_data_rise          : IN std_logic;   
      mask_data_fall          : IN std_logic;   
      phy_dm                  : OUT std_logic
   );
   END COMPONENT;
   -- -----------------------------------------------------------------------------
   -- end of Component Declarations
   -- -----------------------------------------------------------------------------
   
   -- Virtex-4 specific
   SIGNAL qk_delayed               :  std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);   
   SIGNAL qk_delayed_unused        :  std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);   
   SIGNAL rising_first             :  std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);   
   SIGNAL qvld_delayed_rise        :  std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);   
   SIGNAL qvld_delayed_fall        :  std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);
      
   -- Virtex-5 specific
   SIGNAL qk_bufio                 :  std_logic_vector(2 * NUM_OF_DEVS - 1 DOWNTO 0);   
   SIGNAL clk180                   :  std_logic;   
   SIGNAL clk270                   :  std_logic;   
   SIGNAL bit_0                    :  std_logic;   --  constant
   SIGNAL bit_1                    :  std_logic;   
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;        
   SIGNAL i_dm                     :  std_logic;   
   SIGNAL i_dq                     :  std_logic;   
   SIGNAL i_qk                     :  std_logic;   
   SIGNAL i_qvld                   :  std_logic;   

BEGIN
   bit_0             <= '0' ;
   bit_1             <= '1' ;
   vcc               <= '1' ;
   gnd               <= '0' ;
   clk180            <= NOT clk ;
   clk270            <= NOT clk90 ;
   
   -- -----------------------------------------------------------------------------
   -- Generate QK/QK_N instances
   --   Data capture is done with the system clock (clk)
   --   Tap calibration runs with the reference clock (refClk)
   -- -----------------------------------------------------------------------------
   qk_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      qk_bit_insts : FOR i_qk IN  2*NUM_OF_DEVS-1 DOWNTO 0  GENERATE
      BEGIN
         qk : rld_v4_qk_iob
         GENERIC MAP (
            RL_IO_TYPE     => RL_IO_TYPE,
            DEVICE_ARCH    => DEVICE_ARCH,
            CAPTURE_METHOD => CAPTURE_METHOD
         ) 
         PORT MAP (
            clk                     => clk,
            refClk                  => refClk,
            dlyinc                  => qk_idelay_inc(i_qk),
            dlyce                   => qk_idelay_ce(i_qk),
            dlyrst                  => qk_idelay_rst(i_qk),
            phy_qk_p                => phy_qk_p(i_qk),
            phy_qk_n                => phy_qk_n(i_qk),
            rise_capture_qk_level   => qk_delayed(i_qk),
            fall_capture_qk_level   => qk_delayed_unused(i_qk)-- unused for now...
         );
      END GENERATE qk_bit_insts;
   END GENERATE qk_virtex4_insts;
   
   qk_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      qk_bit_insts : FOR i_qk IN  2*NUM_OF_DEVS-1 DOWNTO 0  GENERATE
      BEGIN
         qk : rld_v5_qk_iob 
         PORT MAP (
            clk270         => clk270,
            dlyinc         => qk_idelay_inc(i_qk),
            dlyrst         => qk_idelay_rst(i_qk),
            dlyce          => qk_idelay_ce(i_qk),
            phy_qk_p       => phy_qk_p(i_qk),
            phy_qk_n       => phy_qk_n(i_qk),
            phy_qk_bufio   => qk_bufio(i_qk) -- connect to DQ IOBs
         );
      END GENERATE qk_bit_insts;   
   END GENERATE qk_virtex5_insts;
   
   
   -- -----------------------------------------------------------------------------
   -- DQ instances
   -- -----------------------------------------------------------------------------
   dq_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      dq_bit_insts : FOR i_dq IN  RL_DQ_WIDTH-1 DOWNTO 0  GENERATE
      BEGIN
         dq : rld_v4_dq_iob
         GENERIC MAP (
            RL_IO_TYPE     => RL_IO_TYPE,
            DEVICE_ARCH    => DEVICE_ARCH,
            CAPTURE_METHOD => CAPTURE_METHOD
         ) 
         PORT MAP (
            clk               => clk,
            clk90             => clk90,
            refClk            => refClk,
            rstHard           => rstHard,
            rstHard_90        => rstHard_90,
            rstHard_180       => rstHard_180,
            rstHard_270       => rstHard_270,
            dlyinc            => data_idelay_inc((i_dq / QK_DATA_WIDTH)),
            dlyce             => data_idelay_ce((i_dq / QK_DATA_WIDTH)),
            dlyrst            => data_idelay_rst((i_dq / QK_DATA_WIDTH)),
            write_data_rise   => wr_data_rise(i_dq),
            write_data_fall   => wr_data_fall(i_dq),
            ctrl_wren         => wr_en_clk90((i_dq / DEV_DQ_WIDTH)),-- was... wr_en_clk90
            phy_dq            => phy_dq(i_dq),
            read_data_rise    => rd_data_rise(i_dq),
            read_data_fall    => rd_data_fall(i_dq)
         );   
      END GENERATE dq_bit_insts;
   END GENERATE dq_virtex4_insts;

   dq_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      dq_bit_insts : FOR i_dq IN  RL_DQ_WIDTH-1 DOWNTO 0  GENERATE
      BEGIN
         dq : rld_v5_dq_iob 
         PORT MAP (
            clk270            => clk270,
            bufio_clk         => qk_bufio((i_dq / QK_DATA_WIDTH)),
            reset             => rstHard_270,
            dlyinc            => data_idelay_inc((i_dq / QK_DATA_WIDTH)),
            dlyce             => data_idelay_ce((i_dq / QK_DATA_WIDTH)),
            dlyrst            => data_idelay_rst((i_dq / QK_DATA_WIDTH)),
            write_data_rise   => wr_data_rise(i_dq),
            write_data_fall   => wr_data_fall(i_dq),
            ctrl_wren         => wr_en_clk90((i_dq / DEV_DQ_WIDTH)),
            phy_dq            => phy_dq(i_dq),
            read_data_rise    => rd_data_rise(i_dq),
            read_data_fall    => rd_data_fall(i_dq)
         );   
      END GENERATE dq_bit_insts;
   END GENERATE dq_virtex5_insts;
   
   
   -- -----------------------------------------------------------------------------
   -- QVLD used for data read enable
   --   - "data valid" input, captured similarly as read data DQ
   --   - For x9 device, QVLD for byte 0 is physically near QK0/QK0#
   --   - For x18 & x36 devices, QVLD for byte 0 and 1 is physically near QK0/QK0#
   --
   --   QVLD "enable"  is produced by falling edge QK/QK# "before" the data DQ
   --   QVLD "disable" is produced by falling edge QK/QK# "before" the data DQ
   -- -----------------------------------------------------------------------------
   qvld_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      qvld_b_insts : FOR i_qvld IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
      BEGIN
         qvld_b : rld_qvld_iob
         GENERIC MAP (
            RL_IO_TYPE     => RL_IO_TYPE,
            DEVICE_ARCH    => DEVICE_ARCH,
            CAPTURE_METHOD => CAPTURE_METHOD
         ) 
         PORT MAP (
            clk                     => clk,
            refClk                  => refClk,
            clk270                  => '0',
            bufio_clk               => '0',
            reset                   => '0',
            dlyinc                  => data_idelay_inc((i_qvld*2)),
            dlyce                   => data_idelay_ce((i_qvld*2)),
            dlyrst                  => data_idelay_rst((i_qvld*2)),
            phy_qvld                => phy_qvld(i_qvld),
            first_rising_byte0      => rising_first((i_qvld*2)),
            qvld_delayed_rise_byte0 => qvld_delayed_rise((i_qvld*2)),
            qvld_delayed_fall_byte0 => qvld_delayed_fall((i_qvld*2)),
            first_rising_byte1      => rising_first((i_qvld*2+1)),
            qvld_delayed_rise_byte1 => qvld_delayed_rise((i_qvld*2+1)),
            qvld_delayed_fall_byte1 => qvld_delayed_fall((i_qvld*2+1))
--            qvld_out                => open
         );   
      END GENERATE qvld_b_insts;
   END GENERATE qvld_virtex4_insts;
   
   qvld_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      qvld_b_insts : FOR i_qvld IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
      BEGIN
         qvld_b : rld_qvld_iob
         GENERIC MAP (
            RL_IO_TYPE     => RL_IO_TYPE,
            DEVICE_ARCH    => DEVICE_ARCH,
            CAPTURE_METHOD => CAPTURE_METHOD
         ) 
         PORT MAP (
            clk                     => '0',
            refClk                  => '0',
            clk270                  => clk270,
            bufio_clk               => qk_bufio((i_qvld*2)),   --want to use 0 or 2 for Bufio clock
            reset                   => rstHard_270,
--            dlyinc                  => qvld_idelay_inc[i_qvld],
--            dlyce                   => qvld_idelay_ce[i_qvld]),
--            dlyrst                  => qvld_idelay_rst[i_qvld]),            
            dlyinc                  => data_idelay_inc((i_qvld*2)),
            dlyce                   => data_idelay_ce((i_qvld*2)),
            dlyrst                  => data_idelay_rst((i_qvld*2)),
            phy_qvld                => phy_qvld(i_qvld),
            first_rising_byte0      => '0',
            qvld_delayed_rise_byte0 => open,
            qvld_delayed_fall_byte0 => open,
            first_rising_byte1      => '0',
            qvld_delayed_rise_byte1 => open,
            qvld_delayed_fall_byte1 => open,
            read_enable_rise        => read_enable_rise(i_qvld),
            read_enable_fall        => read_enable_fall(i_qvld)
         );
      END GENERATE qvld_b_insts;
   END GENERATE qvld_virtex5_insts;   
   
   
   -- -----------------------------------------------------------------------------
   -- DM instances
   -- -----------------------------------------------------------------------------
   dm_virtex4_insts : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
      dm_bit_insts : FOR i_dm IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
      BEGIN
         dm : rld_v4_dm_iob
         GENERIC MAP (
            RL_IO_TYPE     => RL_IO_TYPE,
            DEVICE_ARCH    => DEVICE_ARCH,
            CAPTURE_METHOD => CAPTURE_METHOD
         )  
         PORT MAP (
            clk            => clk,
            clk90          => clk90,
            rstHard        => rstHard,
            rstHard_90     => rstHard_90,
            rstHard_180    => rstHard_180,
            rstHard_270    => rstHard_270,
            mask_data_rise => data_mask_rise(i_dm),
            mask_data_fall => data_mask_fall(i_dm),
            phy_dm         => phy_dm(i_dm)
         );   
      END GENERATE dm_bit_insts;
   END GENERATE dm_virtex4_insts;
   
   dm_virtex5_insts : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
      dm_bit_insts : FOR i_dm IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
      BEGIN
         dm : rld_v5_dm_iob 
         PORT MAP (
            clk270         => clk270,
            reset          => rstHard_270,
            mask_data_rise => data_mask_rise(i_dm),
            mask_data_fall => data_mask_fall(i_dm),
            phy_dm         => phy_dm(i_dm)
         );
      END GENERATE dm_bit_insts;
   END GENERATE dm_virtex5_insts;   
   

END ARCHITECTURE rtl;
