--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_data_write.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This module generates the write data and mask bits that go
--		to the Write Data Fifos.  
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_data_write IS
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    -- number of memory devices
      NUM_OF_DKS                     : integer := 4;
      QK_DATA_WIDTH                  : integer := 9;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE;   -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      ctlIoWdEn               : IN  std_logic;
      wdfDM                   : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      wdfD                    : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      wr_data_rise            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_fall            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_en_clk90             : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_rise          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_fall          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0)
   );
END ENTITY rld_data_write;



ARCHITECTURE rtl OF rld_data_write IS


   --(* syn_keep = "TRUE" *)(* syn_preserve = "TRUE" *)  reg  [NUM_OF_DEVS-1:0]       ctlIoWdEn_d;
--   SIGNAL ctlIoWdEn_d2  : std_logic;
--   SIGNAL ctlIoWdEn_d1  : std_logic;
   -- internal wires
   SIGNAL ctlIoWdEn_d   : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL bit_0   : std_logic;  -- constants
   SIGNAL bit_1   : std_logic;
   SIGNAL vcc     : std_logic;
   SIGNAL gnd     : std_logic;

   SIGNAL clk180  : std_logic;
   SIGNAL clk270  : std_logic;

   SIGNAL wr_en_clk90_sig    : std_logic_vector(wr_en_clk90'LENGTH-1 DOWNTO 0);
   SIGNAL data_mask_rise_sig : std_logic_vector(data_mask_rise'LENGTH-1 DOWNTO 0);
   SIGNAL data_mask_fall_sig : std_logic_vector(data_mask_fall'LENGTH-1 DOWNTO 0);

BEGIN
   wr_en_clk90    <= wr_en_clk90_sig;
   data_mask_rise <= data_mask_rise_sig;
   data_mask_fall <= data_mask_fall_sig;

   bit_0 <= '0';
   bit_1 <= '1';

   vcc <= '1';
   gnd <= '0';

   clk180 <= NOT clk;
   clk270 <= NOT clk90;


   -- -----------------------------------------------------------------------------
   -- Adjust the "tri-state control for DQ" timings to match the "command" timings
   --    (a) one flop @clk to grab data from the controller (decoupling)
   --
   -- The data_path module will transfer "tri-state control for DQ" from
   -- rising CLK0 to "falling CLK90" clock domain
   -- -----------------------------------------------------------------------------
   -- add more flops here and see if it changes the routes
   --always @( posedge clk )
   -- always @( posedge clk )
   --    begin : delay_ctlIoWdEn_d1
   --       ctlIoWdEn_d1 <= ctlIoWdEn;
   --    end
   --    always @( posedge clk )
   --    begin : delay_ctlIoWdEn_d2
   --       ctlIoWdEn_d2 <= ctlIoWdEn;
   --    end 
   delay_ctlIoWdEn : PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         ctlIoWdEn_d <= ctlIoWdEn & ctlIoWdEn;
      END IF;
   END PROCESS delay_ctlIoWdEn;


   -- -----------------------------------------------------------------------------
   -- Change clock domain for "write data" DQ =============> "falling edge CLK90"
   -- Change clock domain for "tri-state control" for DQ ==> "falling edge CLK90"
   --    This gives only 3/4 cycle from the previous flop clocked on clk,
   --    but gives a full CLK90 cycle for the OLOGIC DDR flops
   -- align DQ and its tri-state control with respect to falling edge clk90
   -- -----------------------------------------------------------------------------
   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard_270 = '1') THEN
            wr_data_rise   <= (OTHERS => '0');
            wr_data_fall   <= (OTHERS => '0');
--            wr_en_clk90(0) <= '0';
--            wr_en_clk90(1) <= '0';
            wr_en_clk90_sig    <= (OTHERS => '0');
         ELSE
            wr_data_rise   <= wdfD((2*RL_DQ_WIDTH)-1 DOWNTO RL_DQ_WIDTH);   -- first  DDR DQ to go out
            wr_data_fall   <= wdfD(RL_DQ_WIDTH-1 DOWNTO 0);                 -- second DDR DQ to go out
--            wr_en_clk90(0) <= ctlIoWdEn_d1;    -- DQ tri-state control
--            wr_en_clk90(1) <= ctlIoWdEn_d2;
            wr_en_clk90_sig    <= ctlIoWdEn_d;     -- DQ tri-state control
         END IF;
      END IF;
   END PROCESS;

   -- align DM and its tri-state control with respect to falling edge clk90
   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard_270 = '1') THEN
            data_mask_rise_sig <= (OTHERS => '0');
            data_mask_fall_sig <= (OTHERS => '0');
         ELSE
            data_mask_rise_sig <= wdfDM((2*NUM_OF_DEVS)-1 DOWNTO NUM_OF_DEVS);   -- first  DDR DM to go out
            data_mask_fall_sig <= wdfDM(NUM_OF_DEVS-1 DOWNTO 0);                 -- second DDR DM to go out
         END IF;
      END IF;
   END PROCESS;


END ARCHITECTURE rtl;
