--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_infrastructure_iobs.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	 Virtex-5
--	Purpose: This module generates the data	clock (DK) and the
--		address/command	clock (CK) for the memory physical layer. 
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Fixed issue with NUM_OF_DKS not being used to generate DKs
--*****************************************************************************

library	IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library	UNISIM;
use UNISIM.VCOMPONENTS.ALL;


ENTITY rld_infrastructure_iobs IS
   GENERIC (
      RL_DQ_WIDTH                    : integer	:= 36;   -- memory interface data width  
      DEV_DQ_WIDTH                   : integer	:= 18;    --  data width	of the memory device
      NUM_OF_DEVS                    : integer  := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer  := 4
   );
   PORT	(
      clk		      : IN  std_logic;
      clk90		      : IN  std_logic;
      phy_dk_p                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_dk_n                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_ck_p                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_ck_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0)
   );	
END ENTITY rld_infrastructure_iobs;


ARCHITECTURE rtl OF rld_infrastructure_iobs IS

--   ATTRIBUTE iostandard: STRING;
--   -- using component name, might need to details each bit of the bus independently
--   ATTRIBUTE iostandard OF phy_dk_p : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF phy_dk_n : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF phy_ck_p : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF phy_ck_n : LABEL IS "HSTL_II_18";
--   -- using label name
--   ATTRIBUTE iostandard OF ck_obufds : LABEL IS "HSTL_II_18";
--   ATTRIBUTE iostandard OF dk_obufds : LABEL IS "HSTL_II_18";


   SIGNAL dk_p          : std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
   SIGNAL ck_p          : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL vcc           : std_logic;
   SIGNAL gnd           : std_logic;


BEGIN
   vcc <= '1';
   gnd <= '0';


   -- -----------------------------------------------------------------------------
   -- DK data write clock
   -- -----------------------------------------------------------------------------
   --  - The output buffer should be differential
   --  - The I/O standard will be defined in the UCF:
   --	   - HSTL_II (standard,	Vcco=1.5V) or HSTL_II_18 (if Vcco is 1.8V)
   --	   - DCI should	be used	if no external termination exists near the FPGA	output
   --  - Virtex-4 and Virtex-5 support the following:
   --	   - if	external output	terminations exist: DIFF_HSTL_II      -or-  DIFF_HSTL_II_18
   --	   - if	no external terminations output	  : DIFF_HSTL_II_DCI  -or-  DIFF_HSTL_II_DCI_18
   -- -----------------------------------------------------------------------------
   dk_gen_inst : FOR i_dk IN  NUM_OF_DKS-1 DOWNTO 0  GENERATE 
   BEGIN
      dk_obufds : OBUFDS
      GENERIC MAP (
         IOSTANDARD => "DIFF_HSTL_I_18"
      )
      PORT MAP (
         O   => phy_dk_p(i_dk),
         OB  => phy_dk_n(i_dk),
         I   => dk_p(i_dk)
      );

      dk_oddr : ODDR
      GENERIC MAP (
         SRTYPE        => "SYNC",
         DDR_CLK_EDGE  => "OPPOSITE_EDGE"
      ) 
      PORT MAP (
         Q   => dk_p(i_dk),
         C   => clk,
         CE  => vcc,
         D1  => vcc,  -- To get same phase as original clock, needs to use D1=1, D2=0
         D2  => gnd,
         R   => gnd,
         S   => gnd
      );
   END GENERATE;


   -- -----------------------------------------------------------------------------
   -- CK address/command write clock
   --	- used of address (A, BA) and control (CS_N, WE_N, REF_N) signals
   --	- source of memory's internal DLL
   -- -----------------------------------------------------------------------------
   --  - The output buffer should be differential
   --  - The I/O standard will be defined in the UCF:
   --	   - HSTL_II (standard,	Vcco=1.5V) or HSTL_II_18 (if Vcco is 1.8V)
   --	   - DCI should	be used	if no external termination exists near the FPGA	output
   --  - Virtex-4 and Virtex-5 support the following:
   --	   - if	external output	terminations exist: DIFF_HSTL_II      -or-  DIFF_HSTL_II_18
   --	   - if	no external terminations output	  : DIFF_HSTL_II_DCI  -or-  DIFF_HSTL_II_DCI_18	   
   -- -----------------------------------------------------------------------------
   ck_gen_inst : FOR i_ck IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE 
   BEGIN
      ck_obufds : OBUFDS
      GENERIC MAP (
         IOSTANDARD => "DIFF_HSTL_I_18"
      )
      PORT MAP (
         O   => phy_ck_p(i_ck),
         OB  => phy_ck_n(i_ck),
         I   => ck_p(i_ck)
      );

      ck_oddr : ODDR
      GENERIC MAP (
         SRTYPE        => "SYNC",
         DDR_CLK_EDGE  => "OPPOSITE_EDGE"
      )
      PORT MAP (
         Q   => ck_p(i_ck),
         C   => clk,
         CE  => vcc,
         D1  => vcc, -- To get same phase as original clock, needs to use D1=1, D2=0
         D2  => gnd,
         R   => gnd,
         S   => gnd
      );
   END GENERATE;


END ARCHITECTURE rtl;
