--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_infrastructure_top.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This module instantiates clock and reset modules for the design
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_infrastructure_top IS
   GENERIC (
      SIMULATION_ONLY                : boolean := FALSE;    -- if TRUE, it shortens the wait time
      RL_DQ_WIDTH                    : integer := 36;       -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;       --  data width of the memory device
      NUM_OF_DEVS                    : integer := 2;        -- number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;       -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;        -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE;    -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      sysClk_p                : IN  std_logic;
      sysClk_n                : IN  std_logic;
      refClk_p                : IN  std_logic;
      refClk_n                : IN  std_logic;
      sysReset                : IN  std_logic;
      confMReg                : IN  std_logic_vector(17 DOWNTO 0);
      issueMRS                : IN  std_logic;
      calibration_done        : IN  std_logic;   -- QK and DQ calibrations are completed
      sysClk                  : OUT std_logic;
      clk                     : OUT std_logic;
      clk90                   : OUT std_logic;
      refClk                  : OUT std_logic;
      clk33                   : IN  std_logic;
      dcmlocked               : OUT std_logic;
      sync_system_reset       : OUT std_logic;
      rstHard                 : OUT std_logic;
      rstHard_90              : OUT std_logic;
      rstHard_180             : OUT std_logic;
      rstHard_270             : OUT std_logic;
      rstHard_refClk          : OUT std_logic;
      rstConfig               : OUT std_logic;
      Init_Done               : OUT std_logic;
      Init_Done_270           : OUT std_logic;
      mrsDone_o               : OUT std_logic;
      rstCmd                  : OUT std_logic_vector(3 DOWNTO 0);
      rstAd                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0)
   );
END ENTITY rld_infrastructure_top;



ARCHITECTURE rtl OF rld_infrastructure_top IS


   COMPONENT rld_clk_module
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;    -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_AD_WIDTH            : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;    -- bank address width of the memory device
      DUPLICATE_CONTROLS      : boolean := FALSE;   -- Duplicate the ports for A, BA, WE# and REF#
      --
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      sysClk_p   : IN  std_logic;
      sysClk_n   : IN  std_logic;
      refClk_p   : IN  std_logic;
      refClk_n   : IN  std_logic;
      clk33      : IN  std_logic;
      sysReset   : IN  std_logic;
      clk        : OUT std_logic;
      clk90      : OUT std_logic;
      sysClk     : OUT std_logic;
      refClk     : OUT std_logic;
      locked     : OUT std_logic
   );
   END COMPONENT;


   COMPONENT rld_rst
   GENERIC (
      SIMULATION_ONLY         : boolean := FALSE;  -- if TRUE, it shortens the wait time
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      clk180                  : IN  std_logic;
      clk270                  : IN  std_logic;
      refClk                  : IN  std_logic;
      sysReset                : IN  std_logic;
      confMReg                : IN  std_logic_vector(17 DOWNTO 0);
      dcmLocked               : IN  std_logic;
      issueMRS                : IN  std_logic;
      calibration_done        : IN  std_logic;   -- QK and DQ calibrations are completed
      sync_system_reset       : OUT std_logic;
      rstHard                 : OUT std_logic;
      rstHard_90              : OUT std_logic;
      rstHard_180             : OUT std_logic;
      rstHard_270             : OUT std_logic;
      rstHard_refClk          : OUT std_logic;
      rstConfig               : OUT std_logic;
      Init_Done               : OUT std_logic;
      Init_Done_270           : OUT std_logic;
      mrsDone_o               : OUT std_logic;
      rstCmd                  : OUT std_logic_vector(3 DOWNTO 0);
      rstAd                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0)
   );
   END COMPONENT;


   -- signals
   SIGNAL clk180                   : std_logic;
   SIGNAL clk270                   : std_logic;
   SIGNAL not_sysReset             : std_logic;

   SIGNAL sysClk_sig               : std_logic;
   SIGNAL clk_sig                  : std_logic;
   SIGNAL clk90_sig                : std_logic;
   SIGNAL refClk_sig               : std_logic;
   SIGNAL dcmlocked_sig            : std_logic;

   SIGNAL rstCmd_int               : std_logic_vector(3 DOWNTO 0);
   SIGNAL rstAd_int                : std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
   SIGNAL rstBa_int                : std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);


BEGIN
   not_sysReset      <= NOT sysReset;

   sysClk            <= sysClk_sig;
   clk               <= clk_sig;
   clk90             <= clk90_sig;
   refClk            <= refClk_sig;
   dcmlocked         <= dcmlocked_sig;

   rstCmd            <= rstCmd_int;
   rstAd             <= rstAd_int;
   rstBa             <= rstBa_int;

   clk180            <= NOT clk_sig;
   clk270            <= NOT clk90_sig;


   -- -----------------------------------------------------------------------------
   -- Clock module: get differential clock input (sysClk and refClk)
   --    - instantiates DCM_BASE primitive for memory interface
   --    - handles 200 MHz clock for the IDELAYCTRL module
   -- -----------------------------------------------------------------------------
   clk_module0 : rld_clk_module
   GENERIC MAP (
      RL_DQ_WIDTH         => RL_DQ_WIDTH,
      DEV_DQ_WIDTH        => DEV_DQ_WIDTH,
      NUM_OF_DEVS         => NUM_OF_DEVS,
      NUM_OF_DKS          => NUM_OF_DKS,   
      DEV_AD_WIDTH        => DEV_AD_WIDTH,
      DEV_BA_WIDTH        => DEV_BA_WIDTH,
      DUPLICATE_CONTROLS  => DUPLICATE_CONTROLS,
      RL_IO_TYPE          => RL_IO_TYPE,
      DEVICE_ARCH         => DEVICE_ARCH,
      CAPTURE_METHOD      => CAPTURE_METHOD
   )
   PORT MAP (
      sysClk_p  => sysClk_p,       -- differential system clock input
      sysClk_n  => sysClk_n,
      refClk_p  => refClk_p,       -- differential reference clock input
      refClk_n  => refClk_n,
      clk33     => clk33,
      sysReset  => not_sysReset,
      clk       => clk_sig,        -- Global clock to all modules, output from DCM
      clk90     => clk90_sig,
      sysClk    => sysClk_sig,     -- differential clock output, before the DCM
      refClk    => refClk_sig,     -- differential clock output, with BUFG
      locked    => dcmlocked_sig
   );


   -- -----------------------------------------------------------------------------
   -- Reset module: reset statet machine and reset signals
   -- -----------------------------------------------------------------------------
   rst0 : rld_rst
   GENERIC MAP (
      SIMULATION_ONLY     => SIMULATION_ONLY,     -- if TRUE, it shortens the wait time
      RL_DQ_WIDTH         => RL_DQ_WIDTH,
      DEV_DQ_WIDTH        => DEV_DQ_WIDTH,
      NUM_OF_DEVS         => NUM_OF_DEVS,
      NUM_OF_DKS          => NUM_OF_DKS,
      DEV_AD_WIDTH        => DEV_AD_WIDTH,
      DEV_BA_WIDTH        => DEV_BA_WIDTH,
      DUPLICATE_CONTROLS  => DUPLICATE_CONTROLS,
      RL_IO_TYPE          => RL_IO_TYPE,
      DEVICE_ARCH         => DEVICE_ARCH,
      CAPTURE_METHOD      => CAPTURE_METHOD
   )
   PORT MAP (
      clk                => clk_sig,
      clk90              => clk90_sig,
      clk180             => clk180,
      clk270             => clk270,
      refClk             => refClk_sig,
      sysReset           => sysReset,
      confMReg           => confMReg,
      dcmLocked          => dcmlocked_sig,
      issueMRS           => issueMRS,
      calibration_done   => calibration_done,           -- calibration complete for QK and DQ 
      sync_system_reset  => sync_system_reset,
      rstHard            => rstHard,
      rstHard_90         => rstHard_90,
      rstHard_180        => rstHard_180,
      rstHard_270        => rstHard_270,
      rstHard_refClk     => rstHard_refClk,
      rstConfig          => rstConfig,
      Init_Done          => Init_Done,
      Init_Done_270      => Init_Done_270,
      mrsDone_o          => mrsDone_o,
      rstCmd             => rstCmd_int,
      rstAd              => rstAd_int,
      rstBa              => rstBa_int
   );


END ARCHITECTURE rtl;
