--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_iobs.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This module is the top level for the physical layer and  
--		instantiates the following modules: 
--		 * data_path_iobs
--		 * controller_iobs 
--		 * infrastructure_iobs
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_iobs IS
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE;   -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      sync_sysReset           : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      phy_ck_p                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_ck_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dk_p                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_dk_n                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_qk_p                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_qk_n                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_a                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      phy_ba                  : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      phy_cs_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_we_n                : OUT std_logic;
      phy_ref_n               : OUT std_logic;
      phy_dm                  : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dq                  : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      phy_qvld                : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_inc           : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_ce            : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_rst           : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_inc         : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_ce          : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_rst         : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_inc         : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_ce          : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_rst         : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      wr_data_rise            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_fall            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_en_clk90             : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_rise          : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_fall          : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rd_data_rise            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rd_data_fall            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      ctlCmd                  : IN  std_logic_vector(3 DOWNTO 0);
      ctlAd                   : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      ctlBa                   : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      read_enable_rise        : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      read_enable_fall        : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY rld_iobs;



ARCHITECTURE rtl OF rld_iobs IS


   COMPONENT rld_controller_iobs
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk            : IN  std_logic;
      clk90          : IN  std_logic;
      sync_sysReset  : IN  std_logic;
      rstHard        : IN  std_logic;
      rstHard_90     : IN  std_logic;
      rstHard_180    : IN  std_logic;
      rstHard_270    : IN  std_logic;
      phy_a          : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      phy_ba         : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      phy_cs_n       : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_we_n       : OUT std_logic;
      phy_ref_n      : OUT std_logic;
      ctl_Cmd        : IN  std_logic_vector(3 DOWNTO 0);
      ctl_Ad         : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      ctl_Ba         : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      cs_io          : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_data_path_iobs
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;  -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;  -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;   -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_BYTE_WIDTH          : integer := 9;   -- the width of a "byte" on the memory device (x8 or x9)
      DEV_AD_WIDTH            : integer := 20;  -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;   -- bank address width of the memory device
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                : IN  std_logic;
      clk90              : IN  std_logic;
      refClk             : IN  std_logic;
      rstHard            : IN  std_logic;
      rstHard_90         : IN  std_logic;
      rstHard_180        : IN  std_logic;
      rstHard_270        : IN  std_logic;
      phy_qk_p           : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_qk_n           : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_dm             : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dq             : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      phy_qvld           : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_inc      : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_ce       : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_rst      : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_inc    : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_ce     : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_rst    : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_inc    : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_ce     : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_rst    : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      wr_data_rise       : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_fall       : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_en_clk90        : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_rise     : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_fall     : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rd_data_rise       : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rd_data_fall       : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      read_enable_rise   : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      read_enable_fall   : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_infrastructure_iobs
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;  -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;  -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;   -- number of memory devices
      NUM_OF_DKS              : integer := 4
   );
   PORT (
      clk        : IN  std_logic;
      clk90      : IN  std_logic;
      phy_dk_p   : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_dk_n   : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_ck_p   : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_ck_n   : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0)
   );
   END COMPONENT;


   SIGNAL clk180                   : std_logic;
   SIGNAL clk270                   : std_logic;


BEGIN
   cs_io <= (OTHERS => 'Z');
   clk180 <= NOT clk;
   clk270 <= NOT clk90;


   -- -----------------------------------------------------------------------------
   -- rld_controller_iobs
   -- Holds the memory control physical layer: A, BA, WE#, REF# and CS#
   -- -----------------------------------------------------------------------------
   controller_iobs0 : rld_controller_iobs
   GENERIC MAP (
      RL_DQ_WIDTH        => RL_DQ_WIDTH,
      DEV_DQ_WIDTH       => DEV_DQ_WIDTH,
      NUM_OF_DEVS        => NUM_OF_DEVS,
      NUM_OF_DKS         => NUM_OF_DKS,
      DEV_AD_WIDTH       => DEV_AD_WIDTH,
      DEV_BA_WIDTH       => DEV_BA_WIDTH,
      DUPLICATE_CONTROLS => DUPLICATE_CONTROLS,
      RL_IO_TYPE         => RL_IO_TYPE,
      DEVICE_ARCH        => DEVICE_ARCH,
      CAPTURE_METHOD     => CAPTURE_METHOD
   )
   PORT MAP (
      clk            => clk,
      clk90          => clk90,
      sync_sysReset  => sync_sysReset,
      rstHard        => rstHard,
      rstHard_90     => rstHard_90,
      rstHard_180    => rstHard_180,
      rstHard_270    => rstHard_270,

      phy_a          => phy_a,       -- A  (address)
      phy_ba         => phy_ba,      -- BA (bank address)
      phy_cs_n       => phy_cs_n,    -- CS#
      phy_we_n       => phy_we_n,    -- WE#
      phy_ref_n      => phy_ref_n,   -- REF#

      ctl_Cmd        => ctlCmd,
      ctl_Ad         => ctlAd,
      ctl_Ba         => ctlBa,

      cs_io          => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- rld_data_path_iobs
   -- Instantiates iob modules for QK, DQ, QVLD and DM
   -- -----------------------------------------------------------------------------
   data_path_iobs0 : rld_data_path_iobs
   GENERIC MAP (
      RL_DQ_WIDTH    => RL_DQ_WIDTH,
      DEV_DQ_WIDTH   => DEV_DQ_WIDTH,
      NUM_OF_DEVS    => NUM_OF_DEVS,
      NUM_OF_DKS     => NUM_OF_DKS,
      RL_IO_TYPE     => RL_IO_TYPE,
      DEVICE_ARCH    => DEVICE_ARCH,
      CAPTURE_METHOD => CAPTURE_METHOD
   )
   PORT MAP (
      clk               => clk,
      clk90             => clk90,
      refClk            => refClk,
      rstHard           => rstHard,
      rstHard_90        => rstHard_90,
      rstHard_180       => rstHard_180,
      rstHard_270       => rstHard_270,

      phy_qk_p          => phy_qk_p,         -- QK, read data clock
      phy_qk_n          => phy_qk_n,         -- QK#
      phy_dm            => phy_dm,           -- DM
      phy_dq            => phy_dq,           -- DQ, read and write data bus
      phy_qvld          => phy_qvld,         -- QVLD

      qk_idelay_inc     => qk_idelay_inc,    -- IDELAY controls for QK (read clock)
      qk_idelay_ce      => qk_idelay_ce,
      qk_idelay_rst     => qk_idelay_rst,
      data_idelay_inc   => data_idelay_inc,  -- IDELAY controls for DQ (read data)
      data_idelay_ce    => data_idelay_ce,
      data_idelay_rst   => data_idelay_rst,
      qvld_idelay_inc   => qvld_idelay_inc,  -- IDELAY controls for QVLD (read enable)
      qvld_idelay_ce    => qvld_idelay_ce,
      qvld_idelay_rst   => qvld_idelay_rst,

      wr_data_rise      => wr_data_rise,
      wr_data_fall      => wr_data_fall,
      wr_en_clk90       => wr_en_clk90,
      data_mask_rise    => data_mask_rise,
      data_mask_fall    => data_mask_fall,

      rd_data_rise      => rd_data_rise,
      rd_data_fall      => rd_data_fall,

      read_enable_rise  => read_enable_rise,
      read_enable_fall  => read_enable_fall
   );


   -- -----------------------------------------------------------------------------
   -- rld_infrastructure_iobs
   -- Generates data clock (DK) and address/command clock (CK) for memory physical layer
   -- -----------------------------------------------------------------------------
   infrastructure_iobs0 : rld_infrastructure_iobs
   GENERIC MAP (
      RL_DQ_WIDTH  => RL_DQ_WIDTH,
      DEV_DQ_WIDTH => DEV_DQ_WIDTH,
      NUM_OF_DEVS  => NUM_OF_DEVS,
      NUM_OF_DKS   => NUM_OF_DKS
   )
   PORT MAP (
      clk       => clk,
      clk90     => clk90,

      phy_dk_p  => phy_dk_p,
      phy_dk_n  => phy_dk_n,
      phy_ck_p  => phy_ck_p,
      phy_ck_n  => phy_ck_n
   );


END ARCHITECTURE rtl;
