--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_main.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This module instantiates the memory core (top) and the 
--		synthesizable testbench that acts as the user interface.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Pass RL_CK_PERIOD parameter to sub-modules
--
-- Port Definitions:
--
-- keep_qk_fall_unused[2*NUM_OF_DEVS-1:0]    output
--       keep falling edge levels of QK to prevent Q2 output of IDDR from being
--       trimmed, otherwise map 6.3.03 will crash. This has been fixed in 7.1i.
--
-- apAddr[25:0]        input    
--     apAddr[25]    - user refresh
--     apAddr[24]    - R/~W  Reads and writes will be determined by bit 24 of 
--                     the "apAddr" bus
--     apAddr[23:23] - Reserved
--     apAddr[22:3]  - Memory Address bits A[19:0]
--     apAddr[2:0]   - Memory Bank Address Bits BA[2:0]
--
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_main IS
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE; -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD                   : integer := 3003;  -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF                    : std_logic_vector(2 DOWNTO 0) := "001";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH            : std_logic_vector(1 DOWNTO 0) := "00";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX                : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET               : std_logic := '0';    --
      RL_MRS_IMPEDANCE_MATCH         : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external
      RL_MRS_ODT                     : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00";   -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS                    : std_logic_vector(19 DOWNTO 0) := (others => '0')  -- saved location to perform calibration
   );
   PORT (
--      sysClk                  : IN  std_logic;
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      rstHard_refClk          : IN  std_logic;
      sync_sysReset           : IN  std_logic;
      dcmLocked               : IN  std_logic;
      calibration_done        : OUT std_logic;
      rstConfig               : IN  std_logic;
      rstCmd                  : IN  std_logic_vector(3 DOWNTO 0);
      rstAd                   : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      confMReg                : OUT std_logic_vector(17 DOWNTO 0);
--      Init_Done_270           : IN  std_logic;
      Init_Done               : IN  std_logic;
      issueMRS                : OUT std_logic;
      mrsDone_o               : IN  std_logic;
      PASS_FAIL               : OUT std_logic_vector(2 DOWNTO 0);
      -- RLDRAM interface signals
      rld2_ck_p               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_ck_n               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_dk_p               : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      rld2_dk_n               : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      rld2_qk_p               : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      rld2_qk_n               : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      rld2_a                  : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rld2_ba                 : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      rld2_cs_n               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_we_n               : OUT std_logic;
      rld2_ref_n              : OUT std_logic;
      rld2_dm                 : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_dq                 : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rld2_qvld               : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY rld_main;



ARCHITECTURE rtl OF rld_main IS


   COMPONENT rld_top
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE; -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD                   : integer := 3003;   -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF                    : std_logic_vector(2 DOWNTO 0) := "001";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH            : std_logic_vector(1 DOWNTO 0) := "00";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX                : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET               : std_logic := '0';    --
      RL_MRS_IMPEDANCE_MATCH         : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external
      RL_MRS_ODT                     : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00";   -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS                    : std_logic_vector(19 DOWNTO 0) := (others => '0')   -- saved location to perform calibration
   );
   PORT (
      -- global signals
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      sync_sysReset           : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      rstHard_refClk          : IN  std_logic;
      issueCalibration        : IN  std_logic;
      calibration_done        : OUT std_logic;
      rstConfig               : IN  std_logic;
      rstCmd                  : IN  std_logic_vector(3 DOWNTO 0);
      rstAd                   : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      -- RLDRAM-II interface signals
      phy_ck_p                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_ck_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dk_p                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_dk_n                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_qk_p                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_qk_n                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_a                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      phy_ba                  : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      phy_cs_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_we_n                : OUT std_logic;
      phy_ref_n               : OUT std_logic;
      phy_dm                  : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dq                  : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      phy_qvld                : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      -- application interface signals
      apRdfRdEn               : IN  std_logic;
      rldReadData             : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      rldReadDataValid        : OUT std_logic;
      apAddr                  : IN  std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      apValid                 : IN  std_logic;
      apWriteDM               : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      apWriteData             : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      apWriteDValid           : IN  std_logic;
      apConfA                 : IN  std_logic_vector(3 DOWNTO 0);
      apConfWrD               : IN  std_logic_vector(7 DOWNTO 0);
      apConfWr                : IN  std_logic;
      apConfRdD               : OUT std_logic_vector(7 DOWNTO 0);
      apConfRd                : IN  std_logic;
      rlWdfEmpty              : OUT std_logic;
      rlWdfFull               : OUT std_logic;
      rlRdfEmpty              : OUT std_logic;
      rlafEmpty               : OUT std_logic;
      rlafFull                : OUT std_logic;
      rlWdfWrCount            : OUT std_logic_vector(12 DOWNTO 0);   --  write data FIFO (wdfifo) write count
      rlWdfWordCount          : OUT std_logic_vector(12 DOWNTO 0);   --  write data FIFO (wdfifo) write count
      rlafWrCount             : OUT std_logic_vector(12 DOWNTO 0);   --  command/address FIFO (rafifo) write count
      rlafWordCount           : OUT std_logic_vector(12 DOWNTO 0);   --  command/address FIFO (rafifo) word count
      Init_Done               : IN  std_logic;
      issueMRS                : IN  std_logic;
      confMReg                : OUT std_logic_vector(17 DOWNTO 0);
      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT test_bench
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    -- number of memory devices
      NUM_OF_DKS                     : integer := 4;
      APP_AD_WIDTH                   : integer := 26;
      RL_MRS_BURST_LENGTH            : std_logic_vector(1 DOWNTO 0) := "00"     -- "00": BL2;  "01": BL4;  "10": BL8
   );
   PORT (
      CLK                     : IN  std_logic;
      RESET                   : IN  std_logic;
      RESET_90                : IN  std_logic;
      WDF_ALMOST_FULL         : IN  std_logic;
      WDF_WR_COUNT            : IN  std_logic_vector(12 DOWNTO 0);   -- was (4 DOWNTO 0)
      RAF_ALMOST_FULL         : IN  std_logic;
      RAF_WR_COUNT            : IN  std_logic_vector(12 DOWNTO 0);   -- was (3 DOWNTO 0)
      rlafEmpty               : IN  std_logic;
      BURST_LENGTH            : IN  std_logic_vector(1 DOWNTO 0);
      READ_DATA_FIFO_OUT      : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
      READ_DATA_VALID         : IN  std_logic;
      Init_Done               : IN  std_logic;
--      Init_Done_270           : IN  std_logic;
      clk90                   : IN  std_logic;
      rlRdfEmpty              : IN  std_logic;
      rlWdfEmpty              : IN  std_logic;

      APADDR                  : OUT std_logic_vector(25 DOWNTO 0);
      APVALID                 : OUT std_logic;
      APP_WDF_DATA            : OUT std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
      APP_WDF_WREN            : OUT std_logic;

      DISPLAY                 : OUT std_logic_vector(6 DOWNTO 0);
      error_count_BCD         : OUT std_logic_vector(23 DOWNTO 0);

      APCONFA                 : OUT std_logic_vector(3 DOWNTO 0);
      APCONFWRD               : OUT std_logic_vector(7 DOWNTO 0);
      APCONFRD                : OUT std_logic;
      APCONFRDD               : IN  std_logic_vector(7 DOWNTO 0);
      APCONFWR                : OUT std_logic;
      APWRITEDM               : OUT std_logic_vector(3 DOWNTO 0);
      apRdfRdEn               : OUT std_logic;
      issueMRS                : OUT std_logic;
      issueCalibration        : OUT std_logic;

      PASS_FAIL               : OUT std_logic_vector(2 DOWNTO 0);
      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   SIGNAL rldReadData              :  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
   SIGNAL rldReadDataValid         :  std_logic;
   SIGNAL apAddr                   :  std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
   SIGNAL apWriteData              :  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
   SIGNAL apWriteDM                :  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
   SIGNAL apRdfRdEn                :  std_logic;
   SIGNAL apWriteDValid            :  std_logic;
   SIGNAL apConfRdD                :  std_logic_vector(7 DOWNTO 0);
   SIGNAL apValid                  :  std_logic;
   SIGNAL rlWdfFull                :  std_logic;
   SIGNAL rlWdfEmpty               :  std_logic;
   SIGNAL rlafFull                 :  std_logic;
   SIGNAL rlafEmpty                :  std_logic;
   SIGNAL rlRdfEmpty               :  std_logic;
   SIGNAL rlWdfWrCount             :  std_logic_vector(12 DOWNTO 0);   -- write data FIFO write count
   SIGNAL rlWdfWordCount           :  std_logic_vector(12 DOWNTO 0);   -- write data FIFO word count
   SIGNAL rlafWrCount              :  std_logic_vector(12 DOWNTO 0);   -- command/address FIFO write count
   SIGNAL rlafWordCount            :  std_logic_vector(12 DOWNTO 0);   -- command/address FIFO word count
   SIGNAL apConfA                  :  std_logic_vector(3 DOWNTO 0);
   SIGNAL apConfWrD                :  std_logic_vector(7 DOWNTO 0);
   SIGNAL apConfRd                 :  std_logic;
   SIGNAL apConfWr                 :  std_logic;
   SIGNAL issueCalibration         :  std_logic;   -- user command to restart the calibration

   -- internal signals
   SIGNAL confMReg_sig             :  std_logic_vector(17 DOWNTO 0);
   SIGNAL issueMRS_sig             :  std_logic;


BEGIN
   cs_io <= (OTHERS => 'Z');
   -- internal signal assignments
   confMReg          <= confMReg_sig;
   issueMRS          <= issueMRS_sig;


   -- -----------------------------------------------------------------------------
   -- Main controller module:  CIO RLDRAM-II Controller
   -- -----------------------------------------------------------------------------
   top0 : rld_top
   GENERIC MAP (
      RL_DQ_WIDTH            => RL_DQ_WIDTH,
      DEV_DQ_WIDTH           => DEV_DQ_WIDTH,
      NUM_OF_DEVS            => NUM_OF_DEVS,
      NUM_OF_DKS             => NUM_OF_DKS,
      DEV_AD_WIDTH           => DEV_AD_WIDTH,
      DEV_BA_WIDTH           => DEV_BA_WIDTH,
      DUPLICATE_CONTROLS     => DUPLICATE_CONTROLS,
      RL_CK_PERIOD           => RL_CK_PERIOD,
      RL_MRS_CONF            => RL_MRS_CONF,             -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH    => RL_MRS_BURST_LENGTH,     -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX        => RL_MRS_ADDR_MUX,         -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET       => RL_MRS_DLL_RESET,        --
      RL_MRS_IMPEDANCE_MATCH => RL_MRS_IMPEDANCE_MATCH,  -- internal 50ohms output buffer impedance
      RL_MRS_ODT             => RL_MRS_ODT,              -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE             => RL_IO_TYPE,
      DEVICE_ARCH            => DEVICE_ARCH,
      CAPTURE_METHOD         => CAPTURE_METHOD,
      CAL_ADDRESS            => CAL_ADDRESS
   )
   PORT MAP (
      clk               => clk,
      clk90             => clk90,
      refClk            => refClk,
      sync_sysReset     => sync_sysReset,
      rstHard           => rstHard,
      rstHard_90        => rstHard_90,
      rstHard_180       => rstHard_180,
      rstHard_270       => rstHard_270,
      rstHard_refClk    => rstHard_refClk,

      issueCalibration  => issueCalibration,
      calibration_done  => calibration_done,

      rstConfig         => rstConfig,
      rstCmd            => rstCmd,
      rstAd             => rstAd,
      rstBa             => rstBa,

      -- RLDRAM-II interface signals
      phy_ck_p          => rld2_ck_p,
      phy_ck_n          => rld2_ck_n,
      phy_dk_p          => rld2_dk_p,
      phy_dk_n          => rld2_dk_n,
      phy_qk_p          => rld2_qk_p,
      phy_qk_n          => rld2_qk_n,
      phy_a             => rld2_a,
      phy_ba            => rld2_ba,
      phy_cs_n          => rld2_cs_n,
      phy_we_n          => rld2_we_n,
      phy_ref_n         => rld2_ref_n,
      phy_dm            => rld2_dm,
      phy_dq            => rld2_dq,
      phy_qvld          => rld2_qvld,

      -- application interface signals
      apRdfRdEn         => apRdfRdEn,
      rldReadData       => rldReadData,
      rldReadDataValid  => rldReadDataValid,

      apAddr            => apAddr,
      apValid           => apValid,
      apWriteDM         => apWriteDM,
      apWriteData       => apWriteData,
      apWriteDValid     => apWriteDValid,
      apConfA           => apConfA,
      apConfWrD         => apConfWrD,
      apConfWr          => apConfWr,
      apConfRdD         => apConfRdD,

      apConfRd          => apConfRd,
      rlWdfEmpty        => rlWdfEmpty,
      rlWdfFull         => rlWdfFull,
      rlRdfEmpty        => rlRdfEmpty,
      rlafEmpty         => rlafEmpty,
      rlafFull          => rlafFull,
      rlWdfWrCount      => rlWdfWrCount,     -- write data FIFO (wdfifo) write count
      rlWdfWordCount    => rlWdfWordCount,   -- write data FIFO (wdfifo) word count
      rlafWrCount       => rlafWrCount,      -- command/address FIFO (rafifo) write count
      rlafWordCount     => rlafWordCount,    -- command/address FIFO (rafifo) word count

      Init_Done         => Init_Done,
      issueMRS          => issueMRS_sig,
      confMReg          => confMReg_sig,

      cs_io             => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- User Application: synthesizable testbench
   -- -----------------------------------------------------------------------------
   test_bench0 : test_bench 
   GENERIC MAP (
      RL_DQ_WIDTH            => RL_DQ_WIDTH,
--      DEV_DQ_WIDTH           => DEV_DQ_WIDTH,
--      NUM_OF_DEVS            => NUM_OF_DEVS,
--      DEV_AD_WIDTH           => DEV_AD_WIDTH,
--      DEV_BA_WIDTH           => DEV_BA_WIDTH,
        APP_AD_WIDTH           => APP_AD_WIDTH,
--      DUPLICATE_CONTROLS     => DUPLICATE_CONTROLS,
      -- specific rld_conf parameters
--      RL_MRS_CONF            => RL_MRS_CONF,             -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH    => RL_MRS_BURST_LENGTH      -- "00": BL2;  "01": BL4;  "10": BL8
--      RL_MRS_ADDR_MUX        => RL_MRS_ADDR_MUX,         -- '0': non-muxed addr;  '1': muxed addr
--      RL_MRS_DLL_RESET       => RL_MRS_DLL_RESET,        --
--      RL_MRS_IMPEDANCE_MATCH => RL_MRS_IMPEDANCE_MATCH,  -- internal 50ohms output buffer impedance
--      RL_MRS_ODT             => RL_MRS_ODT,              -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
--      RL_IO_TYPE             => RL_IO_TYPE,
--      DEVICE_ARCH            => DEVICE_ARCH,
--      CAPTURE_METHOD         => CAPTURE_METHOD,
--      CAL_ADDRESS            => CAL_ADDRESS
   )
   PORT MAP (
      CLK                => clk,
      clk90              => clk90,
      RESET              => rstHard,
      RESET_90           => rstHard_90,
--      dcmLocked          => dcmLocked,

      WDF_ALMOST_FULL    => rlWdfFull,
      WDF_WR_COUNT       => rlWdfWrCount,      -- write data FIFO (wdfifo) write count
--      wdf_word_count     => rlWdfWordCount,    -- write data FIFO (wdfifo) word count
      RAF_ALMOST_FULL    => rlafFull,
      RAF_WR_COUNT       => rlafWrCount,       -- command/address FIFO (rafifo) write count
--      raf_word_count     => rlafWordCount,     -- command/address FIFO (rafifo) word count
      rlafEmpty          => rlafEmpty,
      BURST_LENGTH       => confMReg_sig(4 DOWNTO 3),
      READ_DATA_FIFO_OUT => rldReadData,
      READ_DATA_VALID    => rldReadDataValid,  -- Read Data valid flag (will be used by the user or test bench)

      Init_Done          => Init_Done,
      rlRdfEmpty         => rlRdfEmpty,
      rlWdfEmpty         => rlWdfEmpty,
--      mrsDone_o          => mrsDone_o,

      APADDR             => apAddr,
      APVALID            => apValid,
      APP_WDF_DATA       => apWriteData,
      APP_WDF_WREN       => apWriteDValid,
      APCONFA            => apConfA,
      APCONFWRD          => apConfWrD,
      APCONFRD           => apConfRd,
      APCONFWR           => apConfWr,
      APCONFRDD          => apConfRdD,
      APWRITEDM          => apWriteDM,
      apRdfRdEn          => apRdfRdEn,

      issueMRS           => issueMRS_sig,
      issueCalibration   => issueCalibration,

      DISPLAY            => open,
      error_count_BCD    => open,
      PASS_FAIL          => PASS_FAIL,

      cs_io              => cs_io
   );


END ARCHITECTURE rtl;
