--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_mem_interface_top.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-5
--	Purpose: Top level of the RLD2 Memory Interface, ready for ML561.
--		Instantiates the main design along with the infastructure 
--		for clocks and resets.  It also includes Chipscope modules.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Pass RL_CK_FREQ parameter to sub-modules, changed default mode to 
--             config 3, Memory DLL enabled by default. 
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_mem_interface_top IS
   GENERIC (
      SIMULATION_ONLY         : boolean := FALSE;  -- if TRUE, it shortens the wait time
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 2;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      --
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "011";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '1';    -- '0': DLL reset, '1': DLL enabled
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01";  -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS             : std_logic_vector(19 DOWNTO 0) := (others => '0')     -- saved location to perform calibration
   );
   PORT (
      -- System signals and debug
      refClk_p                : IN  std_logic;
      refClk_n                : IN  std_logic;
      sysClk_p                : IN  std_logic;
      sysClk_n                : IN  std_logic;
      sysReset                : IN  std_logic;
      dip                     : IN  std_logic;
      clk33                   : IN  std_logic;
      FPGA3_7SEG              : OUT std_logic_vector(7 DOWNTO 0);
      DBG_LED                 : OUT std_logic_vector(3 DOWNTO 0);
      FPGA3_TEST_HDR          : OUT std_logic_vector(7 DOWNTO 0);
      -- RLD2 memory signals
      RLD2_CK_P               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_CK_N               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_DK_P               : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      RLD2_DK_N               : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      RLD2_QK_P               : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_QK_N               : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_A                  : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      RLD2_BA                 : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      RLD2_CS_N               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_WE_N               : OUT std_logic;
      RLD2_REF_N              : OUT std_logic;
      RLD2_DM                 : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_DQ                 : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      RLD2_QVLD               : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0)
--      PASS_FAIL               : OUT std_logic_vector(2 DOWNTO 0)
   );
END ENTITY rld_mem_interface_top;


ARCHITECTURE rtl OF rld_mem_interface_top IS


   COMPONENT rld_infrastructure_top
   GENERIC (
      SIMULATION_ONLY                : boolean := FALSE;    -- if TRUE, it shortens the wait time
      RL_DQ_WIDTH                    : integer := 36;       -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;       --  data width of the memory device
      NUM_OF_DEVS                    : integer := 2;        --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;       -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;        -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE;    -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      sysClk_p                : IN  std_logic;
      sysClk_n                : IN  std_logic;
      refClk_p                : IN  std_logic;
      refClk_n                : IN  std_logic;
      sysReset                : IN  std_logic;
      confMReg                : IN  std_logic_vector(17 DOWNTO 0);
      issueMRS                : IN  std_logic;
      calibration_done        : IN  std_logic;   -- QK and DQ calibrations are completed
      sysClk                  : OUT std_logic;
      clk                     : OUT std_logic;
      clk90                   : OUT std_logic;
      refClk                  : OUT std_logic;
      clk33                   : IN  std_logic;
      dcmlocked               : OUT std_logic;
      sync_system_reset       : OUT std_logic;
      rstHard                 : OUT std_logic;
      rstHard_90              : OUT std_logic;
      rstHard_180             : OUT std_logic;
      rstHard_270             : OUT std_logic;
      rstHard_refClk          : OUT std_logic;
      rstConfig               : OUT std_logic;
      Init_Done               : OUT std_logic;
      Init_Done_270           : OUT std_logic;
      mrsDone_o               : OUT std_logic;
      rstCmd                  : OUT std_logic_vector(3 DOWNTO 0);
      rstAd                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_main
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "010";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '0';    -- 
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01";  -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS             : std_logic_vector(19 DOWNTO 0):= (others => '0')  -- saved location to perform calibration
   );
   PORT (
      clk               : IN  std_logic;
      clk90             : IN  std_logic;
      refClk            : IN  std_logic;
      rstHard           : IN  std_logic;
      rstHard_90        : IN  std_logic;
      rstHard_180       : IN  std_logic;
      rstHard_270       : IN  std_logic;
      rstHard_refClk    : IN  std_logic;
      sync_sysReset     : IN  std_logic;
      dcmLocked         : IN  std_logic;
      calibration_done  : OUT std_logic;
      rstConfig         : IN  std_logic;
      rstCmd            : IN  std_logic_vector(3 DOWNTO 0);
      rstAd             : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa             : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      confMReg          : OUT std_logic_vector(17 DOWNTO 0);
      Init_Done         : IN   std_logic;
      issueMRS          : OUT std_logic;
      mrsDone_o         : IN  std_logic;
      PASS_FAIL         : OUT std_logic_vector(2 DOWNTO 0);
      -- RLDRAM interface signals
      rld2_ck_p         : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_ck_n         : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_dk_p         : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      rld2_dk_n         : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      rld2_qk_p         : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      rld2_qk_n         : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      rld2_a            : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rld2_ba           : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      rld2_cs_n         : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_we_n         : OUT std_logic;
      rld2_ref_n        : OUT std_logic;
      rld2_dm           : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rld2_dq           : INOUT std_logic_vector(RL_DQ_WIDTH- 1 DOWNTO 0);
      rld2_qvld         : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      cs_io             : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_seven_seg
   GENERIC (
      READ_DIV_CNT_WIDTH  : integer := 28
   );
   PORT (
      clk             : IN  std_logic;
      rst_n           : IN  std_logic;
      init_done       : IN  std_logic;
      valid           : IN  std_logic;
      PASS_FAIL       : IN  std_logic_vector(2 DOWNTO 0);
      seven_seg_n     : OUT std_logic_vector(6 DOWNTO 0);
      seven_seg_dp_n  : OUT std_logic;
      LED             : OUT std_logic_vector(3 DOWNTO 0);
      cs_io           : INOUT std_logic_vector(1023 DOWNTO 0);   --  for debug
      dip             : IN  std_logic
   );
   END COMPONENT;


   -- ICON core module declaration
   COMPONENT icon
   PORT (
      control0 : OUT std_logic_vector(35 DOWNTO 0)
   );
   END COMPONENT;

   -- ILA core module declaration
   COMPONENT ila256
   PORT (
      control : IN  std_logic_vector(35 DOWNTO 0);
      clk     : IN  std_logic;
      data    : IN  std_logic_vector(255 DOWNTO 0);
      trig0   : IN  std_logic_vector(15 DOWNTO 0)
   );
   END COMPONENT;


   -- Signals
   SIGNAL clk                      : std_logic;
   SIGNAL clk90                    : std_logic;
   SIGNAL sysClk                   : std_logic;
   SIGNAL refClk                   : std_logic;
   SIGNAL sync_system_reset        : std_logic;
   SIGNAL not_sync_system_reset    : std_logic;   -- required for ModelSim: "not a globally static expression"...
   SIGNAL dcmLocked                : std_logic;
   SIGNAL calibration_done         : std_logic;
   SIGNAL rstConfig                : std_logic;
   SIGNAL Init_Done                : std_logic;
   SIGNAL Init_Done_270            : std_logic;
   SIGNAL rstHard                  : std_logic;
   SIGNAL rstHard_90               : std_logic;
   SIGNAL rstHard_180              : std_logic;
   SIGNAL rstHard_270              : std_logic;
   SIGNAL rstHard_refClk           : std_logic;
   SIGNAL rstCmd                   : std_logic_vector(3 DOWNTO 0);
   SIGNAL rstAd                    : std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
   SIGNAL rstBa                    : std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
   SIGNAL confMReg                 : std_logic_vector(17 DOWNTO 0);
   SIGNAL issueMRS                 : std_logic;
   SIGNAL mrsDone_o                : std_logic;
   SIGNAL PASS_FAIL                : std_logic_vector(2 DOWNTO 0);
   -- chipscope specific signals
   SIGNAL cs_control0              : std_logic_vector(35 DOWNTO 0);
   SIGNAL cs_clk0                  : std_logic;
   SIGNAL cs_data0                 : std_logic_vector(255 DOWNTO 0);
   SIGNAL cs_trig0                 : std_logic_vector(15 DOWNTO 0);
   SIGNAL cs_io                    : std_logic_vector(1023 DOWNTO 0);

   -- internal signals
   SIGNAL FPGA3_7SEG_sig           : std_logic_vector(7 DOWNTO 0);
   SIGNAL DBG_LED_sig              : std_logic_vector(3 DOWNTO 0);
   SIGNAL FPGA3_TEST_HDR_sig       : std_logic_vector(7 DOWNTO 0);


BEGIN
   cs_io <= (others => 'Z');
   not_sync_system_reset <= NOT sync_system_reset;   -- required for ModelSim: "not a globally static expression"...

   -- internal signal assignments
   FPGA3_7SEG     <= FPGA3_7SEG_sig;
   DBG_LED        <= DBG_LED_sig;
   FPGA3_TEST_HDR <= FPGA3_TEST_HDR_sig;

   -- assign debug signals to test header on ML561
   FPGA3_TEST_HDR_sig <= DBG_LED_sig(3) &
                         PASS_FAIL(2)   & 
                         dcmLocked      & 
                         Init_Done      & 
                         dip            & 
                         rstHard        & 
                         refClk         &
                         clk;

   
   -- -----------------------------------------------------------------------------
   -- rld_infrastructure_top
   --   Adds clock and reset signals
   -- -----------------------------------------------------------------------------
   infrastructure_top0 : rld_infrastructure_top
   GENERIC MAP (
      SIMULATION_ONLY    => SIMULATION_ONLY,     -- if TRUE, it shortens the wait time
      RL_DQ_WIDTH        => RL_DQ_WIDTH,
      DEV_DQ_WIDTH       => DEV_DQ_WIDTH,
      NUM_OF_DEVS        => NUM_OF_DEVS,         -- number of memory devices
      NUM_OF_DKS         => NUM_OF_DKS,
      DEV_AD_WIDTH       => DEV_AD_WIDTH,
      DEV_BA_WIDTH       => DEV_BA_WIDTH,
      APP_AD_WIDTH       => APP_AD_WIDTH,
      DUPLICATE_CONTROLS => DUPLICATE_CONTROLS,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE         => RL_IO_TYPE,
      DEVICE_ARCH        => DEVICE_ARCH,
      CAPTURE_METHOD     => CAPTURE_METHOD
   )
   PORT MAP (
      sysClk_p           => sysClk_p,
      sysClk_n           => sysClk_n,
      refClk_p           => refClk_p,
      refClk_n           => refClk_n,
      sysReset           => sysReset,            -- system reset input, acive low, asynchronous
      clk33              => clk33,

      confMReg           => confMReg,
      issueMRS           => issueMRS,
      calibration_done   => calibration_done,

      sysClk             => sysClk,
      clk                => clk,                 -- global clock to all modules
      clk90              => clk90,
      refClk             => refClk,
      dcmlocked          => dcmLocked,

      sync_system_reset  => sync_system_reset,   -- system reset, synchronized to clk
      rstHard            => rstHard,
      rstHard_90         => rstHard_90,
      rstHard_180        => rstHard_180,
      rstHard_270        => rstHard_270,
      rstHard_refClk     => rstHard_refClk,
      rstConfig          => rstConfig,

      Init_Done          => Init_Done,
      Init_Done_270      => Init_Done_270,
      mrsDone_o          => mrsDone_o,

      rstCmd             => rstCmd,
      rstAd              => rstAd,
      rstBa              => rstBa
   );


   -- -----------------------------------------------------------------------------
   -- rld_main
   --   RLDRAM-2 Controller and User Application
   -- -----------------------------------------------------------------------------
   main0 : rld_main 
   GENERIC MAP (
      RL_DQ_WIDTH            => RL_DQ_WIDTH,             -- memory interface data width
      DEV_DQ_WIDTH           => DEV_DQ_WIDTH,            -- data width of the memory device
      NUM_OF_DEVS            => NUM_OF_DEVS,             -- number of memory devices
      NUM_OF_DKS             => NUM_OF_DKS,
      DEV_AD_WIDTH           => DEV_AD_WIDTH,            -- address width of the memory device
      DEV_BA_WIDTH           => DEV_BA_WIDTH,            -- bank address width of the memory device
      APP_AD_WIDTH           => APP_AD_WIDTH,
      DUPLICATE_CONTROLS     => DUPLICATE_CONTROLS,      -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD           => RL_CK_PERIOD,            -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      RL_MRS_CONF            => RL_MRS_CONF,             -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH    => RL_MRS_BURST_LENGTH,     -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX        => RL_MRS_ADDR_MUX,         -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET       => RL_MRS_DLL_RESET,        --
      RL_MRS_IMPEDANCE_MATCH => RL_MRS_IMPEDANCE_MATCH,  -- internal 50ohms output buffer impedance
      RL_MRS_ODT             => RL_MRS_ODT,              -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE             => RL_IO_TYPE,              -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH            => DEVICE_ARCH,             -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD         => CAPTURE_METHOD,          -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS            => CAL_ADDRESS
   )
   PORT MAP (
      clk               => clk,
      clk90             => clk90,
      refClk            => refClk,
      rstHard           => rstHard,
      rstHard_90        => rstHard_90,
      rstHard_180       => rstHard_180,
      rstHard_270       => rstHard_270,
      rstHard_refClk    => rstHard_refClk,
      sync_sysReset     => sync_system_reset,
      dcmLocked         => dcmLocked,
 
      calibration_done  => calibration_done,
      rstConfig         => rstConfig,
      rstCmd            => rstCmd,
      rstAd             => rstAd,
      rstBa             => rstBa,
      confMReg          => confMReg,
      Init_Done         => Init_Done,
      issueMRS          => issueMRS,
      mrsDone_o         => mrsDone_o,
 
      PASS_FAIL         => PASS_FAIL,
 
      rld2_ck_p         => RLD2_CK_P,
      rld2_ck_n         => RLD2_CK_N,
      rld2_dk_p         => RLD2_DK_P,
      rld2_dk_n         => RLD2_DK_N,
      rld2_qk_p         => RLD2_QK_P,
      rld2_qk_n         => RLD2_QK_N,
      rld2_a            => RLD2_A,
      rld2_ba           => RLD2_BA,
      rld2_cs_n         => RLD2_CS_N,
      rld2_we_n         => RLD2_WE_N,
      rld2_ref_n        => RLD2_REF_N,
      rld2_dm           => RLD2_DM,
      rld2_dq           => RLD2_DQ,
      rld2_qvld         => RLD2_QVLD,
 
      cs_io             => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- rld_seven_seg
   --   - module used to control ML561 LEDs
   --   - useful for debug/demo	
   -- -----------------------------------------------------------------------------
   seven_seg0 : rld_seven_seg 
   PORT MAP (
      clk             => clk,
      rst_n           => not_sync_system_reset,

      init_done       => Init_Done,
      valid           => cs_io(90),
      PASS_FAIL       => PASS_FAIL,

      seven_seg_n     => FPGA3_7SEG_sig(6 DOWNTO 0),
      seven_seg_dp_n  => FPGA3_7SEG_sig(7),
      LED             => DBG_LED_sig,

      cs_io           => cs_io,
      dip             => dip
   );


   -- -----------------------------------------------------------------------------
   -- Chipscope module instantiations
   -- -----------------------------------------------------------------------------   
   --i_icon : icon
   --PORT MAP (
   --   control0  => cs_control0
   --);

  --i_ila0 : ila256
  -- PORT MAP (
  --    control  => cs_control0,
  --    clk      => cs_clk0,
  --    data     => cs_data0,
  --    trig0    => cs_trig0
  -- );

   cs_clk0 <= clk;
--   cs_clk0 <= NOT clk90;  -- use clk270 for checking phy signals
   cs_data0 <= cs_io(255 DOWNTO 0);

--   cs_trig0 <= "0000000000" &            -- 10 zeroes
--               PASS_FAIL(2) &            -- Error Signal
--               cs_io(111 DOWNTO 100) &   -- f_cs, data cal SM for debug
--               cs_io(95 DOWNTO 93);      -- ref[2], we[1], cs[0]
   cs_trig0 <= PASS_FAIL(2) &            -- Error Signal
               cs_io(111 DOWNTO 100) &   -- f_cs, data cal SM for debug
               cs_io(95 DOWNTO 93);      -- ref[2], we[1], cs[0]


END ARCHITECTURE rtl;