--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_tap_logic.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-5
--	Purpose: This module instantiates the tap control modules that control
--		the physical layer IDELAY elements.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Pass DEV_DQ_WIDTH, RL_CK_PERIOD, & QK_DATA_WIDTH parameter 
--             to sub-modules
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;


ENTITY rld_tap_logic IS
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      QK_DATA_WIDTH           : integer := 9;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps      
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      rstHard_refClk          : IN  std_logic;

      read_enable_rise        : IN  std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      read_enable_fall        : IN  std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      read_enable_rise_r      : IN  std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      CTRL_DUMMYREAD_START    : IN  std_logic;
      refresh_cmd             : IN  std_logic;
      issueCalibration        : IN  std_logic;

      qk_idelay_inc           : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qk_idelay_ce            : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qk_idelay_rst           : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      data_idelay_inc         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      data_idelay_ce          : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      data_idelay_rst         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      qvld_idelay_inc         : OUT std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qvld_idelay_ce          : OUT std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      qvld_idelay_rst         : OUT std_logic_vector((RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      read_enable_out         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      delay_select            : OUT std_logic_vector(4*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);

      DQ_tap_count            : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)*6-1 DOWNTO 0);  -- 6-bit DQ counts, expandable per device, 2 counts per device

      first_stage_done        : OUT std_logic;   -- QK centered in DQ window 
      second_stage_begin      : IN  std_logic;   -- ready to start next phase
      qvld_cal                : IN  std_logic;   -- align qvld
      data_tap_select         : OUT std_logic_vector(2*(RL_DQ_WIDTH/DEV_DQ_WIDTH)-1 DOWNTO 0);
      idelay_ctrl_rdy_r1      : OUT std_logic;

      rd_data_rise            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rd_data_fall            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY rld_tap_logic;



ARCHITECTURE rtl OF rld_tap_logic IS

   ATTRIBUTE loc : STRING;
   -- This loc is implementation dependent and must be updated upon device or pinout changes
   ATTRIBUTE loc OF idelayctrl0 : LABEL IS "IDELAYCTRL_X2Y5";
   ATTRIBUTE loc OF idelayctrl1 : LABEL IS "IDELAYCTRL_X2Y3";


   function AND_REDUCE(ARG: in STD_LOGIC_VECTOR) return std_logic is
      variable result: STD_LOGIC := '1';
   begin
      for i in ARG'range loop
         result := result AND ARG(i);
      end loop;
      return result;
   end function AND_REDUCE;


   COMPONENT rld_dly_cal_sm
   GENERIC (
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps      
      QK_DATA_WIDTH           : integer := 9
   );
   PORT (
      clk                : IN  std_logic;
      rst                : IN  std_logic;
      start_cal          : IN  std_logic;
      second_stage       : IN  std_logic;
      qvld_cal           : IN  std_logic;
      wait_for_refresh   : IN  std_logic;
      rd_data_rise       : IN  std_logic_vector(8 DOWNTO 0);
      rd_data_fall       : IN  std_logic_vector(8 DOWNTO 0);
      rd_data_valid      : IN  std_logic;
      rd_data_valid_r    : IN  std_logic;

      dq_dly_rst         : OUT std_logic;
      dq_dly_ce          : OUT std_logic;
      dq_dly_inc         : OUT std_logic;

      qk_dly_rst         : OUT std_logic;
      qk_dly_ce          : OUT std_logic;
      qk_dly_inc         : OUT std_logic;

      first_cal_done     : OUT std_logic;
      dly_cal_done       : OUT std_logic;
      delay_select       : OUT std_logic_vector(1 DOWNTO 0);

      QK_tap_count       : OUT std_logic_vector(5 DOWNTO 0);
      DQ_tap_count       : OUT std_logic_vector(5 DOWNTO 0);
      test_cal           : OUT std_logic_vector(127 DOWNTO 0)
   );
   END COMPONENT;


   SIGNAL delay_start_r1            : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL delay_start               : std_logic;

   SIGNAL idelay_ctrl_rdy           : std_logic;

   SIGNAL first_stage_done_r0       : std_logic;
   SIGNAL first_stage_done_r1       : std_logic;
   SIGNAL first_stage_done_r2       : std_logic;
   SIGNAL second_stage_begin_r1     : std_logic;
   SIGNAL second_stage_begin_r2     : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL qvld_cal_r1               : std_logic;
   SIGNAL qvld_cal_r2               : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL refresh_cmd_r1            : std_logic;
   SIGNAL refresh_cmd_r2            : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   --create 2 registers of the "fall" output of QVLD
   SIGNAL read_enable_fall_r1       : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL read_enable_fall_r2       : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL QK_tap_count              : std_logic_vector(2*NUM_OF_DEVS*6-1 DOWNTO 0);  -- 6-bit QK counts, expandable per device, 2 counts per device

   SIGNAL valid_tap_count           : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL first_cal_done            : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);  -- QK centered in DQ window

   SIGNAL wait_for_refresh          : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL qvld_cal_done             : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL idelay_ctrl_rdy1          : std_logic;
   SIGNAL idelay_ctrl_rdy2          : std_logic;

   SIGNAL test_cal                  : std_logic_vector(2*NUM_OF_DEVS*128-1 DOWNTO 0);  -- reserve 2 words of 128-bit per external device for debug
   SIGNAL test_qvld                 : std_logic_vector(NUM_OF_DEVS*36-1 DOWNTO 0);

   SIGNAL clk180                    : std_logic;
   SIGNAL clk270                    : std_logic;

   -- internal reset signal
   SIGNAL tap_sm_reset              : std_logic;  -- combine "rstHard_270 OR issueCalibration"


BEGIN
   cs_io <= (OTHERS => 'Z');
   qvld_idelay_inc    <= (others => '0');
   qvld_idelay_ce     <= (others => '0');
   qvld_idelay_rst    <= (others => '0');
   read_enable_out    <= (others => '0');

   clk180 <= NOT clk;
   clk270 <= NOT clk90;


   -- The following can be used for debugging
   cs_io(173 DOWNTO 100) <= test_cal(73 DOWNTO 0);   -- Test for Byte0
--   cs_io(99 DOWNTO 0)    <= (others => '0');         -- unused
   --
--   cs_io(255 DOWNTO 174) <= 
--      test_cal(432 DOWNTO 421) & test_cal(416 DOWNTO 415) & test_cal(412) & test_cal(395 DOWNTO 384) &
--      test_cal(304 DOWNTO 293) & test_cal(288 DOWNTO 287) & test_cal(284) & test_cal(267 DOWNTO 256) &
--      test_cal(176 DOWNTO 165) & test_cal(160 DOWNTO 159) & test_cal(156) & test_cal(139 DOWNTO 128);
--   cs_io(173 DOWNTO 0) <= (others => '0');           -- unused


   -- -----------------------------------------------------------------------------
   -- IDELAYCTRL instantiation
   -- -----------------------------------------------------------------------------
   idelayctrl0 : IDELAYCTRL 
   PORT MAP (
      RDY    => idelay_ctrl_rdy1,
      REFCLK => refClk,
      RST    => rstHard_refClk
   );

   idelayctrl1 : IDELAYCTRL 
   PORT MAP (
      RDY    => idelay_ctrl_rdy2,
      REFCLK => refClk,
      RST    => rstHard_refClk
   );


   -- Ready Signal transferred to system clock domain
   -- Uses 2 flops for metastability
   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         IF (rstHard = '1') THEN
            idelay_ctrl_rdy    <= '0';
            idelay_ctrl_rdy_r1 <= '0';
         ELSE
            idelay_ctrl_rdy    <= idelay_ctrl_rdy1 AND idelay_ctrl_rdy2;
            idelay_ctrl_rdy_r1 <= idelay_ctrl_rdy;
         END IF;
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- hold up calibration until start signal given and first read data is seen
   -- -----------------------------------------------------------------------------
   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard_270 = '1') THEN
            delay_start <= '0';
         ELSE
            IF ( (CTRL_DUMMYREAD_START AND read_enable_fall_r1(0)) = '1' ) THEN
               delay_start <= '1';
            ELSE
               delay_start <= delay_start;
            END IF;
         END IF;
      END IF;
   END PROCESS;

   dly_start_insts : FOR K IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
   BEGIN
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               delay_start_r1(K) <= '0';
            ELSE
               delay_start_r1(K) <= delay_start;
            END IF;
         END IF;
      END PROCESS;
   END GENERATE;


   -- -----------------------------------------------------------------------------
   -- Wait for all bytes to finish the first step in calibration before moving on
   -- -----------------------------------------------------------------------------
   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard_270 = '1') THEN
            first_stage_done_r0 <= '0';
         ELSE
            first_stage_done_r0 <= AND_REDUCE(first_cal_done(2*NUM_OF_DEVS-1 DOWNTO 0));
         END IF;
      END IF;
   END PROCESS;

   -- transfer from CLK270 to CLK90 Domain first
   PROCESS (clk90)
   BEGIN
      IF RISING_EDGE( clk90 ) THEN
         IF (rstHard_90 = '1') THEN
            first_stage_done_r1 <= '0';
         ELSE
            first_stage_done_r1 <= first_stage_done_r0;
         END IF;
      END IF;
   END PROCESS;

   -- transfer from CLK90 to CLK0 Domain
   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         IF (rstHard = '1') THEN
            first_stage_done_r2 <= '0';
         ELSE
            first_stage_done_r2 <= first_stage_done_r1;
         END IF;
      END IF;
   END PROCESS;

   first_stage_done <= first_stage_done_r2;


   -- -----------------------------------------------------------------------------
   -- Transfer clock domains for signal to start 2nd phase and align qvld signals
   -- ----------------------------------------------------------------------------- 
   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard_270 = '1') THEN
            second_stage_begin_r1 <= '0';
            qvld_cal_r1           <= '0';
         ELSE
            second_stage_begin_r1 <= second_stage_begin;
            qvld_cal_r1           <= qvld_cal;
         END IF;
      END IF;
   END PROCESS;

   second_stage_inst : FOR K IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
   BEGIN
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               second_stage_begin_r2(K) <= '0';
               qvld_cal_r2(K)           <= '0';
            ELSE
               second_stage_begin_r2(K) <= second_stage_begin_r1;
               qvld_cal_r2(K)           <= qvld_cal_r1;
            END IF;
         END IF;
      END PROCESS;
   END GENERATE;


   -- -----------------------------------------------------------------------------
   -- Hold up calibration while a refresh occurs
   -- use the QVLD fall output (less load) to ensure data has started up again
   -- ----------------------------------------------------------------------------- 
   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard_270 = '1') THEN
            refresh_cmd_r1 <= '0';
         ELSE
            refresh_cmd_r1 <= refresh_cmd;
         END IF;
      END IF;
   END PROCESS;

   wait_gen_inst : FOR K IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
   BEGIN
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               read_enable_fall_r1(K) <= '0';
               read_enable_fall_r2(K) <= '0';
               refresh_cmd_r2(K)      <= '0';
            ELSE
               read_enable_fall_r1(K) <= read_enable_fall(K);
               read_enable_fall_r2(K) <= read_enable_fall_r1(K);
               refresh_cmd_r2(K)      <= refresh_cmd_r1;
            END IF;
         END IF;
      END PROCESS;
      --
      wait_for_refresh(K) <= NOT read_enable_fall_r2(K) OR refresh_cmd_r2(K);
      --
   END GENERATE;


   -- -----------------------------------------------------------------------------
   -- rld_dly_cal_sm
   --   - Calibrates IDELAY tap values for DQ and QK inputs
   --   - Allows capture of read data with FPGA clock domain
   --   - Requires memory to constantly send back read data
   --   - Figures out how byte lines up to QVLD signal
   -- 
   -- Number of SM instances depends on the number of independent bytes
   -- -----------------------------------------------------------------------------
   tap_sm_reset <= rstHard_270 OR issueCalibration;

   byte_cal_gen_inst : FOR K IN  NUM_OF_DEVS-1 DOWNTO 0  GENERATE
   BEGIN
      -- even-byte Calibration
      tap_ctrl_e : rld_dly_cal_sm
      GENERIC MAP (
         DEV_DQ_WIDTH    => DEV_DQ_WIDTH,
         RL_CK_PERIOD    => RL_CK_PERIOD,
         QK_DATA_WIDTH   => QK_DATA_WIDTH
      )
      PORT MAP (
         clk               => clk270,
         rst               => tap_sm_reset,
         start_cal         => delay_start_r1(K),
         second_stage      => second_stage_begin_r2(K),
         qvld_cal          => qvld_cal_r2(K),
         wait_for_refresh  => wait_for_refresh(K),
         rd_data_fall      => rd_data_fall( (2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH ),  -- from IOBs
         rd_data_rise      => rd_data_rise( (2*K+1)*QK_DATA_WIDTH-1 DOWNTO (2*K)*QK_DATA_WIDTH ),  -- from IOBs
         rd_data_valid     => read_enable_rise(K),
         rd_data_valid_r   => read_enable_rise_r(K),

         dq_dly_rst        => data_idelay_rst(2*K),
         dq_dly_ce         => data_idelay_ce(2*K),
         dq_dly_inc        => data_idelay_inc(2*K),
         qk_dly_rst        => qk_idelay_rst(2*K),
         qk_dly_ce         => qk_idelay_ce(2*K),
         qk_dly_inc        => qk_idelay_inc(2*K),

         first_cal_done    => first_cal_done(2*K),
         dly_cal_done      => data_tap_select(2*K),
         delay_select      => delay_select(4*K+1 DOWNTO 4*K),

         QK_tap_count      => QK_tap_count((2*K+1)*6-1 DOWNTO 2*K*6),
         DQ_tap_count      => DQ_tap_count((2*K+1)*6-1 DOWNTO 2*K*6),
         test_cal          => test_cal((2*K+1)*128-1 DOWNTO 2*K*128)    -- debug signals
      );

      -- odd-byte Calibration
      tap_ctrl_o : rld_dly_cal_sm
      GENERIC MAP (
         DEV_DQ_WIDTH    => DEV_DQ_WIDTH,
         RL_CK_PERIOD    => RL_CK_PERIOD,
         QK_DATA_WIDTH   => QK_DATA_WIDTH
      )
      PORT MAP (
         clk               => clk270,
         rst               => tap_sm_reset,
         start_cal         => delay_start_r1(K),
         second_stage      => second_stage_begin_r2(K),
         qvld_cal          => qvld_cal_r2(K),
         wait_for_refresh  => wait_for_refresh(K),
         rd_data_fall      => rd_data_fall( (2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH ),  -- from IOBs
         rd_data_rise      => rd_data_rise( (2*K+2)*QK_DATA_WIDTH-1 DOWNTO (2*K+1)*QK_DATA_WIDTH ),  -- from IOBs
         rd_data_valid     => read_enable_rise(K),
         rd_data_valid_r   => read_enable_rise_r(K),
      
         dq_dly_rst        => data_idelay_rst(2*K+1),
         dq_dly_ce         => data_idelay_ce(2*K+1),
         dq_dly_inc        => data_idelay_inc(2*K+1),
         qk_dly_rst        => qk_idelay_rst(2*K+1),
         qk_dly_ce         => qk_idelay_ce(2*K+1),
         qk_dly_inc        => qk_idelay_inc(2*K+1),

         first_cal_done    => first_cal_done(2*K+1),
         dly_cal_done      => data_tap_select(2*K+1),
         delay_select      => delay_select(4*K+3 DOWNTO 4*K+2),

         QK_tap_count      => QK_tap_count((2*K+2)*6-1 DOWNTO (2*K+1)*6),
         DQ_tap_count      => DQ_tap_count((2*K+2)*6-1 DOWNTO (2*K+1)*6),
         test_cal          => test_cal((2*K+2)*128-1 DOWNTO (2*K+1)*128)    -- debug signals
      );
   END GENERATE;


END ARCHITECTURE rtl;
