--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_top.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: This is the RLDRAM-II top level module; the core itself.
--		It gathers data_path, controller, user_interface and iobs modules.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Pass RL_CK_PERIOD parameter to sub-modules, changed the read enable
--             for the read data FIFOs
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY rld_top IS
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE; -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD                   : integer := 3003;  -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF                    : std_logic_vector(2 DOWNTO 0) := "001";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH            : std_logic_vector(1 DOWNTO 0) := "00";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX                : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET               : std_logic := '0';    --
      RL_MRS_IMPEDANCE_MATCH         : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external
      RL_MRS_ODT                     : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00";   -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS                    : std_logic_vector(19 DOWNTO 0) := (others => '0')   -- saved location to perform calibration
   );
   PORT (
      -- global signals
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      sync_sysReset           : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      rstHard_refClk          : IN  std_logic;
      issueCalibration        : IN  std_logic;
      calibration_done        : OUT std_logic;
      rstConfig               : IN  std_logic;
      rstCmd                  : IN  std_logic_vector(3 DOWNTO 0);
      rstAd                   : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      -- RLDRAM-II interface signals
      phy_ck_p                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_ck_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dk_p                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_dk_n                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_qk_p                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_qk_n                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_a                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      phy_ba                  : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      phy_cs_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_we_n                : OUT std_logic;
      phy_ref_n               : OUT std_logic;
      phy_dm                  : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dq                  : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      phy_qvld                : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      -- application interface signals
      apRdfRdEn               : IN  std_logic;
      rldReadData             : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      rldReadDataValid        : OUT std_logic;
      apAddr                  : IN  std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      apValid                 : IN  std_logic;
      apWriteDM               : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      apWriteData             : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      apWriteDValid           : IN  std_logic;
      apConfA                 : IN  std_logic_vector(3 DOWNTO 0);
      apConfWrD               : IN  std_logic_vector(7 DOWNTO 0);
      apConfWr                : IN  std_logic;
      apConfRdD               : OUT std_logic_vector(7 DOWNTO 0);
      apConfRd                : IN  std_logic;
      rlWdfEmpty              : OUT std_logic;
      rlWdfFull               : OUT std_logic;
      rlRdfEmpty              : OUT std_logic;
      rlafEmpty               : OUT std_logic;
      rlafFull                : OUT std_logic;
      rlWdfWrCount            : OUT std_logic_vector(12 DOWNTO 0);   --  write data FIFO (wdfifo) write count
      rlWdfWordCount          : OUT std_logic_vector(12 DOWNTO 0);   --  write data FIFO (wdfifo) write count
      rlafWrCount             : OUT std_logic_vector(12 DOWNTO 0);   --  command/address FIFO (rafifo) write count
      rlafWordCount           : OUT std_logic_vector(12 DOWNTO 0);   --  command/address FIFO (rafifo) word count
      Init_Done               : IN  std_logic;
      issueMRS                : IN  std_logic;
      confMReg                : OUT std_logic_vector(17 DOWNTO 0);
      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY rld_top;



ARCHITECTURE rtl OF rld_top IS


   -- Conditional Constant, based on one "equal to" condition
   function CC (COND_EXPR:INTEGER; COND_VAL:INTEGER; THEN_VAL:INTEGER; ELSE_VAL:INTEGER) return INTEGER is 
   begin 
      if COND_EXPR = COND_VAL then
         return THEN_VAL; 
      else
         return ELSE_VAL; 
      end if; 
   end;

   CONSTANT QK_DATA_WIDTH     : integer := CC(DEV_DQ_WIDTH, 9, DEV_DQ_WIDTH, DEV_DQ_WIDTH/2);
   

   COMPONENT rld_user_interface
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      QK_DATA_WIDTH           : integer := 9;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      apWriteDM               : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      apWriteData             : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      apWriteDValid           : IN  std_logic;
      ctlWdfRdEn              : IN  std_logic;
      apAddr                  : IN  std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      apValid                 : IN  std_logic;
      ctlafRdEn               : IN  std_logic;
      apRdfRdEn               : IN  std_logic;
      dpRdData                : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      wren_RdFifo             : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      afA                     : OUT std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      wdfDM                   : OUT std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      wdfD                    : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      rlWdfEmpty              : OUT std_logic;
      rlWdfFull               : OUT std_logic;
      rldReadData             : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      rldReadDataValid        : OUT std_logic;
      rlafEmpty               : OUT std_logic;
      rlafFull                : OUT std_logic;
      rlafAlmostEmpty         : OUT std_logic;
      rlafAlmostFull          : OUT std_logic;
      rlRdfEmpty              : OUT std_logic;
      rlRdfFull               : OUT std_logic;
      rlWdfAlmostEmpty        : OUT std_logic;
      rlWdfAlmostFull         : OUT std_logic;
      rlWdfWrCount            : OUT std_logic_vector(12 DOWNTO 0);   -- write data FIFO (wdfifo) write count
      rlWdfWordCount          : OUT std_logic_vector(12 DOWNTO 0);   -- write data FIFO (wdfifo) word count
      rlafWrCount             : OUT std_logic_vector(12 DOWNTO 0);   -- command/address FIFO (rafifo) write count
      rlafWordCount           : OUT std_logic_vector(12 DOWNTO 0);   -- command/address FIFO (rafifo) word count
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_data_path
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      QK_DATA_WIDTH                  : integer := 9;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE;   -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD                   : integer := 3003; -- CK clock period of the RLDRAM in ps
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      rstHard_refClk          : IN  std_logic;
      qk_idelay_inc           : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_ce            : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_rst           : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_inc         : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_ce          : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_rst         : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_inc         : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_ce          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_rst         : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rd_data_rise            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rd_data_fall            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      okToSelTap              : IN  std_logic;
      ctlIoWdEn               : IN  std_logic;
      wdfDM                   : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      wdfD                    : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_rise            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_fall            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_en_clk90             : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_rise          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_fall          : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      CTRL_DUMMYREAD_START    : IN  std_logic;
      refresh_cmd             : IN  std_logic;
      CTRL_DQS_RST            : IN  std_logic;
      CTRL_DQS_EN             : IN  std_logic;
      read_enable_rise        : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      read_enable_fall        : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      first_stage_done        : OUT std_logic;
      second_stage_begin      : IN  std_logic;
      qvld_cal                : IN  std_logic;
      SEL_DONE                : OUT std_logic;
      issueCalibration        : IN  std_logic;
      read_en_delayed         : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      read_data_rise          : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      read_data_fall          : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      idelay_ctrl_rdy         : OUT std_logic;
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_iobs
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE;   -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00"    -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      refClk                  : IN  std_logic;
      sync_sysReset           : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      phy_ck_p                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_ck_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dk_p                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_dk_n                : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      phy_qk_p                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_qk_n                : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      phy_a                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      phy_ba                  : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      phy_cs_n                : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_we_n                : OUT std_logic;
      phy_ref_n               : OUT std_logic;
      phy_dm                  : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      phy_dq                  : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      phy_qvld                : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_inc           : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_ce            : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qk_idelay_rst           : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_inc         : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_ce          : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      data_idelay_rst         : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_inc         : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_ce          : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      qvld_idelay_rst         : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      wr_data_rise            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_data_fall            : IN  std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      wr_en_clk90             : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_rise          : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      data_mask_fall          : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      rd_data_rise            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      rd_data_fall            : OUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      ctlCmd                  : IN  std_logic_vector(3 DOWNTO 0);
      ctlAd                   : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      ctlBa                   : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      read_enable_rise        : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      read_enable_fall        : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_controller
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD            : integer := 3003;   -- CK clock period of the RLDRAM in ps
      -- specific to rld_conf 
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "010";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '0';    -- 
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      -- Global signals
      clk                     : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstConfig               : IN  std_logic;
      afEmpty                 : IN  std_logic;
      afA                     : IN  std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      -- Reset configuration signals from reset module
      rstCmd                  : IN  std_logic_vector(3 DOWNTO 0);
      rstAd                   : IN  std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      rstBa                   : IN  std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      ctlafRdEn               : OUT std_logic;
      -- write data enable for pads
      ctlIoWdEn               : OUT std_logic;
      -- commands and address, bank outputs
      ctlCmd                  : OUT std_logic_vector(3 DOWNTO 0);
      ctlAd                   : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      -- write data fifo read
      ctlWdfRdEn              : OUT std_logic;
      ctlBa                   : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      -- indicates okay to change data valid edge for per-bit deskew
      ctlOkEdge               : OUT std_logic;
      rldWriteDone            : OUT std_logic;
      apWdfWrEn               : IN  std_logic;
      wdfAlmostFull           : OUT std_logic;
      apConfRd                : IN  std_logic;
      apConfWrD               : IN  std_logic_vector(7 DOWNTO 0);
      apConfRdD               : OUT std_logic_vector(7 DOWNTO 0);
      apConfWr                : IN  std_logic;
      apConfA                 : IN  std_logic_vector(3 DOWNTO 0);
      confMReg                : OUT std_logic_vector(17 DOWNTO 0);
      confBL                  : OUT std_logic_vector(3 DOWNTO 0);
      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;


   COMPONENT rld_phy_calib
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    --  number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DUPLICATE_CONTROLS             : boolean := FALSE; -- Duplicate the ports for A, BA, WE# and REF#
      BITS_PER_BYTE                  : integer := 9;     -- Micron offers 9-bit byte for RL2
      -- specific rld_conf parameters
      RL_CK_PERIOD                   : integer := 3003;  -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF                    : std_logic_vector(2 DOWNTO 0) := "001";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH            : std_logic_vector(1 DOWNTO 0) := "00";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX                : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET               : std_logic := '0';    --
      RL_MRS_IMPEDANCE_MATCH         : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external
      RL_MRS_ODT                     : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE                     : std_logic_vector(1 DOWNTO 0) := "00";   -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH                    : std_logic_vector(1 DOWNTO 0) := "00";   -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD                 : std_logic_vector(1 DOWNTO 0) := "00";   -- Direct Clocking="00"  SerDes="01"
      -- specific to calibration scheme
      CAL_ADDRESS                    : std_logic_vector(19 DOWNTO 0) := (others => '0')   -- saved location to perform calibration
   );
   PORT (
      clk                     : IN  std_logic;
      rstHard                 : IN  std_logic;
      confBL                  : IN  std_logic_vector(3 DOWNTO 0);  -- BL2=X"2", BL4=X"4", BL8=X"8"
      start                   : IN  std_logic;   
      first_sel_done          : IN  std_logic;  -- from phy, first calibration stage complete
      sel_done                : IN  std_logic;  -- calibration done
      refresh_cmd             : IN  std_logic;  -- for BL2 wait for refresh before aligning QVLD
      issueCalibration        : IN  std_logic;  
      rlWdfFull               : IN  std_logic;  -- write data fifo full
      rlWdfEmpty              : IN  std_logic;  -- write data fifo empty
      rlafFull                : IN  std_logic;  -- write address fifo full
      rlafAlmostEmpty         : IN  std_logic;  -- write address fifo almost full
      rlafEmpty               : IN  std_logic;  -- write address fifo empty
      second_cal_out          : OUT std_logic;  -- signal physical layer to begin 2nd phase cal
      qvld_cal_out            : OUT std_logic;  -- signal physical layer to align QVLD
      cal_done                : OUT std_logic;  -- calibration complete
      apValid                 : OUT std_logic;  -- write enable to write address fifo
      apAddr                  : OUT std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);   -- address to be written to write address fifo
      apWriteDValid           : OUT std_logic;  -- write enable to write data fifo
      apWriteData             : OUT std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);  -- data to be written to write data fifo
      apWriteDM               : OUT std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);  -- data mask to be written to write data fifo
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)   -- chipscope debug ports
   );
   END COMPONENT;


   -- signals
   SIGNAL afA                      : std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
   SIGNAL ctlafRdEn                : std_logic;
   SIGNAL ctlIoWdEn                : std_logic;
   SIGNAL ctlWdfRdEn               : std_logic;
   SIGNAL ctlOkEdge                : std_logic;
   SIGNAL confBL                   : std_logic_vector(3 DOWNTO 0);
   SIGNAL wdfD                     : std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
   SIGNAL wdfDM                    : std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
   SIGNAL wr_data_rise             : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL wr_data_fall             : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL wr_en_clk90              : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL data_mask_rise           : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL data_mask_fall           : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL dpRdData                 : std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);

   SIGNAL rlRdfFull                : std_logic;
   SIGNAL rldWriteDone             : std_logic;

   SIGNAL first_sel_done           : std_logic;   --  first stage of calibration complete
   SIGNAL second_cal               : std_logic;   --  second stage of calibration can begin
   SIGNAL qvld_cal                 : std_logic;   --  qvld calibration can be performed
   SIGNAL sel_done                 : std_logic;   --  completed QK and DQ calibration tap adjustments
   SIGNAL ctlCmd                   : std_logic_vector(3 DOWNTO 0);
   SIGNAL ctlAd                    : std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
   SIGNAL ctlBa                    : std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);

   SIGNAL wren_RdFifo              : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL dpRdData_rise            : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL dpRdData_fall            : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

   SIGNAL qk_idelay_inc            : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL qk_idelay_ce             : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL qk_idelay_rst            : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL data_idelay_inc          : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL data_idelay_ce           : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL data_idelay_rst          : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL qvld_idelay_inc          : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL qvld_idelay_ce           : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL qvld_idelay_rst          : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL rd_data_rise             : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL rd_data_fall             : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

   SIGNAL FIRST_RISING             : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL wdfAlmostFull            : std_logic;
   SIGNAL rlWdfAlmostFull          : std_logic;
   SIGNAL rlWdfAlmostEmpty         : std_logic;
   SIGNAL rlWdfFull_1              : std_logic;
   SIGNAL read_enable_rise         : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL read_enable_fall         : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL apWriteDValid_dummy      : std_logic;
   SIGNAL apWriteDValid_m          : std_logic;
   SIGNAL apValid_dummy            : std_logic;
   SIGNAL apValid_m                : std_logic;

   SIGNAL start                    : std_logic;
   SIGNAL reg_operation            : std_logic;   --  used for mux control based on operation

   SIGNAL apAddr_dummy             : std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
   SIGNAL apAddr_m                 : std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
   SIGNAL apWriteData_dummy        : std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
   SIGNAL apWriteData_m            : std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
   SIGNAL apWriteDM_dummy          : std_logic_vector((2*NUM_OF_DEVS-1) DOWNTO 0);
   SIGNAL apWriteDM_m              : std_logic_vector((2*NUM_OF_DEVS-1) DOWNTO 0);

   SIGNAL wren_RdFifo_mux          : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);

   SIGNAL qk_bufio                 : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL idelay_ctrl_rdy          : std_logic;

   SIGNAL clk180                   : std_logic;
   SIGNAL clk270                   : std_logic;

   SIGNAL refresh_cmd              : std_logic;

   SIGNAL calibration_done_r       : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL read_enable_rise_UI_r1   : std_logic;

   -- internal signals
   SIGNAL calibration_done_sig     : std_logic;
   SIGNAL rlWdfEmpty_sig           : std_logic;
   SIGNAL rlWdfFull_sig            : std_logic;
   SIGNAL rlafEmpty_sig            : std_logic;
   SIGNAL rlafFull_sig             : std_logic;

   SIGNAL rlafAlmostEmpty          : std_logic;
   SIGNAL rlafAlmostFull           : std_logic;

   SIGNAL not_rstConfig            : std_logic;


BEGIN
   cs_io <= (OTHERS => 'Z');
   not_rstConfig     <= NOT rstConfig;   -- required for ModelSim: "not a globally static expression"...

   -- internal signal assignments
   calibration_done  <= calibration_done_sig;

   rlWdfEmpty  <= rlWdfEmpty_sig;
   rlWdfFull   <= rlWdfFull_sig;
   rlafEmpty   <= rlafEmpty_sig;
   rlafFull    <= rlafFull_sig;


   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard = '1') THEN
            read_enable_rise_UI_r1 <= '0';
         ELSE
            IF (calibration_done_sig = '1') THEN
               read_enable_rise_UI_r1 <= read_enable_rise(0);
            ELSE
               read_enable_rise_UI_r1 <= '0';
            END IF;
         END IF;
      END IF;
   END PROCESS;

   clk180 <= NOT clk;
   clk270 <= NOT clk90;


   dpRdData((2*RL_DQ_WIDTH)-1 DOWNTO RL_DQ_WIDTH) <= dpRdData_rise;
   dpRdData(RL_DQ_WIDTH-1 DOWNTO 0)               <= dpRdData_fall;
   rlWdfFull_sig                                  <= rlWdfFull_1 OR wdfAlmostFull;
   
   apWriteDValid_m  <= apWriteDValid WHEN (reg_operation = '1') ELSE apWriteDValid_dummy;
   apValid_m        <= apValid       WHEN (reg_operation = '1') ELSE apValid_dummy;

   apAddr_m       <= apAddr        WHEN (reg_operation = '1') ELSE apAddr_dummy;
   apWriteData_m  <= apWriteData   WHEN (reg_operation = '1') ELSE apWriteData_dummy;
   apWriteDM_m    <= apWriteDM     WHEN (reg_operation = '1') ELSE apWriteDM_dummy;


   -- -----------------------------------------------------------------------------	
   -- Transfer clock domains for the 'calibration_done' signal (clk0->clk270)
   -- this signal used to MUX write_enable to the FIFOs (clk270)
   -- -----------------------------------------------------------------------------	
   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         IF (rstHard = '1') THEN
            calibration_done_sig <= '0';
         ELSE
            calibration_done_sig <= reg_operation;
         END IF;
      END IF;
   END PROCESS;

   cal_done_r_insts : FOR K IN calibration_done_r'RANGE GENERATE
   BEGIN
      PROCESS (clk270)
      BEGIN
         IF RISING_EDGE( clk270 ) THEN
            IF (rstHard_270 = '1') THEN
               calibration_done_r(K) <= '0';
            ELSE
               calibration_done_r(K) <= calibration_done_sig;
            END IF;
         END IF;
      END PROCESS;

      wren_RdFifo_mux(2*K)   <= wren_RdFifo(2*K)   WHEN ( calibration_done_r(K) = '1') ELSE '0';
      wren_RdFifo_mux(2*K+1) <= wren_RdFifo(2*K+1) WHEN ( calibration_done_r(K) = '1') ELSE '0';
   END GENERATE;


   -- -----------------------------------------------------------------------------	
   -- flag the refresh command so calibration can wait until refresh complete
   -- -----------------------------------------------------------------------------
   --    Signal "ctlCmd" holds control bits for CS#, WE# and REF#
   --       ctlCmd(3) : CS#
   --       ctlCmd(2) : <unused>  (was AS#)
   --       ctlCmd(1) : WE#
   --       ctlCmd(0) : REF#
   --    So ctlCmd = "0110" means CS#=0, unused=1, WE#=1, REF#=0  [Refresh command]
   -- -----------------------------------------------------------------------------	
   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         IF (rstHard = '1') THEN
            refresh_cmd <= '0';
         ELSE
            IF (ctlCmd = "0110") THEN   -- Refresh command
               refresh_cmd <= '1';
            ELSE
               refresh_cmd <= '0';
            END IF;
         END IF;
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- rld_user_interface
   --   Instantiates read fifo, write fifo and merged fifos.
   --   All user interface signals: 
   --     - write data
   --     - write address
   --     - read data
   --     - FIFO status signals
   -- -----------------------------------------------------------------------------
   user_interface0 : rld_user_interface
   GENERIC MAP (
      RL_DQ_WIDTH         => RL_DQ_WIDTH,
      DEV_DQ_WIDTH        => DEV_DQ_WIDTH,
      NUM_OF_DEVS         => NUM_OF_DEVS,
      NUM_OF_DKS          => NUM_OF_DKS,
      QK_DATA_WIDTH       => QK_DATA_WIDTH,
      DEV_AD_WIDTH        => DEV_AD_WIDTH,
      DEV_BA_WIDTH        => DEV_BA_WIDTH,
      APP_AD_WIDTH        => APP_AD_WIDTH,
      DUPLICATE_CONTROLS  => DUPLICATE_CONTROLS,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE          => RL_IO_TYPE,
      DEVICE_ARCH         => DEVICE_ARCH,
      CAPTURE_METHOD      => CAPTURE_METHOD
   )
   PORT MAP (
      clk               => clk,
      clk90             => clk90,
      rstHard           => rstHard,
      rstHard_90        => rstHard_90,
      rstHard_180       => rstHard_180,
      rstHard_270       => rstHard_270,

      apWriteDM         => apWriteDM_m,
      apWriteData       => apWriteData_m,
      apWriteDValid     => apWriteDValid_m,

      ctlWdfRdEn        => ctlWdfRdEn,

      apAddr            => apAddr_m,          -- output address
      apValid           => apValid_m,         -- address fifo read enable

      ctlafRdEn         => ctlafRdEn,
--      apRdfRdEn         => apRdfRdEn,
      apRdfRdEn         => read_enable_rise_UI_r1,

      dpRdData          => dpRdData,

      wren_RdFifo       => wren_RdFifo_mux,

      afA               => afA,
      wdfDM             => wdfDM,
      wdfD              => wdfD,
      rlWdfEmpty        => rlWdfEmpty_sig,
      rlWdfFull         => rlWdfFull_1,
      rldReadData       => rldReadData,
      rldReadDataValid  => rldReadDataValid,
      rlafEmpty         => rlafEmpty_sig,
      rlafFull          => rlafFull_sig,
      rlafAlmostEmpty   => rlafAlmostEmpty,
      rlafAlmostFull    => rlafAlmostFull,
      rlRdfEmpty        => rlRdfEmpty,
      rlRdfFull         => rlRdfFull,
      rlWdfAlmostEmpty  => rlWdfAlmostEmpty,
      rlWdfAlmostFull   => rlWdfAlmostFull,
      rlWdfWrCount      => rlWdfWrCount,      -- write data FIFO (wdfifo) write count
      rlWdfWordCount    => rlWdfWordCount,    -- write data FIFO (wdfifo) word count
      rlafWrCount       => rlafWrCount,       -- command/address FIFO (rafifo) write count
      rlafWordCount     => rlafWordCount,     -- command/address FIFO (rafifo) word count

      cs_io             => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- rld_data_path
   --   Instantiates tap_logic (calibration) and data_write (data mask) modules
   -- -----------------------------------------------------------------------------
   data_path0 : rld_data_path
   GENERIC MAP (
      RL_DQ_WIDTH         => RL_DQ_WIDTH,
      DEV_DQ_WIDTH        => DEV_DQ_WIDTH,
      NUM_OF_DEVS         => NUM_OF_DEVS,
      NUM_OF_DKS          => NUM_OF_DKS,
      QK_DATA_WIDTH       => QK_DATA_WIDTH,
      DEV_AD_WIDTH        => DEV_AD_WIDTH,
      DEV_BA_WIDTH        => DEV_BA_WIDTH,
      APP_AD_WIDTH        => APP_AD_WIDTH,
      DUPLICATE_CONTROLS  => DUPLICATE_CONTROLS,
      RL_CK_PERIOD        => RL_CK_PERIOD,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE          => RL_IO_TYPE,
      DEVICE_ARCH         => DEVICE_ARCH,
      CAPTURE_METHOD      => CAPTURE_METHOD
   )
   PORT MAP (
      clk                   => clk,
      clk90                 => clk90,
      refClk                => refClk,
      rstHard               => rstHard,
      rstHard_90            => rstHard_90,
      rstHard_180           => rstHard_180,
      rstHard_270           => rstHard_270,
      rstHard_refClk        => rstHard_refClk,

      qk_idelay_inc         => qk_idelay_inc,
      qk_idelay_ce          => qk_idelay_ce,
      qk_idelay_rst         => qk_idelay_rst,
      data_idelay_inc       => data_idelay_inc,
      data_idelay_ce        => data_idelay_ce,
      data_idelay_rst       => data_idelay_rst,
      qvld_idelay_inc       => qvld_idelay_inc,
      qvld_idelay_ce        => qvld_idelay_ce,
      qvld_idelay_rst       => qvld_idelay_rst,

      rd_data_rise          => rd_data_rise,
      rd_data_fall          => rd_data_fall,

      okToSelTap            => ctlOkEdge,

      ctlIoWdEn             => ctlIoWdEn,
      wdfDM                 => wdfDM,
      wdfD                  => wdfD,
      wr_data_rise          => wr_data_rise,
      wr_data_fall          => wr_data_fall,
      wr_en_clk90           => wr_en_clk90,
      data_mask_rise        => data_mask_rise,
      data_mask_fall        => data_mask_fall,

      CTRL_DUMMYREAD_START  => not_rstConfig,
      refresh_cmd           => refresh_cmd,
      CTRL_DQS_RST          => '0',
      CTRL_DQS_EN           => '0',
      read_enable_rise      => read_enable_rise,
      read_enable_fall      => read_enable_fall,
      first_stage_done      => first_sel_done,
      second_stage_begin    => second_cal,
      qvld_cal              => qvld_cal,
      SEL_DONE              => sel_done,
      issueCalibration      => issueCalibration,

      read_en_delayed       => wren_RdFifo,
      read_data_rise        => dpRdData_rise,
      read_data_fall        => dpRdData_fall,

      idelay_ctrl_rdy       => idelay_ctrl_rdy,

      cs_io                 => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- rld_iobs
   --   Top level for the physical layer (PHY).
   --   Instantiates the following modules:
   --     - data_path_iobs      : QK, DQ, QVLD and DM
   --     - controller_iobs     : A, BA, WE#, REF# and CS#
   --     - infrastructure_iobs : DK and CK
   -- -----------------------------------------------------------------------------
   iobs0 : rld_iobs
   GENERIC MAP (
      RL_DQ_WIDTH         => RL_DQ_WIDTH,
      DEV_DQ_WIDTH        => DEV_DQ_WIDTH,
      NUM_OF_DEVS         => NUM_OF_DEVS,
      NUM_OF_DKS          => NUM_OF_DKS,
      DEV_AD_WIDTH        => DEV_AD_WIDTH,
      DEV_BA_WIDTH        => DEV_BA_WIDTH,
      APP_AD_WIDTH        => APP_AD_WIDTH,
      DUPLICATE_CONTROLS  => DUPLICATE_CONTROLS,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE          => RL_IO_TYPE,
      DEVICE_ARCH         => DEVICE_ARCH,
      CAPTURE_METHOD      => CAPTURE_METHOD
   )
   PORT MAP (
      clk               => clk,
      clk90             => clk90,
      refClk            => refClk,
      sync_sysReset     => sync_sysReset,
      rstHard           => rstHard,
      rstHard_90        => rstHard_90,
      rstHard_180       => rstHard_180,
      rstHard_270       => rstHard_270,

      phy_ck_p          => phy_ck_p,
      phy_ck_n          => phy_ck_n,
      phy_dk_p          => phy_dk_p,
      phy_dk_n          => phy_dk_n,
      phy_qk_p          => phy_qk_p,
      phy_qk_n          => phy_qk_n,
      phy_a             => phy_a,
      phy_ba            => phy_ba,
      phy_cs_n          => phy_cs_n,
      phy_we_n          => phy_we_n,
      phy_ref_n         => phy_ref_n,
      phy_dm            => phy_dm,
      phy_dq            => phy_dq,
      phy_qvld          => phy_qvld,

      qk_idelay_inc     => qk_idelay_inc,
      qk_idelay_ce      => qk_idelay_ce,
      qk_idelay_rst     => qk_idelay_rst,
      data_idelay_inc   => data_idelay_inc,
      data_idelay_ce    => data_idelay_ce,
      data_idelay_rst   => data_idelay_rst,
      qvld_idelay_inc   => qvld_idelay_inc,
      qvld_idelay_ce    => qvld_idelay_ce,
      qvld_idelay_rst   => qvld_idelay_rst,

      wr_data_rise      => wr_data_rise,
      wr_data_fall      => wr_data_fall,
      wr_en_clk90       => wr_en_clk90,
      data_mask_rise    => data_mask_rise,
      data_mask_fall    => data_mask_fall,

      rd_data_rise      => rd_data_rise,
      rd_data_fall      => rd_data_fall,

      ctlCmd            => ctlCmd,
      ctlAd             => ctlAd,
      ctlBa             => ctlBa,

      read_enable_rise  => read_enable_rise,
      read_enable_fall  => read_enable_fall,

      cs_io             => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- rld_controller
   --   Instantiates the control and configuration modules:
   --     * Control module has the main control logic
   --     * Includes refresh logic, write logic and read logic
   --     * config module has the parameters for config register
   -- -----------------------------------------------------------------------------
   controller0 : rld_controller
   GENERIC MAP (
      RL_DQ_WIDTH             => RL_DQ_WIDTH,
      DEV_DQ_WIDTH            => DEV_DQ_WIDTH,
      NUM_OF_DEVS             => NUM_OF_DEVS,
      NUM_OF_DKS              => NUM_OF_DKS,
      DEV_AD_WIDTH            => DEV_AD_WIDTH,
      DEV_BA_WIDTH            => DEV_BA_WIDTH,
      APP_AD_WIDTH            => APP_AD_WIDTH,
      DUPLICATE_CONTROLS      => DUPLICATE_CONTROLS,
      RL_CK_PERIOD            => RL_CK_PERIOD,
      -- specific to rld_conf 
      RL_MRS_CONF             => RL_MRS_CONF,
      RL_MRS_BURST_LENGTH     => RL_MRS_BURST_LENGTH,
      RL_MRS_ADDR_MUX         => RL_MRS_ADDR_MUX,
      RL_MRS_DLL_RESET        => RL_MRS_DLL_RESET,
      RL_MRS_IMPEDANCE_MATCH  => RL_MRS_IMPEDANCE_MATCH,
      RL_MRS_ODT              => RL_MRS_ODT,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              => RL_IO_TYPE,
      DEVICE_ARCH             => DEVICE_ARCH,
      CAPTURE_METHOD          => CAPTURE_METHOD
   )
   PORT MAP (
      clk            => clk,
      rstHard        => rstHard,
      rstHard_180    => rstHard_180,
      rstConfig      => rstConfig,

      afEmpty        => rlafEmpty_sig,
      afA            => afA,

      rstCmd         => rstCmd,
      rstAd          => rstAd,
      rstBa          => rstBa,

      ctlafRdEn      => ctlafRdEn,         -- address fifo read enable    
      ctlIoWdEn      => ctlIoWdEn,
      ctlCmd         => ctlCmd,
      ctlAd          => ctlAd,
      ctlWdfRdEn     => ctlWdfRdEn,        -- write data fifo read enable   
      ctlBa          => ctlBa,
      ctlOkEdge      => ctlOkEdge,
      rldWriteDone   => rldWriteDone,

      apWdfWrEn      => apWriteDValid_m,   -- used to count transfers to fifo
      wdfAlmostFull  => wdfAlmostFull,

      apConfRd       => apConfRd,
      apConfWrD      => apConfWrD,
      apConfRdD      => apConfRdD,
      apConfWr       => apConfWr,
      apConfA        => apConfA,
      confMReg       => confMReg,
      confBL         => confBL,

      cs_io          => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- rld_phy_calib
   --   Creates dummy read and write data for phy calibration, 
   --   writing data to FIFOs for processing by the controller.
   --   Supports BL=2 and BL=4.
   -- -----------------------------------------------------------------------------
   start <= NOT rstConfig AND idelay_ctrl_rdy;

   phy_calib0 : rld_phy_calib
   GENERIC MAP (
      RL_DQ_WIDTH             => RL_DQ_WIDTH,
      DEV_DQ_WIDTH            => DEV_DQ_WIDTH,
      NUM_OF_DEVS             => NUM_OF_DEVS,
      NUM_OF_DKS              => NUM_OF_DKS,
      DEV_AD_WIDTH            => DEV_AD_WIDTH,
      DEV_BA_WIDTH            => DEV_BA_WIDTH,
      APP_AD_WIDTH            => APP_AD_WIDTH,
      DUPLICATE_CONTROLS      => DUPLICATE_CONTROLS,
      RL_CK_PERIOD            => RL_CK_PERIOD,
      -- specific to rld_conf 
      RL_MRS_CONF             => RL_MRS_CONF,
      RL_MRS_BURST_LENGTH     => RL_MRS_BURST_LENGTH,
      RL_MRS_ADDR_MUX         => RL_MRS_ADDR_MUX,
      RL_MRS_DLL_RESET        => RL_MRS_DLL_RESET,
      RL_MRS_IMPEDANCE_MATCH  => RL_MRS_IMPEDANCE_MATCH,
      RL_MRS_ODT              => RL_MRS_ODT,
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              => RL_IO_TYPE,
      DEVICE_ARCH             => DEVICE_ARCH,
      CAPTURE_METHOD          => CAPTURE_METHOD,
      -- specific to calibration scheme
      CAL_ADDRESS             => CAL_ADDRESS
   )
   PORT MAP (
      clk               => clk,
      rstHard           => rstHard,
      confBL            => confBL,
      start             => start,
      first_sel_done    => first_sel_done,
      sel_done          => sel_done,
      refresh_cmd       => refresh_cmd,
      issueCalibration  => issueCalibration,

      rlWdfFull         => rlWdfFull_sig,
      rlWdfEmpty        => rlWdfEmpty_sig,
      rlafFull          => rlafFull_sig,
      rlafAlmostEmpty   => rlafAlmostEmpty,
      rlafEmpty         => rlafEmpty_sig,

      second_cal_out    => second_cal,
      qvld_cal_out      => qvld_cal,
      cal_done          => reg_operation,
      apValid           => apValid_dummy,
      apAddr            => apAddr_dummy,
      apWriteDValid     => apWriteDValid_dummy,
      apWriteData       => apWriteData_dummy,
      apWriteDM         => apWriteDM_dummy,

      cs_io             => cs_io
   );


END ARCHITECTURE rtl;
