--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: rld_user_interface.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	 Virtex-5
--	Purpose: This module instantiates the read fifo, write fifo and 
--		merged fifos. It has all the user interface signals, like
--		write data, write address, read data and FIFO status signals.
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Added support for FIFO generation for 72-bit width
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library UNISIM;
use UNISIM.VCOMPONENTS.ALL;

ENTITY rld_user_interface IS
   GENERIC (
      RL_DQ_WIDTH             : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;    -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      QK_DATA_WIDTH           : integer := 9;
      DEV_AD_WIDTH            : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01"   -- Direct Clocking="00"  SerDes="01"
   );
   PORT (
      clk                     : IN  std_logic;
      clk90                   : IN  std_logic;
      rstHard                 : IN  std_logic;
      rstHard_90              : IN  std_logic;
      rstHard_180             : IN  std_logic;
      rstHard_270             : IN  std_logic;
      apWriteDM               : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      apWriteData             : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      apWriteDValid           : IN  std_logic;
      ctlWdfRdEn              : IN  std_logic;
      apAddr                  : IN  std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      apValid                 : IN  std_logic;
      ctlafRdEn               : IN  std_logic;
      apRdfRdEn               : IN  std_logic;
      dpRdData                : IN  std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      wren_RdFifo             : IN  std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      afA                     : OUT std_logic_vector(APP_AD_WIDTH-1 DOWNTO 0);
      wdfDM                   : OUT std_logic_vector((2*NUM_OF_DEVS)-1 DOWNTO 0);
      wdfD                    : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      rlWdfEmpty              : OUT std_logic;
      rlWdfFull               : OUT std_logic;
      rldReadData             : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      rldReadDataValid        : OUT std_logic;
      rlafEmpty               : OUT std_logic;
      rlafFull                : OUT std_logic;
      rlafAlmostEmpty         : OUT std_logic;
      rlafAlmostFull          : OUT std_logic;
      rlRdfEmpty              : OUT std_logic;
      rlRdfFull               : OUT std_logic;
      rlWdfAlmostEmpty        : OUT std_logic;
      rlWdfAlmostFull         : OUT std_logic;
      rlWdfWrCount            : OUT std_logic_vector(12 DOWNTO 0);   -- write data FIFO (wdfifo) write count
      rlWdfWordCount          : OUT std_logic_vector(12 DOWNTO 0);   -- write data FIFO (wdfifo) word count
      rlafWrCount             : OUT std_logic_vector(12 DOWNTO 0);   -- command/address FIFO (rafifo) write count
      rlafWordCount           : OUT std_logic_vector(12 DOWNTO 0);   -- command/address FIFO (rafifo) word count
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY rld_user_interface;



ARCHITECTURE rtl OF rld_user_interface IS

   function OR_REDUCE(ARG: in STD_LOGIC_VECTOR) return std_logic is
      variable result: STD_LOGIC := '0';
   begin
      for i in ARG'range loop
         result := result OR ARG(i);
      end loop;
      return result;
   end function OR_REDUCE;

   function AND_REDUCE(ARG: in STD_LOGIC_VECTOR) return std_logic is
      variable result: STD_LOGIC := '1';
   begin
      for i in ARG'range loop
         result := result AND ARG(i);
      end loop;
      return result;
   end function AND_REDUCE;

      -- Conditional Constant, based on one "equal to" condition
   function CC (COND_EXPR:INTEGER; COND_VAL:INTEGER; THEN_VAL:INTEGER; ELSE_VAL:INTEGER) return INTEGER is 
   begin 
      if COND_EXPR = COND_VAL then
         return THEN_VAL; 
      else
         return ELSE_VAL; 
      end if; 
   end;

   -- Conditional Constant, based on two "less than" conditions
   function CC_LT2 (COND_EXPR1:INTEGER; COND_VAL1:INTEGER; THEN_VAL1:INTEGER; 
                    COND_EXPR2:INTEGER; COND_VAL2:INTEGER; THEN_VAL2:INTEGER; 
                    ELSE_VAL:INTEGER) return INTEGER is 
   begin
      if COND_EXPR1 < COND_VAL1 then
         return THEN_VAL1;
      else
         if COND_EXPR2 < COND_VAL2 then
           return THEN_VAL2;
         else
            return ELSE_VAL;
         end if;
      end if;
   end;
   
   CONSTANT NUM_OF_FIFOS           : integer := CC(RL_DQ_WIDTH, 36, 2, 1);

   SIGNAL rlWdfRdCount             : std_logic_vector(12 DOWNTO 0);  -- write data FIFO (wdfifo) read count
   SIGNAL rlafRdCount              : std_logic_vector(12 DOWNTO 0);  -- command/address FIFO (rafifo) read count
   SIGNAL wren_RdFifo_r            : std_logic_vector(5 DOWNTO 0);
--   SIGNAL rlRdfEmpty               : std_logic;

--   SIGNAL rlRdfSubFull             : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
--   SIGNAL rlRdfSubEmpty            : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL rlRdfSubFull             : std_logic_vector((RL_DQ_WIDTH/18)-1 DOWNTO 0);
   SIGNAL rlRdfSubEmpty            : std_logic_vector((RL_DQ_WIDTH/18)-1 DOWNTO 0);
   SIGNAL dpRdData_lower           : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL dpRdData_higher          : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL rldReadData_lower        : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL rldReadData_higher       : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);

   SIGNAL rlWdfFull_f              : std_logic_vector((RL_DQ_WIDTH/18)-1 DOWNTO 0);   -- Full  flag signal for each FIFO36 instances
   SIGNAL rlWdfFull_DM             : std_logic;
   SIGNAL rlWdfEmpty_f             : std_logic_vector((RL_DQ_WIDTH/18)-1 DOWNTO 0);   -- Empty flag signal for each FIFO36 instances
   SIGNAL rlWdfEmpty_DM            : std_logic;
   SIGNAL rlWdfAlmostFull_f        : std_logic_vector((RL_DQ_WIDTH/18)-1 DOWNTO 0);   -- AlmostFull  flag signal for each FIFO36 instances
   SIGNAL rlWdfAlmostFull_DM       : std_logic;
   SIGNAL rlWdfAlmostEmpty_f       : std_logic_vector((RL_DQ_WIDTH/18)-1 DOWNTO 0);   -- AlmostEmpty flag signal for each FIFO36 instances
   SIGNAL rlWdfAlmostEmpty_DM      : std_logic;
   SIGNAL rlWdfWrCount_f           : std_logic_vector(((RL_DQ_WIDTH/18)*13)-1 DOWNTO 0);   --  "write data FIFO write count", 13-bit count
   SIGNAL rlWdfRdCount_f           : std_logic_vector(((RL_DQ_WIDTH/18)*13)-1 DOWNTO 0);   --  "write data FIFO read count",  13-bit count

   SIGNAL clk180                   : std_logic;
   SIGNAL clk270                   : std_logic;

   SIGNAL rlafWrCount_sig          : std_logic_vector(12 DOWNTO 0);

   SIGNAL wdfDM_f                  : std_logic_vector(15 DOWNTO 0);
   SIGNAL apWriteDM_f              : std_logic_vector(15 DOWNTO 0);
   
   SIGNAL apAddr_f                 : std_logic_vector(31 DOWNTO 0);
   SIGNAL afA_f                    : std_logic_vector(31 DOWNTO 0);

BEGIN
   cs_io        <= (others => 'Z');
   rlafWrCount  <= rlafWrCount_sig;

   clk180 <= NOT clk;
   clk270 <= NOT clk90;

   -- read data from PHY, to be stored in FIFO
   dpRdData_lower  <= dpRdData(RL_DQ_WIDTH-1 DOWNTO 0);                -- falling data read from phy interface
   dpRdData_higher <= dpRdData((2*RL_DQ_WIDTH)-1 DOWNTO RL_DQ_WIDTH);  -- rising data read from phy interface 

   -- read data going back to the user (output from FIFO)
   rldReadData(RL_DQ_WIDTH-1 DOWNTO 0)               <= rldReadData_lower;   -- falling data read from FIFO
   rldReadData((2*RL_DQ_WIDTH)-1 DOWNTO RL_DQ_WIDTH) <= rldReadData_higher;  -- rising data read from FIFO

--   -- -----------------------------------------------------------
--   -- Chipscope observation points
--   -- -----------------------------------------------------------
--   cs_io(8 DOWNTO 0)   <= rldReadData_higher(8 DOWNTO 0);     -- 1st byte
--   cs_io(17 DOWNTO 9)  <= rldReadData_lower(8 DOWNTO 0);
--
--   cs_io(26 DOWNTO 18) <= rldReadData_higher(17 DOWNTO 9);    -- 2nd byte
--   cs_io(35 DOWNTO 27) <= rldReadData_lower(17 DOWNTO 9);
--
--   cs_io(44 DOWNTO 36) <= rldReadData_higher(26 DOWNTO 18);   -- 3rd byte
--   cs_io(53 DOWNTO 45) <= rldReadData_lower(26 DOWNTO 18);
--
--   cs_io(62 DOWNTO 54) <= rldReadData_higher(35 DOWNTO 27);   -- 4th byte
--   cs_io(71 DOWNTO 63) <= rldReadData_lower(35 DOWNTO 27);
--
--   cs_io(89 DOWNTO 88) <= wren_RdFifo(2) & wren_RdFifo(0);


   -- -----------------------------------------------------------------------------
   -- Read Data FIFO "rdfifo"
   --   - Capture read data from physical interface
   --   - Buffer back to the user interface, storing data
   -- -----------------------------------------------------------------------------
   -- The read data format is a concatenation of the rising data (in MSB) and 
   -- the falling data (in LSB).  The data is read from FIFO36 where 18-bit rise
   -- and 18-bit fall concatenation is using {MSB | LSB} 36-bit format.
   --
   -- For instance,
   -- 18-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --
   -- 36-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 1)
   --                          [35:18] rise  [35:18] fall
   --
   -- 54-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 1)
   --                          [35:18] rise  [35:18] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 2)
   --                          [53:36] rise  [53:36] fall
   --
   -- 72-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 1)
   --                          [35:18] rise  [35:18] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 2)
   --                          [53:36] rise  [53:36] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 3)
   --                          [71:54] rise  [71:54] fall
   --
   -- The FIFO36 18-bit data is splitted among Parity (2-bit) and Data bits (16-bit).
   -- Two MSB bits goes to Parity, the rest (16-bit) goes to data bits.
   --
   -- A special case is required to handle shorter than 18-bit data width.  This
   -- has not been implemented yet.
   -- -----------------------------------------------------------------------------

   rdfifo_gen_inst : FOR K IN  RL_DQ_WIDTH/18-1 DOWNTO 0  GENERATE
      -- local signals
      SIGNAL rldReadData_DO  : std_logic_vector(2*16-1 DOWNTO 0) := (others=>'0');
      SIGNAL rldReadData_DOP : std_logic_vector(2*2-1 DOWNTO 0)  := (others=>'0');
      SIGNAL dpRdData_DI     : std_logic_vector(2*16-1 DOWNTO 0) := (others=>'0');
      SIGNAL dpRdData_DIP    : std_logic_vector(2*2-1 DOWNTO 0)  := (others=>'0');
   BEGIN
      -- DO data slice
      rldReadData_higher( ((K+1)*18-2-1) DOWNTO ((K+1)*18-2-1)-(18-2-1) )  <= rldReadData_DO(31 DOWNTO 16);
      rldReadData_lower( ((K+1)*18-2-1) DOWNTO ((K+1)*18-2-1)-(18-2-1) )   <= rldReadData_DO(15 DOWNTO 0);
      -- DOP data slice (on the parity)
      rldReadData_higher( ((K+1)*18-1) DOWNTO ((K+1)*18-1)-(2-1) )  <= rldReadData_DOP(3 DOWNTO 2);
      rldReadData_lower( ((K+1)*18-1) DOWNTO ((K+1)*18-1)-(2-1) )   <= rldReadData_DOP(1 DOWNTO 0);
      --
      dpRdData_DI  <= dpRdData_higher( ((K+1)*18-2-1) DOWNTO ((K+1)*18-2-1)-(18-2-1) ) & dpRdData_lower( ((K+1)*18-2-1) DOWNTO ((K+1)*18-2-1)-(18-2-1) );
      dpRdData_DIP <= dpRdData_higher( ((K+1)*18-1) DOWNTO ((K+1)*18-1)-(2-1) ) & dpRdData_lower( ((K+1)*18-1) DOWNTO ((K+1)*18-1)-(2-1) );

      rdfifo : FIFO36
      GENERIC MAP (
--         ALMOST_FULL_OFFSET      => X"080",
--         ALMOST_EMPTY_OFFSET     => X"007",
         DATA_WIDTH              => 36,
         DO_REG                  => 1,
         EN_SYN                  => FALSE,
         FIRST_WORD_FALL_THROUGH => FALSE
      )
      PORT MAP (
         ALMOSTEMPTY  => open,
         ALMOSTFULL   => open,
--         DO           => rldReadData_higher(15 DOWNTO 0)  & rldReadData_lower(15 DOWNTO 0),
--         DOP          => rldReadData_higher(17 DOWNTO 16) & rldReadData_lower(17 DOWNTO 16),
         DO           => rldReadData_DO,
         DOP          => rldReadData_DOP,
         EMPTY        => rlRdfSubEmpty(K),
         FULL         => rlRdfSubFull(K),
         RDCOUNT      => open,
         RDERR        => open,
         WRCOUNT      => open,
         WRERR        => open,
--         DI           => dpRdData_higher(15 DOWNTO 0)  & dpRdData_lower(15 DOWNTO 0),
--         DIP          => dpRdData_higher(17 DOWNTO 16) & dpRdData_lower(17 DOWNTO 16),
         DI           => dpRdData_DI,
         DIP          => dpRdData_DIP,
         RDCLK        => clk,
--         RDEN         => apRdfRdEn,
         RDEN         => wren_RdFifo_r(5),
         RST          => rstHard,
         WRCLK        => clk270,
         WREN         => wren_RdFifo(K*NUM_OF_FIFOS)   -- use even byte (skip odd byte)
      );
   END GENERATE;

   -- concatenate flags from individual "read data" FIFOs
   -- do not use FLAGS, issues seen.
   rlRdfFull   <= OR_REDUCE( rlRdfSubFull ((RL_DQ_WIDTH/18)-1 DOWNTO 0) );
   rlRdfEmpty  <= OR_REDUCE( rlRdfSubEmpty((RL_DQ_WIDTH/18)-1 DOWNTO 0) );


   -- -----------------------------------------------------------------------------
   -- Read Data valid flag (will be used by the user or test bench)
   -- -----------------------------------------------------------------------------
   -- User to change the Read-enable for the read Data FIFOs from automatic to manual
--   PROCESS (clk)
--   BEGIN
--      IF RISING_EDGE( clk ) THEN
--         IF (rstHard = '1') THEN
--            rldReadDataValid <= '0';
--         ELSE
--            rldReadDataValid <= apRdfRdEn AND NOT rlRdfEmpty;
--         END IF;
--      END IF;
--   END PROCESS;
   
--   PROCESS (clk)
--   BEGIN
--      IF RISING_EDGE( clk ) THEN
--         IF (rstHard = '1') THEN
--            rlRdfEmpty <= '1';
--         ELSE
--            rlRdfEmpty <= OR_REDUCE( rlRdfSubEmpty(NUM_OF_DEVS-1 DOWNTO 0) );
--         END IF;
--      END IF;
--   END PROCESS;

   PROCESS (clk270)
   BEGIN
      IF RISING_EDGE( clk270 ) THEN
         IF (rstHard_270 = '1') THEN
            wren_RdFifo_r(0) <= '0'; 
         ELSE
            wren_RdFifo_r(0) <= apRdfRdEn; 
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk90)
   BEGIN
      IF RISING_EDGE( clk90 ) THEN
         IF (rstHard_90 = '1') THEN
            wren_RdFifo_r(1) <= '0'; 
         ELSE
            wren_RdFifo_r(1) <= wren_RdFifo_r(0); 
         END IF;
      END IF;
   END PROCESS;

   PROCESS (clk)
   BEGIN
      IF RISING_EDGE( clk ) THEN
         IF (rstHard = '1') THEN
            wren_RdFifo_r(5 DOWNTO 2) <= "0000"; 
            rldReadDataValid <= '0'; 
         ELSE
            wren_RdFifo_r(2) <= wren_RdFifo_r(1);
            wren_RdFifo_r(3) <= wren_RdFifo_r(2);
            wren_RdFifo_r(4) <= wren_RdFifo_r(3);
            wren_RdFifo_r(5) <= wren_RdFifo_r(4);

            rldReadDataValid <= wren_RdFifo_r(5);   -- data is valid one clock cycle after read_enable
         END IF;
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- Write Data FIFO "wdfifo"
   --   - include bits for DM signals (i.e. larger FIFO to store data + mask)
   --   - store data (from the user) to be written to RL2
   -- -----------------------------------------------------------------------------
   -- The write data format is a concatenation of the rising data (in MSB) and 
   -- the falling data (in LSB).  The data is stored in FIFO36 where 18-bit rise
   -- and 18-bit fall are stored using {MSB | LSB} 36-bit format.
   --
   -- For instance,
   -- 18-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --
   -- 36-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 1)
   --                          [35:18] rise  [35:18] fall
   --
   -- 54-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 1)
   --                          [35:18] rise  [35:18] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 2)
   --                          [53:36] rise  [53:36] fall
   --
   -- 72-bit RL2 data width => 18-bit rise | 18-bit fall (FIFO36, instance 0)
   --                          [17:0] rise   [17:0] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 1)
   --                          [35:18] rise  [35:18] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 2)
   --                          [53:36] rise  [53:36] fall
   --                          18-bit rise | 18-bit fall (FIFO36, instance 3)
   --                          [71:54] rise  [71:54] fall
   --
   -- The FIFO36 18-bit data is splitted among Parity (2-bit) and Data bits (16-bit).
   -- Two MSB bits goes to Parity, the rest (16-bit) goes to data bits.
   --
   -- A special case is required to handle shorter than 18-bit data width.  This
   -- has not been implemented yet.
   -- -----------------------------------------------------------------------------

   wdfifo_gen_inst : FOR K IN  RL_DQ_WIDTH/18-1 DOWNTO 0  GENERATE
      -- local signals
      SIGNAL wdfD_DO         : std_logic_vector(2*16-1 DOWNTO 0) := (others=>'0');
      SIGNAL wdfD_DOP        : std_logic_vector(2*2-1 DOWNTO 0)  := (others=>'0');
      SIGNAL apWriteData_DI  : std_logic_vector(2*16-1 DOWNTO 0) := (others=>'0');
      SIGNAL apWriteData_DIP : std_logic_vector(2*2-1 DOWNTO 0)  := (others=>'0');
   BEGIN
      -- DO data slice
      wdfD( (RL_DQ_WIDTH+(K+1)*18-2-1) DOWNTO (RL_DQ_WIDTH+(K+1)*18-2-1)-(18-2-1) )  <= wdfD_DO(31 DOWNTO 16);
      wdfD( ((K+1)*18-2-1) DOWNTO ((K+1)*18-2-1)-(18-2-1) )                          <= wdfD_DO(15 DOWNTO 0);
      -- DOP data slice (on the parity)
      wdfD( (RL_DQ_WIDTH+(K+1)*18-1) DOWNTO (RL_DQ_WIDTH+(K+1)*18-1)-(2-1) )  <= wdfD_DOP(3 DOWNTO 2);
      wdfD( ((K+1)*18-1) DOWNTO ((K+1)*18-1)-(2-1) )                          <= wdfD_DOP(1 DOWNTO 0);
      --
      apWriteData_DI  <= apWriteData( (RL_DQ_WIDTH+(K+1)*18-2-1) DOWNTO (RL_DQ_WIDTH+(K+1)*18-2-1)-(18-2-1) ) & apWriteData( ((K+1)*18-2-1) DOWNTO ((K+1)*18-2-1)-(18-2-1) );
      apWriteData_DIP <= apWriteData( (RL_DQ_WIDTH+(K+1)*18-1) DOWNTO (RL_DQ_WIDTH+(K+1)*18-1)-(2-1) )        & apWriteData( ((K+1)*18-1) DOWNTO ((K+1)*18-1)-(2-1) ); 

      wdfifo : FIFO36
      GENERIC MAP (
--         ALMOST_FULL_OFFSET       => X"080",
--         ALMOST_EMPTY_OFFSET      => X"007",
         DATA_WIDTH               => 36,
         DO_REG                   => 1,
         EN_SYN                   => FALSE,
         FIRST_WORD_FALL_THROUGH  => FALSE
      )
      PORT MAP (
         ALMOSTEMPTY  => rlWdfAlmostEmpty_f(K),
         ALMOSTFULL   => rlWdfAlmostFull_f(K),
--         DO           => wdfD(51 DOWNTO 36) & wdfD(15 DOWNTO 0),
--         DOP          => wdfD(53 DOWNTO 52) & wdfD(17 DOWNTO 16),
         DO           => wdfD_DO,
         DOP          => wdfD_DOP,
         EMPTY        => rlWdfEmpty_f(K),
         FULL         => rlWdfFull_f(K),
         RDCOUNT      => rlWdfRdCount_f((K+1)*13-1 DOWNTO K*13),
         RDERR        => open,
         WRCOUNT      => rlWdfWrCount_f((K+1)*13-1 DOWNTO K*13),
         WRERR        => open,
--         DI           => apWriteData(51 DOWNTO 36) & apWriteData(15 DOWNTO 0),
--         DIP          => apWriteData(53 DOWNTO 52) & apWriteData(17 DOWNTO 16),
         DI           => apWriteData_DI,
         DIP          => apWriteData_DIP,
         RDCLK        => clk,
         RDEN         => ctlWdfRdEn,
         RST          => rstHard,
         WRCLK        => clk,
         WREN         => apWriteDValid
      );
   END GENERATE;

   -- hold the Data Mask (DM) information
   wdfDM        <= wdfDM_f((2*NUM_OF_DEVS)-1 DOWNTO 0);
   apWriteDM_f  <= x"000" & apWriteDM((2*NUM_OF_DEVS)-1 DOWNTO 0);
   
   wdfifo_DM : FIFO18
   GENERIC MAP (
--      ALMOST_FULL_OFFSET       => X"080",
--      ALMOST_EMPTY_OFFSET      => X"007",
      -- valid DATA_WIDTH values are 4, 9 or 18
      DATA_WIDTH               => CC_LT2( 2*NUM_OF_DEVS, 5, 4, 2*NUM_OF_DEVS, 10, 9, 18),   -- 9 devices max per Controller
      DO_REG                   => 1,
      EN_SYN                   => FALSE,
      FIRST_WORD_FALL_THROUGH  => FALSE
   )
   PORT MAP (
      ALMOSTEMPTY  => rlWdfAlmostEmpty_DM,
      ALMOSTFULL   => rlWdfAlmostFull_DM,
      DO           => wdfDM_f,
      DOP          => open,           -- not used
      EMPTY        => rlWdfEmpty_DM,
      FULL         => rlWdfFull_DM,
      RDCOUNT      => open,
      RDERR        => open,
      WRCOUNT      => open,
      WRERR        => open,
      DI           => apWriteDM_f,
      DIP          => "00",           -- not used
      RDCLK        => clk,
      RDEN         => ctlWdfRdEn,
      RST          => rstHard,
      WRCLK        => clk,
      WREN         => apWriteDValid
   );

   -- concatenate flags from individual "write data" FIFOs
--   rlWdfEmpty  <= rlWdfEmpty_0 OR rlWdfEmpty_1 OR rlWdfEmpty_2;
--   rlWdfFull   <= rlWdfFull_0  OR rlWdfFull_1  OR rlWdfFull_2;
   rlWdfEmpty  <= OR_REDUCE( rlWdfEmpty_f((RL_DQ_WIDTH/18)-1 DOWNTO 0) ) OR rlWdfEmpty_DM;
   rlWdfFull   <= OR_REDUCE( rlWdfFull_f((RL_DQ_WIDTH/18)-1 DOWNTO 0) )  OR rlWdfFull_DM;

   rlWdfAlmostEmpty  <= OR_REDUCE( rlWdfAlmostEmpty_f((RL_DQ_WIDTH/18)-1 DOWNTO 0) ) OR rlWdfAlmostEmpty_DM;
   rlWdfAlmostFull   <= OR_REDUCE( rlWdfAlmostFull_f((RL_DQ_WIDTH/18)-1 DOWNTO 0) )  OR rlWdfAlmostFull_DM;

   rlWdfWrCount    <= rlWdfWrCount_f(12 DOWNTO 0);  -- use first FIFO's count of "write data FIFO write count"
   rlWdfRdCount    <= rlWdfRdCount_f(12 DOWNTO 0);  -- use first FIFO's count of "write data FIFO read count"
   -- number of words in "write data FIFO"
   rlWdfWordCount  <= STD_LOGIC_VECTOR( UNSIGNED(rlWdfWrCount_f(12 DOWNTO 0))-UNSIGNED(rlWdfRdCount_f(12 DOWNTO 0)) );
   -- -----------------------------------------------------------------------------
   -- Command & Address merged FIFO "rafifo"
   --   - Store command and address (from the user)
   -- -----------------------------------------------------------------------------
   afA      <= afA_f(APP_AD_WIDTH-1 DOWNTO 0);
   apAddr_f <= b"00_0000" & apAddr(APP_AD_WIDTH-1 DOWNTO 0);
   
   rafifo0 : FIFO36
   GENERIC MAP (
--      ALMOST_FULL_OFFSET       => X"080",
      ALMOST_EMPTY_OFFSET      => X"009",   -- arbitrary setting, may need to change based on HW tests
      DATA_WIDTH               => 36,
      DO_REG                   => 1,
      EN_SYN                   => FALSE,
      FIRST_WORD_FALL_THROUGH  => FALSE
   )
   PORT MAP (
      ALMOSTEMPTY  => rlafAlmostEmpty,
      ALMOSTFULL   => rlafAlmostFull,
      DO           => afA_f,
      DOP          => open,           -- not used
      EMPTY        => rlafEmpty,
      FULL         => rlafFull,
      RDCOUNT      => rlafRdCount,
      RDERR        => open,
      WRCOUNT      => rlafWrCount_sig,
      WRERR        => open,
      DI           => apAddr_f,
      DIP          => "0000",         -- not used
      RDCLK        => clk,
      RDEN         => ctlafRdEn,
      RST          => rstHard,
      WRCLK        => clk,
      WREN         => apValid
   );

   -- number of words in "command/address FIFO"
   rlafWordCount <= STD_LOGIC_VECTOR( UNSIGNED(rlafWrCount_sig)-UNSIGNED(rlafRdCount) );

END ARCHITECTURE rtl;
