--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_v4_dq_iob.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	Virtex-4
--	Purpose: This module implements the physical interface for the 
--		      bi-directional DQ data signal.  ODDR is used for write; IDDR is
--          used for read through an IDELAY.
--
--*****************************************************************************

library	IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library	UNISIM;
use UNISIM.VCOMPONENTS.ALL;

ENTITY rld_v4_dq_iob IS
   GENERIC (
      RL_IO_TYPE                     :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH                    :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD                 :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk                     : IN std_logic;   
      clk90                   : IN std_logic;   
      refClk                  : IN std_logic;   -- XBP: NOT USED!
      rstHard                 : IN std_logic;   
      rstHard_90              : IN std_logic;   
      rstHard_180             : IN std_logic;   
      rstHard_270             : IN std_logic;   
      dlyinc                  : IN std_logic;   
      dlyce                   : IN std_logic;   
      dlyrst                  : IN std_logic;   
      write_data_rise         : IN std_logic;   
      write_data_fall         : IN std_logic;   
      ctrl_wren               : IN std_logic;   
      phy_dq                  : INOUT std_logic;
      read_data_rise          : OUT std_logic;   
      read_data_fall          : OUT std_logic
   );   
END ENTITY rld_v4_dq_iob;

ARCHITECTURE rtl OF rld_v4_dq_iob IS


   SIGNAL dq_in                    :  std_logic;   
   SIGNAL dq_out                   :  std_logic;   
   SIGNAL dq_delayed               :  std_logic;   
   SIGNAL write_en_L               :  std_logic;   
   SIGNAL write_en_L_r1            :  std_logic;   
   SIGNAL clk180                   :  std_logic;   
   SIGNAL clk270                   :  std_logic;   
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;   
   SIGNAL phy_dq_temp              :  std_logic;    

BEGIN
   vcc <= '1' ;
   gnd <= '0' ;
   clk180 <= NOT clk ;
   clk270 <= NOT clk90 ;
   write_en_L <= NOT ctrl_wren ;
   
   
   dq_oddr : ODDR
   GENERIC MAP (
      SRTYPE         => "SYNC",
      DDR_CLK_EDGE   => "SAME_EDGE"
   )     
   PORT MAP (
      Q  => dq_out,
      C  => clk270,
      CE => vcc,
      D1 => write_data_rise,
      D2 => write_data_fall,
      R  => rstHard_270,      -- Forced to use the reset in tri-state flop??? Else IDDR doesn't work in hardware
      S  => gnd
   );   
   
   
   dq_tri_state : FDCE
--   GENERIC MAP (
--      IOB   => TRUE   -- passing "IOB" attribute to FDCE
--   )     
   PORT MAP (
      D     => write_en_L,
      CLR   => rstHard_270,
      C     => clk270,
      Q     => write_en_L_r1,
      CE    => vcc
   );   
   
   
--   dq_iobuf : IOBUF
--   PORT MAP (
--      O  => dq_in,
--      IO => phy_dq,
--      T  => write_en_L_r1,
--      I  => dq_out
--   );
-- XBP: infer the IOBUF instead of usign its instantiation
   
   phy_dq_temp <= dq_out WHEN write_en_L_r1 = '1' ELSE 'Z';
   phy_dq <= phy_dq_temp ;
   
  
   dq_idelay : IDELAY
   GENERIC MAP (
      IOBDELAY_TYPE    => "VARIABLE",
      IOBDELAY_VALUE   => 0
   )       
   PORT MAP (
      O     => dq_delayed,
--      I     => dq_in,
      I     => phy_dq, 
      C     => clk,
      CE    => dlyce,
      INC   => dlyinc,
      RST   => dlyrst
   );   


   dq_iddr : IDDR
   GENERIC MAP (
      SRTYPE         => "SYNC",
      DDR_CLK_EDGE   => "SAME_EDGE_PIPELINED"
   )      
   PORT MAP (
         Q1 => read_data_rise,
         Q2 => read_data_fall,
         C  => clk,
         CE => vcc,
         D  => dq_delayed,
         R  => rstHard,   -- Forced to use the reset in tri-state flop??? Else IDDR doesn't work in hardware
         S  => gnd
   );   
   

END ARCHITECTURE rtl;
