--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_v4_qvld_iob.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	 Virtex-4 and Virtex-5
--	Purpose: This module implements the physical interface for the QVLD signal 
--		      provided by the memory.  QVLD follows the same input path as read
--          data and is captured by the IDDR (Virtex-4) or ISERDES (Virtex-5).
--
--*****************************************************************************

library	IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library	UNISIM;
use UNISIM.VCOMPONENTS.ALL;

ENTITY rld_qvld_iob IS
   GENERIC (
      RL_IO_TYPE                     :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH                    :  std_logic_vector(1 DOWNTO 0) := "00";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD                 :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk                     : IN std_logic;   
      refClk                  : IN std_logic;   
      clk270                  : IN std_logic;   
      bufio_clk               : IN std_logic;   
      reset                   : IN std_logic;   
      dlyinc                  : IN std_logic;   
      dlyce                   : IN std_logic;   
      dlyrst                  : IN std_logic;   
      phy_qvld                : IN std_logic;
      
      -- Virtex-4 interface
      first_rising_byte0      : IN std_logic;   
      qvld_delayed_rise_byte0 : OUT std_logic;   
      qvld_delayed_fall_byte0 : OUT std_logic;
         
      first_rising_byte1      : IN std_logic;   
      qvld_delayed_rise_byte1 : OUT std_logic;   
      qvld_delayed_fall_byte1 : OUT std_logic; 
        
      -- Virtex-5 interface
--      qvld_out                : OUT std_logic;
      read_enable_rise        : OUT std_logic;   
      read_enable_fall        : OUT std_logic);   
END ENTITY rld_qvld_iob;

ARCHITECTURE rtl OF rld_qvld_iob IS

   -- Virtex-4 interface
   SIGNAL rd_en_delayed            :  std_logic;   
   SIGNAL qvld_rise                :  std_logic;   
   SIGNAL qvld_fall                :  std_logic;   
   SIGNAL qvld_fall_r1             :  std_logic;
      
   -- Virtex-5 interface
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;
   
   SIGNAL temp_fall_0              :  std_logic;   
   SIGNAL temp_rise_0              :  std_logic;   
   SIGNAL temp_fall_1              :  std_logic;   
   SIGNAL temp_rise_1              :  std_logic;     
   
   SIGNAL not_reset                : std_logic;   

BEGIN
   vcc <= '1' ;
   gnd <= '0' ;
   
   not_reset <= NOT reset;
   
   qvld_virtex4 : IF ( DEVICE_ARCH = "00") GENERATE
   BEGIN
   
      idelay_read_en : IDELAY
      GENERIC MAP (
         IOBDELAY_TYPE  => "VARIABLE",
         IOBDELAY_VALUE => 0
      )
      PORT MAP (
         O     => rd_en_delayed,
         I     => phy_qvld,
         C     => clk,
         CE    => dlyce,
         INC   => dlyinc,
         RST   => dlyrst
      );   
   
      iddr_qvld : IDDR
      GENERIC MAP (
         SRTYPE         => "SYNC",
         DDR_CLK_EDGE   => "SAME_EDGE_PIPELINED"
      ) 
      PORT MAP (
         Q1    => qvld_rise,
         Q2    => qvld_fall,
         C     => clk,
         CE    => vcc,
         D     => rd_en_delayed,
         R     => gnd,
         S     => gnd
      );   
   
      -- -----------------------------------------------------------------------------
      -- Read enable signals from captured QVLD
      --   - QVLD tap calibration uses one tap count, assuming equal PCB routing
      --   - independant output signals for two bytes of the same memory
      -- -----------------------------------------------------------------------------
      -- first byte
      temp_fall_0 <= qvld_fall WHEN (first_rising_byte0) = '1' ELSE qvld_rise;
      qvld_delayed_fall_byte0 <= temp_fall_0 ;
      temp_rise_0 <= qvld_fall_r1 WHEN (first_rising_byte0) = '1' ELSE qvld_rise;
      qvld_delayed_rise_byte0 <= temp_rise_0 ;
      -- second byte
      temp_fall_1 <= qvld_fall WHEN (first_rising_byte1) = '1' ELSE qvld_rise;
      qvld_delayed_fall_byte1 <= temp_fall_1 ;
      temp_rise_1 <= qvld_fall_r1 WHEN (first_rising_byte1) = '1' ELSE qvld_rise;
      qvld_delayed_rise_byte1 <= temp_rise_1 ;

      qvld_fd : PROCESS (clk)
      BEGIN
         IF RISING_EDGE ( clk ) THEN
            qvld_fall_r1 <= qvld_fall;    
         END IF;
      END PROCESS qvld_fd;
   
   END GENERATE qvld_virtex4;


   qvld_virtex5 : IF ( DEVICE_ARCH = "01") GENERATE
   BEGIN
   
      qvld_iserdes : ISERDES
      GENERIC MAP (
--         LOW_JITTER     => TRUE,  --passing "LOW_JITTER" attribute to ISERDES
         BITSLIP_ENABLE => FALSE,
         DATA_RATE      => "DDR",
         DATA_WIDTH     => 4,
         INTERFACE_TYPE => "MEMORY",
         IOBDELAY       => "IFD",
         IOBDELAY_TYPE  => "VARIABLE",
         IOBDELAY_VALUE => 0,
         NUM_CE         => 2,
         SERDES_MODE    => "MASTER"
      ) 
      PORT MAP (
         O           => open,
         Q1          => qvld_fall,
         Q2          => qvld_rise,
         Q3          => open,
         Q4          => open,
         Q5          => open,
         Q6          => open,
         SHIFTOUT1   => open,
         SHIFTOUT2   => open,
         BITSLIP     => '0',
         CE1         => not_reset,
         CE2         => not_reset,
         CLK         => bufio_clk,  -- high speed forwarded clock input, driven by BUFIO
         CLKDIV      => clk270,     -- divided high-speed forwarded clock input, drive last flop stage + IDELAY
         D           => phy_qvld,
         DLYCE       => dlyce,
         DLYINC      => dlyinc,
         DLYRST      => dlyrst,
         OCLK        => clk270,     -- high speed clock for memory apps, drive the serial-to-parallel converter
         REV         => gnd,
         SHIFTIN1    => '0',
         SHIFTIN2    => '0',
         SR          => reset
      );   
   
      read_enable_rise <= qvld_rise ;
      read_enable_fall <= qvld_fall ;

   END GENERATE qvld_virtex5;
   

END ARCHITECTURE rtl;
