--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.0
--  \   \         Filename: rld_v5_dq_iob.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:	Virtex-5
--	Purpose: This module implements the physical interface for the Read 
--		      data path.  Generates the read path (RLD2_DQ) from the IBUFs
--          to the read data FIFOs.
--
--*****************************************************************************

library	IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;

library	UNISIM;
use UNISIM.VCOMPONENTS.ALL;

ENTITY rld_v5_dq_iob IS
   GENERIC (
      RL_IO_TYPE                     :  std_logic_vector(1 DOWNTO 0) := "00";    --  CIO=2'b00  SIO_CIO_HYBRID=2'b01  SIO=2'b10
      DEVICE_ARCH                    :  std_logic_vector(1 DOWNTO 0) := "01";    --  Virtex4=2'b00  Virtex5=2'b01
      CAPTURE_METHOD                 :  std_logic_vector(1 DOWNTO 0) := "00"     --  Direct Clocking=2'b00  SerDes=2'b01
   );
   PORT (
      clk270                  : IN std_logic;   
      bufio_clk               : IN std_logic;   
      reset                   : IN std_logic;   
      dlyinc                  : IN std_logic;   --  the tap calibration runs with CLKDIV
      dlyce                   : IN std_logic;   
      dlyrst                  : IN std_logic;   
      write_data_rise         : IN std_logic;   
      write_data_fall         : IN std_logic;   
      ctrl_wren               : IN std_logic;   
      phy_dq                  : INOUT std_logic; 
      read_data_rise          : OUT std_logic;   
      read_data_fall          : OUT std_logic
   );   
END ENTITY rld_v5_dq_iob;

ARCHITECTURE rtl OF rld_v5_dq_iob IS


   SIGNAL dq_in                    :  std_logic;   
   SIGNAL dq_out                   :  std_logic;   
   SIGNAL write_en_L               :  std_logic;   
   SIGNAL write_en_L_r1            :  std_logic;   
   SIGNAL vcc                      :  std_logic;   
   SIGNAL gnd                      :  std_logic;   
   SIGNAL Q1                       :  std_logic;   --  for testing
   SIGNAL Q2                       :  std_logic;   --  for testing
   SIGNAL Q3                       :  std_logic;   --  for testing
   SIGNAL Q4                       :  std_logic;   --  for testing
   SIGNAL Q5                       :  std_logic;   --  for testing
   SIGNAL Q6                       :  std_logic;   --  for testing
   SIGNAL read_data_rise_sig       :  std_logic;   
   SIGNAL read_data_fall_sig       :  std_logic;   

   SIGNAL not_reset                :  std_logic;

BEGIN
   read_data_rise <= read_data_rise_sig;
   read_data_fall <= read_data_fall_sig;
   vcc <= '1' ;
   gnd <= '0' ;
   write_en_L <= NOT ctrl_wren ;
   not_reset <= NOT reset;
   
   dq_oddr : ODDR
   GENERIC MAP (
      SRTYPE         => "SYNC",
      DDR_CLK_EDGE   => "SAME_EDGE"
   )  
   PORT MAP (
      Q => dq_out,
      C => clk270,
      CE => vcc,
      D1 => write_data_rise,
      D2 => write_data_fall,
      R => '0',
      S => gnd
   );   
   
  
   dq_tri_state : FDCE
--   GENERIC MAP (
--      IOB   => TRUE
--   ) 
   PORT MAP (
      D     => write_en_L,
      CLR   => reset,
      C     => clk270,
      Q     => write_en_L_r1,
      CE    => vcc
   );   
   
   dq_iobuf : IOBUF 
   PORT MAP (
      I  => dq_out,
      T  => write_en_L_r1,
      IO => phy_dq,
      O  => dq_in
   );   
   
   Q1 <= read_data_fall_sig ;
   Q2 <= read_data_rise_sig ;
   
   dq_iserdes : ISERDES
   GENERIC MAP (
--         LOW_JITTER     => TRUE,  --passing "LOW_JITTER" attribute to ISERDES
         BITSLIP_ENABLE => FALSE,
         DATA_RATE      => "DDR",
         DATA_WIDTH     => 4,
         INTERFACE_TYPE => "MEMORY",
         IOBDELAY       => "IFD",
         IOBDELAY_TYPE  => "VARIABLE",
         IOBDELAY_VALUE => 0,
         NUM_CE         => 2,
         SERDES_MODE    => "MASTER"
      )  
   PORT MAP (
      O           => open,
      Q1          => read_data_fall_sig,
      Q2          => read_data_rise_sig,
      Q3          => Q3,
      Q4          => Q4,
      Q5          => Q5,
      Q6          => Q6,
      SHIFTOUT1   => open,
      SHIFTOUT2   => open,
      BITSLIP     => '0',
      CE1         => not_reset,
      CE2         => not_reset,
      CLK         => bufio_clk,  -- high speed forwarded clock input, driven by BUFIO
      CLKDIV      => clk270,     -- divided high-speed forwarded clock input, drive last flop stage + IDELAY 
      D           => dq_in,
      DLYCE       => dlyce,
      DLYINC      => dlyinc,
      DLYRST      => dlyrst,
      OCLK        => clk270,     -- high speed clock for memory apps, drive the serial-to-parallel converter
      REV         => gnd,
      SHIFTIN1    => '0',
      SHIFTIN2    => '0',
      SR          => reset
   );   
   

END ARCHITECTURE rtl;
