--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2006-2008 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.2
--  \   \         Filename: ext_test_bench.vhd
--  /   /         Timestamp: 14 March 2008
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-5
--	Purpose: Simulation test bench
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Changed the simulation_only parameter to default to false, replaced
--		         the Micron model, changed the default to config3 for 333MHz operation,
--             enable the DLL by default, changed clock frequency parameter to 
--             clock period
--   Rev 1.2 - Changed the default frequency to 250MHz
--*****************************************************************************

library ieee;
use ieee.std_logic_1164.all;


ENTITY ext_test_bench IS
END ENTITY ext_test_bench;


ARCHITECTURE rtl_sim OF ext_test_bench IS

   function CC (COND_EXPR:INTEGER; COND_VAL:INTEGER; IF_VAL:INTEGER; THEN_VAL:INTEGER) return INTEGER is 
   begin 
     if COND_EXPR = COND_VAL then
       return IF_VAL; 
     else
       return THEN_VAL; 
     end if; 
   end;


   CONSTANT BOARD_SKEW     : TIME := 500 ps;    -- put a 500 ps delay on the QK lines between FPGA and RAMs
   --
--   CONSTANT SYSCLK_PERIOD  : TIME := 2.500 ns;  -- System Clock period (400MHz)
--   CONSTANT SYSCLK_PERIOD  : TIME := 2.858 ns;  -- System Clock period (350MHz)
--   CONSTANT SYSCLK_PERIOD  : TIME := 3.002 ns;  -- System Clock period (333MHz)
--   CONSTANT SYSCLK_PERIOD  : TIME := 3.334 ns;  -- System Clock period (300MHz)
--   CONSTANT SYSCLK_PERIOD  : TIME := 3.700 ns;  -- System Clock period (270MHz)
   CONSTANT SYSCLK_PERIOD  : TIME := 4.000 ns;  -- System Clock period (250MHz)
--   CONSTANT SYSCLK_PERIOD  : TIME := 6.666 ns;  -- System Clock period (150MHz)
   --
   CONSTANT REFCLK_PERIOD  : TIME := 5.000 ns;  -- Reference Clock period (200MHz)
   CONSTANT TESTCLK_PERIOD : TIME := 30.30 ns;  -- Test Clock period (33MHz)


   -- Testbench constant
   CONSTANT SIMULATION_ONLY : boolean := FALSE;  -- if TRUE, it shortens the wait time but Micron model requires that time
   --
   CONSTANT RL_DQ_WIDTH  : integer := 36;  -- memory interface data width
   CONSTANT DEV_DQ_WIDTH : integer := 18;  -- data width of the memory device
   CONSTANT NUM_OF_DEVS  : integer := RL_DQ_WIDTH/DEV_DQ_WIDTH;  -- number of memory devices
   CONSTANT NUM_OF_DKS   : integer := CC(DEV_DQ_WIDTH, 36, 2*NUM_OF_DEVS, NUM_OF_DEVS);
   CONSTANT DEV_AD_WIDTH : integer := 20;  -- address width of the memory device
   CONSTANT DEV_BA_WIDTH : integer := 3;   -- bank address width of the memory device
   CONSTANT APP_AD_WIDTH : integer := 1+1+1+DEV_AD_WIDTH+DEV_BA_WIDTH;
   CONSTANT DUPLICATE_CONTROLS : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
   CONSTANT RL_CK_PERIOD   : integer := 3003;       -- CK clock period of the RLDRAM in ps
   -- MRS (Mode Register Set command) parameters   
   CONSTANT RL_MRS_CONF            : std_logic_vector(2 downto 0) := "011";  -- Config3   -- Config1="001"  Config2="010"  Config3="011"
   CONSTANT RL_MRS_BURST_LENGTH    : std_logic_vector(1 downto 0) := "01";   -- BL4       -- BL2="00"  BL4="01"  BL8="10"
   CONSTANT RL_MRS_ADDR_MUX        : std_logic := '0';    -- non-mux   -- non-mux address (default)='0';  multiplexed address='1'
   CONSTANT RL_MRS_DLL_RESET       : std_logic := '1';    -- '0': DLL reset, '1': DLL enabled
   CONSTANT RL_MRS_IMPEDANCE_MATCH : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
   CONSTANT RL_MRS_ODT             : std_logic := '0';    -- '0': disable term;  '1': enable term
   -- specific to FPGA/memory devices and capture method
   CONSTANT RL_IO_TYPE     : std_logic_vector(1 downto 0)  := "00";    -- CIO  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
   CONSTANT DEVICE_ARCH    : std_logic_vector(1 downto 0)  := "01";    -- V5   -- Virtex4="00"  Virtex5="01"
   CONSTANT CAPTURE_METHOD : std_logic_vector(1 downto 0)  := "01";    -- SerDes   -- Direct Clocking="00"  SerDes="01"
   CONSTANT CAL_ADDRESS    : std_logic_vector(DEV_AD_WIDTH-1 downto 0) := (others => '0');  -- saved location to perform calibration

   COMPONENT rld_mem_interface_top
   GENERIC (
      SIMULATION_ONLY         : boolean := FALSE;  -- if TRUE, it shortens the wait time
      RL_DQ_WIDTH             : integer := 36;     -- memory interface data width
      DEV_DQ_WIDTH            : integer := 18;     -- data width of the memory device
      NUM_OF_DEVS             : integer := 2;      -- number of memory devices
      NUM_OF_DKS              : integer := 4;
      DEV_AD_WIDTH            : integer := 20;     -- address width of the memory device
      DEV_BA_WIDTH            : integer := 3;      -- bank address width of the memory device
      APP_AD_WIDTH            : integer := 26;
      DUPLICATE_CONTROLS      : boolean := FALSE;  -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD              : integer := 3003; -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      --    please check Micron RLDRAM-II datasheet for definitions of these parameters
      RL_MRS_CONF             : std_logic_vector(2 DOWNTO 0) := "010";   -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH     : std_logic_vector(1 DOWNTO 0) := "01";    -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX         : std_logic := '0';    -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET        : std_logic := '0';    -- 
      RL_MRS_IMPEDANCE_MATCH  : std_logic := '1';    -- '0': internal 50ohms output buffer impedance, '1': external 
      RL_MRS_ODT              : std_logic := '0';    -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE              : std_logic_vector(1 DOWNTO 0) := "00";  -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH             : std_logic_vector(1 DOWNTO 0) := "01";  -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD          : std_logic_vector(1 DOWNTO 0) := "01";  -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS             : std_logic_vector(19 DOWNTO 0) := (others => '0')     -- saved location to perform calibration
   );
   PORT (
   -- System signals and debug
      refClk_p                : IN  std_logic;
      refClk_n                : IN  std_logic;
      sysClk_p                : IN  std_logic;
      sysClk_n                : IN  std_logic;
      sysReset                : IN  std_logic;
      dip                     : IN  std_logic;
      clk33                   : IN  std_logic;
      FPGA3_7SEG              : OUT std_logic_vector(7 DOWNTO 0);
      DBG_LED                 : OUT std_logic_vector(3 DOWNTO 0);
      FPGA3_TEST_HDR          : OUT std_logic_vector(7 DOWNTO 0);
      -- RLD2 memory signals
      RLD2_CK_P               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_CK_N               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_DK_P               : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      RLD2_DK_N               : OUT std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
      RLD2_QK_P               : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_QK_N               : IN  std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_A                  : OUT std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
      RLD2_BA                 : OUT std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
      RLD2_CS_N               : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_WE_N               : OUT std_logic;
      RLD2_REF_N              : OUT std_logic;
      RLD2_DM                 : OUT std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
      RLD2_DQ                 : INOUT std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
      RLD2_QVLD               : IN  std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0)
   );
   END COMPONENT;

   COMPONENT rldram2
   PORT (
      ck    : IN    std_logic;
      ck_n  : IN    std_logic;
      dk    : IN    std_logic;
      dk_n  : IN    std_logic;
      cs_n  : IN    std_logic;
      we_n  : IN    std_logic;
      ref_n : IN    std_logic;
      a     : IN    std_logic_vector(DEV_AD_WIDTH-1 downto 0);
      ba    : IN    std_logic_vector(DEV_BA_WIDTH-1 downto 0);
      dm    : IN    std_logic; 
      dq    : INOUT std_logic_vector(DEV_DQ_WIDTH-1 downto 0);
      qvld  : OUT   std_logic;
      qk    : OUT   std_logic_vector(1 downto 0);
      qk_n  : OUT   std_logic_vector(1 downto 0);
      tck   : IN    std_logic;
      tms   : IN    std_logic;
      tdi   : IN    std_logic;
      tdo   : OUT   std_logic
   );
   END COMPONENT;


   SIGNAL RESET               : std_logic := '0';
   SIGNAL intClk              : std_logic := '0';
   SIGNAL CLK                 : std_logic;
   SIGNAL intRefClk           : std_logic := '0';
   SIGNAL RefClk              : std_logic;   --  200MHz differential reference clock 
   SIGNAL clk33               : std_logic := '0';
   SIGNAL clk_select          : std_logic_vector(1 DOWNTO 0);
   --
   SIGNAL dip                 : std_logic;
   SIGNAL DISPLAY             : std_logic_vector(7 DOWNTO 0);
   SIGNAL LED                 : std_logic_vector(3 DOWNTO 0);
   SIGNAL TEST_HDR            : std_logic_vector(7 DOWNTO 0);
   --
   SIGNAL RC_CK               : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL RC_CK_N             : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL RC_CS_N             : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL RC_REF_N            : std_logic;
   SIGNAL RC_WE_N             : std_logic;
   SIGNAL RC_A                : std_logic_vector(DEV_AD_WIDTH-1 DOWNTO 0);
   SIGNAL RC_BA               : std_logic_vector(DEV_BA_WIDTH-1 DOWNTO 0);
   SIGNAL RC_DM               : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL RL_QK               : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL RL_QK_N             : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL RL_QVLD             : std_logic_vector(NUM_OF_DEVS-1 DOWNTO 0);
   SIGNAL DQ                  : std_logic_vector(RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL RC_DK               : std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
   SIGNAL RC_DK_N             : std_logic_vector(NUM_OF_DKS-1 DOWNTO 0);
   --
   SIGNAL bRL_QK              : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0) := (others => '0');
   SIGNAL bRL_QK_N            : std_logic_vector(2*NUM_OF_DEVS-1 DOWNTO 0) := (others => '1');
   --
   SIGNAL not_CLK             : std_logic;
   SIGNAL not_RefClk          : std_logic;


BEGIN
   CLK    <= intClk;
   RefClk <= intRefClk;

   not_CLK    <= NOT CLK;     -- required for ModelSim: "not a globally static expression"...
   not_RefClk <= NOT RefClk;  -- required for ModelSim: "not a globally static expression"...

   RL_QK   <= bRL_QK    AFTER BOARD_SKEW;
   RL_QK_N <= bRL_QK_N  AFTER BOARD_SKEW;

   clk_select <= "00";

   dip <= '0';

   -- Generate System Clock
   intClk <= NOT intClk AFTER (SYSCLK_PERIOD/2);

   -- Generate 200MHz Reference Clock
   intRefClk <= NOT intRefClk AFTER (REFCLK_PERIOD/2);

   -- Generate 33MHz test clock
   clk33 <= NOT clk33 AFTER (TESTCLK_PERIOD/2);

   -- Generate Reset
   RESET <= transport '1'  AFTER 1000.0 ns;    --  reset on board is active low


   -- =============================================================================
   -- V5 implementation
   -- =============================================================================

   -- RLDRAM-2 Controller and User Application			     
   ml561_top0 : rld_mem_interface_top
   GENERIC MAP (
      SIMULATION_ONLY        => SIMULATION_ONLY,         -- if set, it shortens the wait time
      RL_DQ_WIDTH            => RL_DQ_WIDTH,
      DEV_DQ_WIDTH           => DEV_DQ_WIDTH,            -- data width of the memory device
      NUM_OF_DEVS            => NUM_OF_DEVS,             -- number of memory devices
      NUM_OF_DKS             => NUM_OF_DKS,
      DEV_AD_WIDTH           => DEV_AD_WIDTH,            -- address width of the memory device
      DEV_BA_WIDTH           => DEV_BA_WIDTH,            -- bank address width of the memory device
      APP_AD_WIDTH           => APP_AD_WIDTH,
      DUPLICATE_CONTROLS     => DUPLICATE_CONTROLS,      -- Duplicate the ports for A, BA, WE# and REF#
      RL_CK_PERIOD           => RL_CK_PERIOD,            -- CK clock period of the RLDRAM in ps
      -- MRS (Mode Register Set command) parameters   
      RL_MRS_CONF            => RL_MRS_CONF,             -- "001": mode1;  "010": mode2;  "011": mode3
      RL_MRS_BURST_LENGTH    => RL_MRS_BURST_LENGTH,     -- "00": BL2;  "01": BL4;  "10": BL8
      RL_MRS_ADDR_MUX        => RL_MRS_ADDR_MUX,         -- '0': non-muxed addr;  '1': muxed addr
      RL_MRS_DLL_RESET       => RL_MRS_DLL_RESET,        --
      RL_MRS_IMPEDANCE_MATCH => RL_MRS_IMPEDANCE_MATCH,  -- internal 50ohms output buffer impedance
      RL_MRS_ODT             => RL_MRS_ODT,              -- '0': disable term;  '1': enable term
      -- specific to FPGA/memory devices and capture method
      RL_IO_TYPE             => RL_IO_TYPE,              -- CIO="00"  SIO_CIO_HYBRID="01"  SIO="10"
      DEVICE_ARCH            => DEVICE_ARCH,             -- Virtex4="00"  Virtex5="01"
      CAPTURE_METHOD         => CAPTURE_METHOD,          -- Direct Clocking="00"  SerDes="01"
      CAL_ADDRESS            => CAL_ADDRESS
   )
   PORT MAP (
      -- globals
      refClk_p       => RefClk,   -- 200MHz differential reference clock
      refClk_n       => not_RefClk,
      sysClk_p       => CLK,
      sysClk_n       => not_CLK,
      sysReset       => RESET,   
      clk33          => clk33,

      -- RLDRAM-II memory interface signals
      RLD2_CK_P      => RC_CK,
      RLD2_CK_N      => RC_CK_N,
      RLD2_DK_P      => RC_DK,
      RLD2_DK_N      => RC_DK_N,
      RLD2_QK_P      => RL_QK,
      RLD2_QK_N      => RL_QK_N,
      RLD2_A         => RC_A,
      RLD2_BA        => RC_BA,
      RLD2_CS_N      => RC_CS_N,
      RLD2_WE_N      => RC_WE_N,
      RLD2_REF_N     => RC_REF_N,
      RLD2_DM        => RC_DM,
      RLD2_DQ        => DQ,
      RLD2_QVLD      => RL_QVLD,
      
      -- observation points
      --PASS_FAIL	   => OPEN,
      dip            => dip,
      FPGA3_7SEG     => DISPLAY,
      DBG_LED        => LED,
      FPGA3_TEST_HDR => TEST_HDR       
   );


   -- RLDRAM-II memory models
   --needs to be downloaded from Micron's website (not included in XAPP852)	
   ram0 : rldram2
   PORT MAP (
      ck    => RC_CK(0),
      ck_n  => RC_CK_N(0),
      cs_n  => RC_CS_N(0),
      we_n  => RC_WE_N,
      ref_n => RC_REF_N,
      ba    => RC_BA,
      a     => RC_A,   -- 1 less adress bit             
      dq    => DQ(RL_DQ_WIDTH/2-1 DOWNTO 0),
      qk    => bRL_QK(2*NUM_OF_DEVS/2-1 DOWNTO 0),
      qk_n  => bRL_QK_N(2*NUM_OF_DEVS/2-1 DOWNTO 0),
      dk    => RC_DK(0),
      dk_n  => RC_DK_N(0),
      dm    => RC_DM(0),
      qvld  => RL_QVLD(0),
-- JTAG PORTS
      tck   => '0',
      tms   => '0',
      tdi   => '0',
      tdo   => OPEN
   );
   
   ram1 : rldram2
   PORT MAP (
      ck    => RC_CK(1),
      ck_n  => RC_CK_N(1),
      cs_n  => RC_CS_N(1),
      we_n  => RC_WE_N,
      ref_n => RC_REF_N,
      ba    => RC_BA,
      a     => RC_A,     -- 1 less adress bit           
      dq    => DQ(RL_DQ_WIDTH-1 DOWNTO RL_DQ_WIDTH/2),
      qk    => bRL_QK(2*NUM_OF_DEVS-1 DOWNTO 2*NUM_OF_DEVS/2),
      qk_n  => bRL_QK_N(2*NUM_OF_DEVS-1 DOWNTO 2*NUM_OF_DEVS/2),
      dk    => RC_DK(0),
      dk_n  => RC_DK_N(0),
      dm    => RC_DM(1),
      qvld  => RL_QVLD(1),
-- JTAG PORTS
      tck   => '0',
      tms   => '0',
      tdi   => '0',
      tdo   => OPEN
   );


END ARCHITECTURE rtl_sim;
