--*****************************************************************************
-- DISCLAIMER OF LIABILITY
-- 
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you a 
-- license to use this text/file solely for design, simulation, 
-- implementation and creation of design files limited 
-- to Xilinx devices or technologies. Use with non-Xilinx 
-- devices or technologies is expressly prohibited and 
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information 
-- "as-is" solely for use in developing programs and 
-- solutions for Xilinx devices, with no obligation on the 
-- part of Xilinx to provide support. By providing this design, 
-- code, or information as one possible implementation of 
-- this feature, application or standard, Xilinx is making no 
-- representation that this implementation is free from any 
-- claims of infringement. You are responsible for 
-- obtaining any rights you may require for your implementation. 
-- Xilinx expressly disclaims any warranty whatsoever with 
-- respect to the adequacy of the implementation, including 
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied 
-- warranties of merchantability or fitness for a particular 
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications is
-- expressly prohibited.
--
-- Any modifications that are made to the Source Code are 
-- done at the user's sole risk and will be unsupported.
--
-- Copyright (c) 2007 Xilinx, Inc. All rights reserved.
--
-- This copyright and support notice must be retained as part 
-- of this text at all times. 
--*****************************************************************************
--   ____  ____
--  /   /\/   /
-- /___/  \  /    Vendor: Xilinx
-- \   \   \/     Version: 1.1
--  \   \         Filename: test_bench.vhd
--  /   /         Timestamp: 24 August 2007
-- /___/   /\     
-- \   \  /  \
--  \___\/\___\
--
--
--	Device:  Virtex-4 and Virtex-5
--	Purpose: Synthesizable Testbench for Hardware Verification
-- Revision History:
--   Rev 1.0 - initial release
--   Rev 1.1 - Remove some use of RAF_ALMOST_FULL/WDF_ALMOST_FULL to improve timing
--*****************************************************************************

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.numeric_std.ALL;


ENTITY test_bench IS
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;    -- number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DEV_BYTE_WIDTH                 : integer := 9;    -- the width of a "byte" on the memory device (x8 or x9)
      RL_MRS_BURST_LENGTH            : std_logic_vector(1 DOWNTO 0) := "00"     -- "00": BL2;  "01": BL4;  "10": BL8
   );
   PORT (
      CLK                     : IN  std_logic;
      RESET                   : IN  std_logic;
      RESET_90                : IN  std_logic;
      WDF_ALMOST_FULL         : IN  std_logic;
      WDF_WR_COUNT            : IN  std_logic_vector(12 DOWNTO 0);   -- was (4 DOWNTO 0)
      RAF_ALMOST_FULL         : IN  std_logic;
      RAF_WR_COUNT            : IN  std_logic_vector(12 DOWNTO 0);   -- was (3 DOWNTO 0)
      rlafEmpty               : IN  std_logic;
      BURST_LENGTH            : IN  std_logic_vector(1 DOWNTO 0);
      READ_DATA_FIFO_OUT      : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
      READ_DATA_VALID         : IN  std_logic;
      Init_Done               : IN  std_logic;
--      Init_Done_270           : IN  std_logic;
      clk90                   : IN  std_logic;
      rlRdfEmpty              : IN  std_logic;
      rlWdfEmpty              : IN  std_logic;

      APADDR                  : OUT std_logic_vector(25 DOWNTO 0);
      APVALID                 : OUT std_logic;
      APP_WDF_DATA            : OUT std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
      APP_WDF_WREN            : OUT std_logic;

      DISPLAY                 : OUT std_logic_vector(6 DOWNTO 0);
      error_count_BCD         : OUT std_logic_vector(23 DOWNTO 0);

      APCONFA                 : OUT std_logic_vector(3 DOWNTO 0);
      APCONFWRD               : OUT std_logic_vector(7 DOWNTO 0);
      APCONFRD                : OUT std_logic;
      APCONFRDD               : IN  std_logic_vector(7 DOWNTO 0);
      APCONFWR                : OUT std_logic;
      APWRITEDM               : OUT std_logic_vector(3 DOWNTO 0);
      apRdfRdEn               : OUT std_logic;
      issueMRS                : OUT std_logic;
      issueCalibration        : OUT std_logic;

      PASS_FAIL               : OUT std_logic_vector(2 DOWNTO 0);

      -- chipscope debug ports
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
END ENTITY test_bench;



ARCHITECTURE rtl OF test_bench IS


   function AND_REDUCE(ARG: in STD_LOGIC_VECTOR) return std_logic is
      variable result: STD_LOGIC := '1';
   begin
      for i in ARG'range loop
         result := result AND ARG(i);
      end loop;
      return result;
   end function AND_REDUCE;


   COMPONENT cmp_rd_data
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_BYTE_WIDTH                 : integer := 9     -- the width of a "byte" on the memory device (x8 or x9)
   );
   PORT (
      CLK                     : IN  std_logic;   
      RESET                   : IN  std_logic;   
      READ_DATA_VALID         : IN  std_logic;   
      READ_DATA_FIFO_OUT      : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);   
      APP_COMPARE_DATA        : IN  std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);   
      DISPLAY                 : OUT std_logic_vector(6 DOWNTO 0);   
      enable_error_counter    : OUT std_logic;   
      PASS_FAIL               : OUT std_logic_vector(2 DOWNTO 0);   
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;

   COMPONENT backend_rom
   GENERIC (
      RL_DQ_WIDTH                    : integer := 36;   -- memory interface data width
      DEV_DQ_WIDTH                   : integer := 18;   -- data width of the memory device
      NUM_OF_DEVS                    : integer := 2;  -- number of memory devices
      NUM_OF_DKS                     : integer := 4;
      DEV_AD_WIDTH                   : integer := 20;   -- address width of the memory device
      DEV_BA_WIDTH                   : integer := 3;    -- bank address width of the memory device
      APP_AD_WIDTH                   : integer := 26;
      DEV_BYTE_WIDTH                 : integer := 9;    -- the width of a "byte" on the memory device (x8 or x9)
      RL_MRS_BURST_LENGTH            : std_logic_vector(1 DOWNTO 0) := "00"     -- "00": BL2;  "01": BL4;  "10": BL8
   );
   PORT (
      clk                     : IN  std_logic;
      rst                     : IN  std_logic;
      rst90                   : IN  std_logic;
      clk90                   : IN  std_logic;
      bkend_data_en           : IN  std_logic;
      bkend_wraddr_en         : IN  std_logic;
      bkend_rdaddr_en         : IN  std_logic;
      apRdfRdEn               : IN  std_logic;
      burst_length            : IN  std_logic_vector(1 DOWNTO 0);
      app_Waf_addr            : OUT std_logic_vector(25 DOWNTO 0);
      app_Waf_WrEn            : OUT std_logic;
      app_Wdf_data            : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      app_compare_data        : OUT std_logic_vector((2*RL_DQ_WIDTH)-1 DOWNTO 0);
      app_Wdf_WrEn            : OUT std_logic;
      app_Raf_addr            : OUT std_logic_vector(25 DOWNTO 0);
      app_Raf_WrEn            : OUT std_logic;
      cs_io                   : INOUT std_logic_vector(1023 DOWNTO 0)
   );
   END COMPONENT;

   COMPONENT bcd6
   PORT (
      clk        : IN  std_logic;                        -- clock
      rst        : IN  std_logic;                        -- reset
      clr        : IN  std_logic;                        -- clear to 0000000
      enablein   : IN  std_logic;                        -- enable in
      bcdvec     : OUT std_logic_vector(23 DOWNTO 0)     -- 6 decade output
   );
   END COMPONENT;


   -- Encoded states for Testbench FSM
   TYPE TestbenchStateValues is ( idle, start, write, read, mrs, restart_state );
   SIGNAL state :  TestbenchStateValues := idle;

   SIGNAL burst_count              : std_logic_vector(2 DOWNTO 0);
   SIGNAL temp_count               : std_logic_vector(2 DOWNTO 0);
   SIGNAL write_data_en            : std_logic;
   SIGNAL write_addr_en            : std_logic;
   SIGNAL read_addr_en             : std_logic;
   SIGNAL wr_count                 : std_logic_vector(3 DOWNTO 0);
   SIGNAL rd_count                 : std_logic_vector(3 DOWNTO 0);
--   SIGNAL read_flag                : std_logic;
   SIGNAL end_of_write             : std_logic;
   SIGNAL ready_for_rd             : std_logic;
   SIGNAL restart_state_en         : std_logic;
   SIGNAL comp_data_valid          : std_logic;
   SIGNAL data_check_state         : std_logic_vector(1 DOWNTO 0);
   SIGNAL Trc_cnt                  : std_logic_vector(5 DOWNTO 0);
   SIGNAL rlWdfEmpty_clk0          : std_logic;
   SIGNAL rlWdfEmpty_r             : std_logic;
   SIGNAL burst_length_r           : std_logic_vector(1 DOWNTO 0);
--   SIGNAL RdWrAddrEn               : std_logic_vector(1 DOWNTO 0);
--   SIGNAL RdWrAddr                 : std_logic_vector(23 DOWNTO 0);
   SIGNAL app_cmp_data             : std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL app_cmp_data_tmp         : std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL APP_WAF_ADDR             : std_logic_vector(25 DOWNTO 0);
   SIGNAL APP_RAF_ADDR             : std_logic_vector(25 DOWNTO 0);
   SIGNAL APP_WAF_WREN             : std_logic;
   SIGNAL APP_RAF_WREN             : std_logic;
   SIGNAL APP_WDF_DATA_tmp         : std_logic_vector(2*RL_DQ_WIDTH-1 DOWNTO 0);
   SIGNAL enable_error_counter     : std_logic;
   SIGNAL apRdfRdEn_r0             : std_logic;
   SIGNAL APP_WAF_WREN_r           : std_logic;
   SIGNAL APP_RAF_WREN_r           : std_logic;

   -- internal signals
   SIGNAL APVALID_condition        : std_logic;
   SIGNAL READ_DATA_VALID_sig      : std_logic;


BEGIN
   cs_io <= (OTHERS => 'Z');
   -- -----------------------------------------------------------------------------
   -- apAddr[25]   [ 1 bit]  - user refresh
   -- apAddr[24]   [ 1 bit]  - R/~W  (1'b0: write command;  1'b1: read command)
   -- apAddr[23:3] [21 bits] - memory address  A[21-1:0]
   -- apAddr[2:0]  [ 3 bits] - memory bank address  BA[2:0]
   -- -----------------------------------------------------------------------------
   APADDR <= APP_WAF_ADDR OR APP_RAF_ADDR;
   APVALID_condition   <= APP_RAF_WREN_r WHEN (burst_length_r = "01") ELSE APP_RAF_WREN;
   APVALID <= APP_WAF_WREN_r OR (APVALID_condition);
--   APVALID <= APP_WAF_WREN_r OR ( APP_RAF_WREN_r WHEN (burst_length_r = "01") ELSE APP_RAF_WREN );

   APCONFRD         <= '0';    -- currently does not read Configuration registers
   APWRITEDM        <= (others=>'0');
   issueCalibration <= '0';   -- not stimulated


   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF (RESET = '1') THEN
            APP_WAF_WREN_r <= '0';
            APP_RAF_WREN_r <= '0';
         ELSE
            IF (BURST_LENGTH = "01") THEN
               APP_WAF_WREN_r <= APP_WAF_WREN;
            ELSE
               APP_WAF_WREN_r <= write_addr_en;
            END IF;

            APP_RAF_WREN_r <= APP_RAF_WREN;
         END IF;
      END IF;
   END PROCESS;

   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         burst_length_r <= BURST_LENGTH;
      END IF;
   END PROCESS;

   -- Txfer rlWdfEmpty to clk0 (clkGlob) domain
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         rlWdfEmpty_r    <= rlWdfEmpty;
         rlWdfEmpty_clk0 <= rlWdfEmpty_r;
      END IF;
   END PROCESS;

   -- Read from read data fifo if it is not empty.
   -- register apRdfRdEn for better timing
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         apRdfRdEn    <= NOT rlRdfEmpty;
      END IF;
   END PROCESS;


   -- --------------------------------------------------------------------------
   -- State Machine               
   -- --------------------------------------------------------------------------
   --    1- After power-up, wait until Initialization of the RLDRAM is complete
   --       --> (Burst Length has been set at compile time to either 2 or 4 in rld_conf)
   --    2- Make 8 writes followed by 8 reads
   --    3- Make another 8 writes followed by 8 reads
   --    4- Drive DISPLAY: Pass (P), Error(E)
   --    5- Go to 2nd step after Trc_cnt elapses
   -- 
   -- --------------------------------------------------------------------------
   
   -- --------------------------------------------------------------------------
   -- State Machine
   --   Generates write address, data enable and read address enable
   -- --------------------------------------------------------------------------
   PROCESS (CLK)
   BEGIN
      IF RISING_EDGE( CLK ) THEN
         IF ( RESET = '1' ) THEN
            -- State Machine in IDLE state
            write_data_en     <= '0';
            write_addr_en     <= '0';
            read_addr_en      <= '0';
            wr_count          <= "0000";
            rd_count          <= "0000";
            end_of_write      <= '0';
            ready_for_rd      <= '0';
            restart_state_en  <= '0';
            APCONFWR          <= '0';
            issueMRS          <= '0';
            Trc_cnt           <= "000000";
            state             <= idle;
            APCONFA           <= "0000";
            -- Description of apconfWrD at address apConfA = 0 :
            --  bit[7]   = 0               MRS_DLL_RESET    ('0':DLL reset (default); '1':DLL enabled)
            --  bit[6]   = 0               ..static zero..  ('0')
            --  bit[5]   = 0               MRS_ADDR_MUX     ('0': non-muxed addr;  '1': muxed addr)
            --  bit[4:3] = "burst_length"  MRS_BURST_LENGTH ("00": BL2;  "01": BL4;  "10": BL8)
            --  bit[2:0] = 010             MRS_CONF         ("001": mode1;  "010": mode2;  "011": mode3)
            apConfWrD         <= "000" & burst_length & "010";
            burst_count       <= "000";
            temp_count        <= "000";
         ELSE
            CASE state IS
               -- --------------------------------------------------------------
               WHEN idle =>
               -- --------------------------------------------------------------
                  write_data_en    <= '0';
                  write_addr_en    <= '0';
                  read_addr_en     <= '0';
                  restart_state_en <= '0';
                  APCONFWR         <= '0';
                  issueMRS         <= '0';
                  Trc_cnt          <= "000000";

                  -- power-up - wait until init_done
                  IF ( Init_Done = '1' ) THEN
                     state <= start;
                  END IF;

               -- --------------------------------------------------------------
               WHEN start =>
               -- --------------------------------------------------------------
                  write_data_en <= '0';
                  write_addr_en <= '0';
                  read_addr_en  <= '0';

                  -- do 8 writes
                  IF ( wr_count < "1000" ) THEN
                     state <= write;
                     IF (burst_length_r = "01") THEN   -- BL4
                        burst_count <= "010";          -- count = 2
                        temp_count  <= "001";          -- count = 1
                     ELSE                       -- BL2 or BL8 (not BL4)
                        burst_count <= "001";   -- count = 1
                        temp_count  <= "000";   -- count = 0
                     END IF;                 

                  -- do 8 reads when 8 writes are complete
                  ELSE
                     IF ( (ready_for_rd = '1') AND (rd_count < "1000") ) THEN
                        state <= read;

                     -- wait after 8 writes for writes to complete (rlWdfEmpty flag) before reads can be started
                     ELSE
                        IF ( (wr_count = "1000" AND rlWdfEmpty_clk0 = '1') AND rd_count = "0000" ) THEN
                           ready_for_rd <= '1';
                           wr_count     <= STD_LOGIC_VECTOR( TO_UNSIGNED( TO_INTEGER(UNSIGNED(wr_count)) + 1 , wr_count'LENGTH ) );
                           state        <= start;

                        -- after 8 reads are done, do another 8 writes followed by 8 reads, then go to mrs state
                        ELSE
                           IF ( rd_count = "1000" ) THEN
                              IF (restart_state_en = '1') THEN
                                 state <= mrs;
                              ELSE
                                 IF ( rlafEmpty = '1' ) THEN
                                    wr_count         <= "0000";
                                    ready_for_rd     <= '0';
                                    rd_count         <= "0000";
                                    restart_state_en <= '1';
                                    state            <= start;
                                 END IF;
                              END IF;
                           END IF;
                        END IF;
                     END IF;
                  END IF;

               -- --------------------------------------------------------------
               WHEN write =>
               -- --------------------------------------------------------------

                  --IF ( WDF_ALMOST_FULL = '0' ) THEN
                     IF ( burst_count /= "000" ) THEN
                        read_addr_en  <= '0';
                        write_data_en <= '1';
                        burst_count   <= STD_LOGIC_VECTOR( TO_UNSIGNED( TO_INTEGER(UNSIGNED(burst_count)) - 1 , burst_count'LENGTH ) );
                        wr_count      <= wr_count;
                        state         <= write;
                     ELSE
                        IF ( wr_count < "0111" ) THEN   -- less than 7
                           read_addr_en  <= '0';
                           write_data_en <= '1';
                           write_addr_en <= '0';
                           burst_count   <= temp_count;   -- load temp_count into burst_count
                           wr_count      <= STD_LOGIC_VECTOR( TO_UNSIGNED( TO_INTEGER(UNSIGNED(wr_count)) + 1 , wr_count'LENGTH ) );
                           state         <= write;
                        ELSE
                           read_addr_en  <= '0';
                           write_data_en <= '0';
                           write_addr_en <= '0';
                           wr_count      <= STD_LOGIC_VECTOR( TO_UNSIGNED( TO_INTEGER(UNSIGNED(wr_count)) + 1 , wr_count'LENGTH ) );
                           state         <= start;
                        END IF;
                     END IF;

                     IF ( (burst_length_r = "00" AND burst_count = "000") OR       -- case for BL2
                          (burst_length_r = "01" AND burst_count = "001") ) THEN   -- case for BL4
                        write_addr_en <= '1';
                     ELSE
                        write_addr_en <= '0';
                     END IF;

                  --ELSE
                  --   read_addr_en  <= '0';
                  --   write_data_en <= '0';
                  --   write_addr_en <= '0';
                  --   burst_count   <= burst_count;
                  --   wr_count      <= wr_count;
                  --   state         <= write;    --  Control will stay in the same state until wdfifo is not full
                  --END IF;

               -- --------------------------------------------------------------
               WHEN read =>
               -- --------------------------------------------------------------
                  
                  write_data_en <= '0';
                  write_addr_en <= '0';

                  IF ( RAF_ALMOST_FULL = '0' ) THEN
                     read_addr_en <= '1';
                     rd_count     <= STD_LOGIC_VECTOR( TO_UNSIGNED( TO_INTEGER(UNSIGNED(rd_count)) + 1 , rd_count'LENGTH ) );
                     IF ( rd_count < "0111" ) THEN   -- less than 7
                        state <= read;
                     ELSE
                        state <= start;
                     END IF;
                  ELSE
                     read_addr_en <= '0';
                     rd_count     <= rd_count;
                     state        <= start;
                  END IF;

               -- --------------------------------------------------------------
               WHEN mrs =>
               -- --------------------------------------------------------------
                  -- This functionality is taken out, but Mode Register bits can be programmed if desired as shown below
                  -- IF ( restart_edge = '1' ) THEN   -- wait for restart, then program burst length
                  --    APCONFA   <= "0000";
                  --    APCONFWR  <= '1';
                  --    APCONFWRD <= "0000" & BURST_LENGTH & "010";   -- except BURST LENGTH make the other bits equal to their default values
                  --    issueMRS  <= '1';
                  --    Trc_cnt   <= "000000";
                  --    state     <= restart_state;
                  -- END IF;
                  --

                  Trc_cnt <= "000000";
                  IF ( rlafEmpty = '1' ) THEN
                     state <= restart_state;
                  END IF;

               -- --------------------------------------------------------------
               WHEN restart_state =>
               -- --------------------------------------------------------------
                  -- go to start after Trc_cnt elapses
                  wr_count         <= "0000";
                  ready_for_rd     <= '0';
                  rd_count         <= "0000";
                  restart_state_en <= '0';
                  APCONFWR         <= '0';
                  issueMRS         <= '0';
                  IF ( Trc_cnt = "111111" ) THEN
                     state   <= start;
                  ELSE
                     Trc_cnt <= STD_LOGIC_VECTOR( TO_UNSIGNED( TO_INTEGER(UNSIGNED(Trc_cnt)) + 1 , Trc_cnt'LENGTH ) );
                     state   <= restart_state;
                  END IF;

               -- --------------------------------------------------------------
               WHEN OTHERS  =>
               -- --------------------------------------------------------------
                  write_data_en    <= '0';
                  write_addr_en    <= '0';
                  read_addr_en     <= '0';
                  wr_count         <= "0000";
                  rd_count         <= "0000";
                  end_of_write     <= '0';
                  ready_for_rd     <= '0';
                  restart_state_en <= '0';
                  APCONFWR         <= '0';
                  issueMRS         <= '0';
                  Trc_cnt          <= "000000";
                  state            <= idle;
            END CASE;
         END IF;
      END IF;
   END PROCESS;


   -- -----------------------------------------------------------------------------
   -- Comparison: read data vs. expected data
   -- -----------------------------------------------------------------------------
   READ_DATA_VALID_sig <= (READ_DATA_VALID AND Init_Done);

   cmp_rd_data0 : cmp_rd_data
   GENERIC MAP (
      RL_DQ_WIDTH     => RL_DQ_WIDTH,
      DEV_BYTE_WIDTH  => DEV_BYTE_WIDTH   -- the width of a "byte" on the memory device (x8 or x9)
   )
   PORT MAP (
      CLK                  => CLK,   -- data read from memory is in system clock domain
      RESET                => RESET,
      READ_DATA_VALID      => READ_DATA_VALID_sig,
      READ_DATA_FIFO_OUT   => READ_DATA_FIFO_OUT(2*RL_DQ_WIDTH-1 DOWNTO 0),
      APP_COMPARE_DATA     => app_cmp_data(2*RL_DQ_WIDTH-1 DOWNTO 0),
      DISPLAY              => DISPLAY,
      enable_error_counter => enable_error_counter,
      PASS_FAIL            => PASS_FAIL,
      cs_io                => cs_io
   );


   -- -----------------------------------------------------------------------------
   -- Block of RAM, holds test vectors
   -- -----------------------------------------------------------------------------
   backend_rom0 : backend_rom
   GENERIC MAP (
      RL_DQ_WIDTH          => RL_DQ_WIDTH,
      DEV_DQ_WIDTH         => DEV_DQ_WIDTH,
      NUM_OF_DEVS          => NUM_OF_DEVS,
      NUM_OF_DKS           => NUM_OF_DKS,
      DEV_AD_WIDTH         => DEV_AD_WIDTH,
      DEV_BA_WIDTH         => DEV_BA_WIDTH,
      APP_AD_WIDTH         => APP_AD_WIDTH,
      DEV_BYTE_WIDTH       => DEV_BYTE_WIDTH,
      RL_MRS_BURST_LENGTH  => RL_MRS_BURST_LENGTH
   )
   PORT MAP (
      clk               => CLK,
      rst               => RESET,
      rst90             => RESET_90,
      clk90             => clk90,
      bkend_data_en     => write_data_en,
      bkend_wraddr_en   => write_addr_en,
      bkend_rdaddr_en   => read_addr_en,
      apRdfRdEn         => READ_DATA_VALID,
      burst_length      => burst_length_r,

      app_Waf_addr      => APP_WAF_ADDR,
      app_Waf_WrEn      => APP_WAF_WREN,
      app_Wdf_data      => APP_WDF_DATA_tmp,
      app_compare_data  => app_cmp_data_tmp,
      app_Wdf_WrEn      => APP_WDF_WREN,
      app_Raf_addr      => APP_RAF_ADDR,
      app_Raf_WrEn      => APP_RAF_WREN,

      cs_io             => cs_io
   );

   APP_WDF_DATA  <= APP_WDF_DATA_tmp;
   app_cmp_data  <= app_cmp_data_tmp;


   -- -----------------------------------------------------------------------------
   -- BCD error counter, 6 decades
   -- -----------------------------------------------------------------------------
   RLD2_bcd_error_counter : bcd6
   PORT MAP (
      clk       => CLK,                   -- clock
      rst       => RESET,                 -- reset
      clr       => '0',                   -- clear to 0000000
      enablein  => enable_error_counter,  -- enable in
      bcdvec    => error_count_BCD        -- 6 decade output
   );


END ARCHITECTURE rtl;
